/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/IOlayer.c,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    Function implementation for the IO layer of the 88E6208 RD board and 
 *	  the 88E6218 Development board.  
 *
 * Public Procedures:
 * 	  strToInt - Convert a string to a decimal number. 
 *	  GT_StrLen - Gets a STRING (char*) and count its charecters.
 *
 * Private Procedures:
 *    None
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

#include "IOlayer.h"


#define __  255
unsigned char Translation_Table[256] = {
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, __, __, __, __, __, __,
  __, 10, 11, 12, 13, 14, 15, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, 10, 11, 12, 13, 14, 15, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
  __, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
};
#undef __


/********************************************************************
* strToInt - Convert a string to a decimal number.
*          - This function gets a STRING (char*) it convert it to a decimal
*            format number.
*          - If the number of charecters given greater then 8 The number is
*            out of unsigned int format.
*
*
* INPUTS:   char* str - The String to be converted.
* OUPUT:    N/A
* RETURNS: - On success - the converted unsigned int, on failure it returns 
*            0xffffffff.
*********************************************************************/

unsigned int strToInt(char* str)
{
    unsigned int num =0,i=0, strLength;
    unsigned char c;
    strLength = GT_StrLen(str); 
    if(strLength > 8 ) return 0xffffffff;
    for (i = 0; i < strLength; i++) 
    {
        num = num * 16;
        /* Bad hex number */
        if((c = asciiToBinary(str[i])) == 255) return 0xffffffff;
        num = num + c;
    }
    return num;
}

/********************************************************************
* GT_StrLen - Gets a STRING (char*) and count its charecters.
*
*
* INPUT:    char* s - A string which its charecters is to be counted.
* OUTPUT:   N/A
* RETURN:   unsigned int i - The number of charecters in the string.
*********************************************************************/

unsigned int GT_StrLen(char* s)
{
    unsigned int i = (unsigned int)s;
	
    while (*s != '\0') // while not the end of the string 
	{ 
		s++; 
	}
	return ((unsigned int)s - i);
}


/********************************************************************
* asciiToBinary - Convert a character to binary.
*
*
* INPUTS:   char c - The charecter we wish to convert.
* OUTPUT:   N/A
* RETURNS: The binary value of the character.
*********************************************************************/

unsigned char asciiToBinary(char c)
{
    return Translation_Table[(unsigned int)c];
}

