/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header:
 *
 * Purpose:
 *    Function implementation for the bootMonitor.
 *
 * Public Procedures:
 *    bootMonitor                    Main entry for the shell monitor.
 *
 * Private Procedures:
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "MvCore.h"
#include "board.h"
#include "boot_monitor.h"
#include "xmodem.h"
#include "IOlayer.h"
#include "boot.h"
//#include "uart.h"
//#include "gtStdio.h" /* Rabeeh */
#include "gtTftpE.h"
#include <setjmp.h>
jmp_buf out_of_here;

#define RESET_FACTORY_DEFAULT 1
/******************************************************************************
 *
 * Name: bootMonitor
 *
 * Description:
 *   This function creates shell window. 
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/

static void print_logo()
{
//	armond_printf("%s\n","Test");
//	armond_printf("%d\n",4);
//	armond_printf("%x\n",4);

//	armond_printf(GT_ARMBM_LOGO);
	
//    armond_printf("ARMOND - ARM Monitor Deluxe for MV88E6218\n"
//           "For help on ARMOND Commands, press 'h'.  \n"
//           "Enjoy!...                               \n");
}


union {
    void (*branch) (void);
    int hackfest;
} u;

void run_image(void) {
#if 0
  unsigned int j;
  /* Copy 2KByte which has the vectors to the tightly coupled memory */
//  armond_printf ("Copying linux vectors to address 0x0\n");
  for (j = 0 ; j < 0x400 ; j += 4) {
    * (unsigned int *) j = * (unsigned int *)(j + 0xd0000000);
  }
#endif
  //  memcpy ((void *)0x00000000, (void *)0xd0000000, 0x00000400);
  /* Branch to reset vector at address 0x0 */
  //  armond_printf ("Branching to address 0x0\n");
  u.hackfest = 0x0;
  /*  for (j = 0 ; j < 0x100 ; j += 4) {
    armond_printf ("%08x : %08x\n",j, * (unsigned int *)j);
    }*/
  * (unsigned int *) 0x0 = 0xea000007; /* Hack alert */
  u.branch();
}
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))
#define FLASHBASE 0xfff00000
void amdPreamble()
{
    int flashbase = FLASHBASE;
#ifdef SST_FLASH
    _REG16(flashbase | 0xAAAA) = 0x00AA;
    _REG16(flashbase | 0x5554) = 0x0055;
    _REG16(flashbase | 0xAAAA) = 0x0080;
    _REG16(flashbase | 0xAAAA) = 0x00AA;
    _REG16(flashbase | 0x5554) = 0x0055;
#else
   _REG16(flashbase | 0xAAA) = 0x00AA;
   _REG16(flashbase | 0x555) = 0x0055;
   _REG16(flashbase | 0xAAA) = 0x0080;
   _REG16(flashbase | 0xAAA) = 0x00AA;
   _REG16(flashbase | 0x555) = 0x0055;
#endif
}
void amdEraseChip(int start_sec, int num)
{
    unsigned addr;
    int flashbase = FLASHBASE;
    int i, j;
    int split_sector = 0; // HACK!!! FOR LV800BB ONLY!!!
    int split_sector_data[4];
#ifndef PARTIAL
    amdPreamble();
#ifdef SST_FLASH
    _REG16(flashbase | 0xAAAA) = 0x0010;
#else
   _REG16(flashbase | 0xAAA) = 0x0010;
#endif
   while(!(_REG(0x80006000)&0x1));
#else
   split_sector_data[0] = 0;
   split_sector_data[1] = 0x4000;
   split_sector_data[2] = 0x6000;
   split_sector_data[3] = 0x8000;
   for(i=start_sec;i < start_sec + num;i++) {
       if(i == split_sector) {
	   unsigned start = (i << (16));
	   for(j=0;j < sizeof(split_sector_data)/sizeof(int);j++) {
	       addr= flashbase + start + split_sector_data[j];
	       amdPreamble();
#ifdef SST_FLASH
	       _REG16(addr) = 0x0050;
#else
	       _REG16(addr) = 0x0030;
#endif
	       while(!(_REG16(addr) == 0xffff));
	   }
       } else {
       addr= flashbase + (i << (16));
       amdPreamble();
#ifdef SST_FLASH
       _REG16(addr) = 0x0050;
#else
       _REG16(addr) = 0x0030;
#endif
	   while(!(_REG16(addr) == 0xffff));
       }
   }
#endif
}
void 
amdFlashWriteWord( int offset, unsigned short dat)
{
    int done;
    int flashbase = FLASHBASE;
    unsigned addr = flashbase+offset;
#ifdef SST_FLASH
    _REG16(flashbase | 0xAAAA) = 0x00AA;  // word address 555
    _REG16(flashbase | 0x5554) = 0x0055;  // word address 2AA
    _REG16(flashbase | 0xAAAA) = 0x00A0;  // word address 555
#else
   _REG16(flashbase | 0xAAA) = 0x00AA;  // word address 555
   _REG16(flashbase | 0x555) = 0x0055;  // word address 2AA
   _REG16(flashbase | 0xAAA) = 0x00A0;  // word address 555
#endif
   _REG16(addr) = dat;  // word address 0xFC000000
   do {
       done = 0;
       if(_REG16(addr) == dat) {
	   done++;
       }
       if(_REG16(addr) == dat) {
	   done++;
       }
   } while(done != 2);
   _REG16(flashbase) = 0xf0;

}
void
RestoreFactoryDefault()
{
    int i;
    unsigned short val;
    unsigned short *addr;
    unsigned len;
    /* 2k of factory setting ought to be in SDRAM now. */
    // AMD FLASH test
    // Erase sector
    amdEraseChip(14, 1);

    len = 0x800;
    addr = (unsigned short *)0xd0000000;
    
    for(i=0;i < len;i+= 2) {
	val = *addr++;
	amdFlashWriteWord(0xe0000+i, val);
    }
}
void
program_image()
{
    int i;
    unsigned short val;
    unsigned short *addr;
    unsigned len;
    /* 1MB of code ought to be in SDRAM now. */
    // AMD FLASH test
    // Erase sector
    amdEraseChip(0, 13);

#ifndef PARTIAL
    len = 1024*1024;
#else
    len = (1024*1024) - 0x20000;
#endif

    addr = (unsigned short *)0xd0000000;
  
    for(i=0;i < len;i+= 2) {
	val = *addr++;
	amdFlashWriteWord(0+i, val);
    }
}

int flash_boot = 0;
void
write_gpio(unsigned x)
{
    *(volatile unsigned *)0x8000d00c = x;
}
unsigned read_gpio(void)
{
	return (*(volatile unsigned *)(0x8000d010));
}

unsigned read_pmu_ctrl(void)
{
	return (*(volatile unsigned *)(0x8000d800));
}
void write_pmu_ctrl(unsigned x)
{
	*(volatile unsigned *)(0x8000d800) = x;
}


int 
main(void)
{
    int x;
    if(*(int *)0xc000b000) {
	flash_boot = 1;
    }
#if 0
    if(*(int *)0xc000b000) {
	__asm {
	    swi  0x0
	}
    }
#endif
//    setup_uart(); // When booting from flash this needs to be done.
    print_logo();
//    armond_printf("Initializing SDRAM\n");
    //init_sdram();
//    armond_printf("Downloading image through ethernet\n");

	disable_data_cache();
	if ( x = (read_pmu_ctrl() & 0x0008) )
	{
		write_pmu_ctrl( x & 0xfff7); 
		goto directly_out;
	}
#ifdef RESET_FACTORY_DEFAULT 
	if ((read_gpio() & 0x0020)==0 )
    {  
#if 1
        void gtEtherStartTx(void);
        int i, j, k;
        memcpy((void *)0xd0000000, (void *)0xfffe0000, 0x400);
        memcpy((void *)0xd0000400, (void *)0xfffe0000, 0x400);
	    memcpy((void *)0x1f000, (void *)amdPreamble, (int)gtEtherStartTx - (int)amdPreamble);
	    u.hackfest = 0x1f000 + ((int)RestoreFactoryDefault - (int)amdPreamble);
	    u.branch();
#else
        RestoreFactoryDefault();
#endif
		goto directly_out;
    }
#endif
    if((x=setjmp(out_of_here)) != 0) {
	extern void unpack_and_go(void);
directly_out:
	write_gpio(~(1 << 5));
	enable_cache();
	invalidate_cache();
	unpack_and_go();
    } else {
	void gtEtherStartTx(void);
	write_gpio(~((1 << 5) | ( 1 << 4)));
//        armond_printf ("main: before ltftp ()\n");
	ltftp();
//        armond_printf ("main: after ltftp ()\n");
	write_gpio(~(1 << 4));
	enable_cache();
	invalidate_cache();
	disable_data_cache();
#if 1
	memcpy((void *)0x1f000, (void *)amdPreamble, (int)gtEtherStartTx - (int)amdPreamble);
	u.hackfest = 0x1f000 + ((int)program_image - (int)amdPreamble);
	u.branch();
#else
	program_image();
#endif
    }
	enable_data_cache();
	invalidate_data_cache();
    write_gpio(~0);
	write_gpio(~(1 << 5));
	unpack_and_go();
    for(;;);
    return 0;
}


