/******************** (c) Marvell Semiconductor, Inc., 2001 ******************* 
* 
* $Header: G:/W81/802.11_MAC-DRIVER_SW/AP/Src/bridge.c,v 1.10 2003/02/23 22:21:52 jshen Exp $ 
* 
* Purpose: 
*    This file contains the implementations of the function prototypes given 
*    in the associated header file for the Distribution Service Task. 
* 
* Public Procedures: 
*    ds_Init       Initialzies Distribution Service Task and 
*                           related components 
*    ds_Start      Starts running the Distribution Service Task 
*    ds_SendEvent  Mechanism used to trigger an event indicating that 
*                           messages are present in the queue or receive
*                           buffers 
* 
* Private Procedures: 
*    MacMgmtTask            Distribution Service Task  
* 
* Notes: 
*    None. 
* 
*****************************************************************************/ 

//============================================================================= 
//                               INCLUDE FILES 
//============================================================================= 
#include "basic_types.h" 
#include "IEEE_types.h" 
#include "buildModes.h"
#include "os_if.h" 
#include "w8100.h"
#include "rx80211.h"
#include "ds.h" 
#include "sysinfo.h" 
#include "StaDb.h"
#include "tx80211.h"
#include "txsched.h"
#include "macMgmtMlme.h"
#include "etherlow.h"
#include "ether.h"
#include "api.h"
#include "mib.h"

//#define __ECOS
#include <pkgconf/net.h>
#include <pkgconf/net_w81_eth_drivers.h>

#include <cyg/infra/cyg_type.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/infra/diag.h>
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_cache.h>

#include <netdev.h>
#include <eth_drv.h>
//============================================================================= 
//                                DEFINITIONS 
//============================================================================= 

#define MEM_COPY 0
#define PROCEED_AFTER_DMA_DONE  1

#define DS_EVENTS ds_802_11_DATA_RCVD  | \
                  ds_802_3_DATA_RCVD   | \
                  ds_MGMT_NOTIFICATION_RCVD | \
                  ds_LLC_DATA_RCVD | \
				  ds_802_3_TX_DONE | \
				  ds_802_11_TX_DONE | \
                  ds_POLLING_TIMEOUT_RCVD

#define TIMER1_LEN    (W81_TIMER_BASE + 0x0)
#define TIMER2_LEN    (W81_TIMER_BASE + 0x4)
#define TIMER3_LEN    (W81_TIMER_BASE + 0x8)
#define TIMER4_LEN    (W81_TIMER_BASE + 0xc)
#define TIMER_CONTROL (W81_TIMER_BASE + 0x10)
#define TIMER_IMR     (W81_TIMER_BASE + 0x28)
#define TIMER_SRC     (W81_TIMER_BASE + 0x24)

//pool_ID_t rxWLDataPoolId;
//pool_ID_t rxEtherDataPoolId;
extern pool_ID_t llcDataPoolId;
#if 0
pool_ID_t etherWLCopyPoolId;
sdpool_CREATE(rxEtherDataPool, "rxEtherDataPool", sizeof(rx8023_DataFrame_t),80)
sdpool_CREATE(rxWLDataPool, "rxWLDataPool", sizeof(rx80211_DataFrame_t),16)
sdpool_CREATE(etherWLCopyPool, "etherWLCopyPool", sizeof(tx_pktbuf_t),64)
sdpool_CREATE(llcDataPool, "llcDataPool", sizeof(dsQ_LlcData_t),16)
#endif
extern Statistics Stats;

extern void w8100_eth_RxEvent(int length);
extern struct eth_drv_sc *sc1;


//============================================================================= 
//                              TYPE DEFINITIONS 
//============================================================================= 
// 


//============================================================================= 
//                         IMPORTED PUBLIC VARIABLES 
//============================================================================= 
extern IEEEtypes_MacAddr_t macBssId; 
extern IEEEtypes_MacAddr_t macStaAddr; 

unsigned char * rcvdData;

unsigned char  moreWlPkts;
extern Boolean GetWirelessData(rx80211_Frame_t **Data_p, uint32 *Len_p);


extern Status_e FreeRxDataBuf(uint32 RxDataPtr);


extern apio_ErrCode_t apio_wlanwrite( apio_bufdescr_t *pBufDescr);

//============================================================================ 
//                          MODULE LEVEL VARIABLES 
//============================================================================= 


uint32 WlRxDone;
uint32 EtherRxDone;


//============================================================================= 
//                   PRIVATE PROCEDURES (ANSI Prototypes) 
//============================================================================= 
static void    BridgeTask( os_Addrword_t Data ); 
void InitDefaultHeader(IEEEtypes_GenHdr_t * Hdr_p);
void MemDma(uint32 x, uint32 y, uint32 z);
void DoDmaDone(void);
void InitEtherWLCopyPool();
void SetupPollingTimer();
uint32 ProcessWirelessData(void);
uint32 ProcessEtherData(void);
uint32 ProcessLlcData(void);

apio_handle_t etherHandle,wlanHandle;



//============================================================================= 
//                         CODED PUBLIC PROCEDURES 
//============================================================================= 

/****************************************************************************** 
 * 
 * Name: bridge_Init 
 * 
 * Description: 
 *   This routine is called to initialize the the Distribution Service Task 
 *   and related components. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *   Create dsMgmtQ  
 *   If the queue was successfully created Then 
 *      Create the Distribution Service Task by calling os_TaskCreate() 
 *      If creating the Distribution Service Task succeeded Then  
 *         Return SUCCESS 
 *      End If 
 *   End If 
 * 
 *   Return FAIL 
 * END PDL 
 * 
 *****************************************************************************/ 
extern Status_e bridge_Init( void ) 
{ 
  Status_e  status; 
  apio_ErrCode_t errCode;
  if (os_QueueCreate(sysinfo_DS_LLC_DATA_Q) != os_SUCCESS) {
    return(FAIL);
  }


  errCode = apio_open("eth0",&etherHandle,NULL);
  errCode = apio_open("wl0",&wlanHandle,NULL);
  { 

    if ((status = os_TaskCreate(sysinfo_DS_TASK, 
                                BridgeTask, 
                                (os_Addrword_t)0, 
                                sysinfo_DS_TASK_NAME, 
                                sysinfo_DS_TASK_STACK_SIZE, 
                                sysinfo_DS_TASK_PRI)) == SUCCESS) {

      return(SUCCESS); 
    }
  } 

  return(FAIL); 
} 


/****************************************************************************** 
 * 
 * Name: ds_Start 
 * 
 * Description: 
 *   This routine is called to start the Distribution Service Task running. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   ds_Init(). 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *    Start the Distribution Service Task by calling os_TaskResume() 
 * END PDL 
 * 
 *****************************************************************************/ 
extern void bridge_Start( void ) 
{ 
  os_TaskResume(sysinfo_DS_TASK); 
} 


//============================================================================= 
//                         CODED PRIVATE PROCEDURES 
//============================================================================= 

/****************************************************************************** 
 * 
 * Name: BridgeTask 
 * 
 * Description: 
 *   This routine is the task that runs continuously, waking to process 
 *   time outs and messages received from the different message queues. 
 *   It commences when the ds_Start() routine is invoked. 
 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): Data - Data passed to the task; not actually used - just 
 *                      included to conform to the structure the OS 
 *                      expects when creating a task 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *   While forever 
 *      Wait for an event to occur by calling os_EventWait() 
 *      If an error occurred waiting for an event Then 
 *         Do some error handling 
 *      Else 
 *      End If 
 *   End While 
 * END PDL 
 * 
 *****************************************************************************/ 
#define GPIO_WLAN 4
#define GPIO_LAN 5
static void BridgeTask( os_Addrword_t Data ) 
{ 
  os_IndicatedEvents_t         EventsTriggered; 
  uint32 noPkts;


#ifdef POLLING   
  SetupPollingTimer();
#endif
  while (1) {
    EventsTriggered =  apio_select();

#ifdef WATCHDOG_ENABLE
    //WatchdogReset();   
#endif

    /*------------------------------*/ 
    /* Check for an error condition */ 
    /*------------------------------*/ 
    if (EventsTriggered == 0) {
      // Do some error handling 
    }
    else {
#ifdef POLLING
      WlRxDone = FALSE;
      EtherRxDone = FALSE;
      if (EventsTriggered & ds_POLLING_TIMEOUT_RCVD) {
        while (!(WlRxDone && EtherRxDone)) {
#endif
#ifndef POLLING
          if (EventsTriggered & APIO_POLL_ETH_DATA_RDY)
#endif
          {
#ifndef AP21N
      	     apctlSwLedEnable(GPIO_LAN);
#endif
            noPkts = ProcessEtherData();
            
#ifdef AP21N
            ADD_ETHERDATA_CNT(noPkts);
#endif
            if (noPkts < NO_OF_PKTS) {
              EtherRxDone = TRUE;
            }

            else {
#ifndef POLLING
#ifndef FREE_IN_TXTASK
              ds_SendEvent(APIO_POLL_ETH_DATA_RDY);
#endif
#endif

            }

          }
	   else{
#ifndef AP21N
	   	apctlSwLedDisable(GPIO_LAN);
#endif
	   }
#ifndef FREE_IN_TXTASK		   
          ProcessTxDone();
#endif

          /* INTR is enabled within Process Tx Done */

          /*-------------------------------------------------*/ 
          /* Check for data on the 802.11 Data */ 
          /*-------------------------------------------------*/
#ifndef POLLING
          if (EventsTriggered & APIO_POLL_RADIO_DATA_RDY)
#endif
          {

#ifndef AP21N
	     apctlSwLedEnable(GPIO_WLAN);
#endif
	    
            noPkts = ProcessWirelessData();
#ifdef AP21N
            ADD_WLANDATA_CNT(noPkts);
#endif
            //clear  interrupt if all pks processed
          	  if ( noPkts < NO_OF_PKTS) 
          	  {

              WlRxDone = TRUE;
            }
            	else 
            	{
#ifndef POLLING
#ifndef FREE_IN_TXTASK
              ds_SendEvent(APIO_POLL_RADIO_DATA_RDY);
#endif
#endif
            }


          }
	   else{
#ifndef AP21N
	   	apctlSwLedDisable(GPIO_WLAN);
#endif
	   	}
	   	
	   
	    	   
	    	

#ifndef FREE_IN_TXTASK
          w8100_eth_free_tx_queue (&eth_dev,0);

#endif
#ifdef POLLING
        } /* end of while WlRxDone , EtherRxDone */

        W81_READ_BYTE(TIMER_SRC, TimerSrc);
        W81_WRITE_BYTE(TIMER_SRC, (TimerSrc |0x4));
      } /* end of if POLLING_TIMEOUT_RCVD */
#endif


      if (EventsTriggered & ds_LLC_DATA_RCVD) {
        ProcessLlcData();

      }

    } 
  } 
} 

/****************************************************************************** 
 * 
 * Name: ProcessWirelessData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 * 
 *****************************************************************************/ 
uint32 ProcessWirelessData(void)
{

  ether_hdr_t *EtherHdr_p, *EtherHdr2_p;

  uint32 Len;

  uint32 noPkts;


  apio_bufdescr_t BufDescr;

  noPkts = 0;
  while ( (noPkts++ < NO_OF_PKTS) && apio_read(wlanHandle,&BufDescr)) {

    EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
    Len = BufDescr.frameLen;

    if (IS_GROUP((uint8 *)&EtherHdr_p->da )) {

      /* First, send the packet to AP protocol layer */
      rcvdData = (uint8 *)EtherHdr_p;
      w8100_eth_RxEvent(Len );



#if 0
      if ((EtherHdr2_p = (ether_hdr_t *)apio_GetWlanTxBuf())  != NULL) {

        memcpy(EtherHdr2_p,EtherHdr_p,Len);
        /*send over ethernet */
        /* send a copy over ethernet */



        /* Buffer the data to ether tx Desc */

        /* First, send the packet to AP protocol layer */
        rcvdData = (uint8 *)EtherHdr2_p;
        w8100_eth_RxEvent(Len );
        /* send the same to ethernet port */

        BufDescr.frameLen = Len;
        BufDescr.framePtr = (uint8*)EtherHdr2_p;
        apio_write(etherHandle,&BufDescr);

      }

#endif
      /* send the orig copy over wireless */
      BufDescr.frameLen = Len;
      BufDescr.framePtr = (uint8*)EtherHdr_p;
      apio_write(etherHandle,&BufDescr);

    }

    else if (!memcmp(&EtherHdr_p->da ,&macStaAddr,
                     sizeof(IEEEtypes_MacAddr_t))) {

      rcvdData = (uint8 *)EtherHdr_p;
      w8100_eth_RxEvent(Len);



      /* free the packet */

      apio_free(wlanHandle,&BufDescr);

    }
    else {


      /*     send the packet over the ethernet */



      BufDescr.frameLen = Len;
      BufDescr.framePtr = (uint8*)EtherHdr_p;
      apio_write(etherHandle,&BufDescr);

    }

  } /* End While GetWireless Data */
  return(noPkts);
}
/****************************************************************************** 
 * 
 * Name: ProcessEtherData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 * 
 *****************************************************************************/ 
uint32 ProcessEtherData(void)
{
  IEEEtypes_8023_Frame_t  *DataEtherFrame_p; 
  Boolean BcastFlag;
  uint32 Len;
  uint32 noPkts;


  apio_bufdescr_t wBufDescr,rBufDescr;

  /* Retrieve data from the receive buffers one by one
   * and process them. The Frame has ethernet header and the
   * destination address is checked to see how the frame has
   * to be routed */
  noPkts = 0;
  /* retrieve the frame */
  while ((noPkts++ < NO_OF_PKTS) &&  apio_read(etherHandle,&rBufDescr)) {

#ifdef INCL_STATS
    Stats.EthRecvCnt++;
#endif
    DataEtherFrame_p = (IEEEtypes_8023_Frame_t*)rBufDescr.framePtr;
    Len = rBufDescr.frameLen;
    BcastFlag = FALSE;

    if (IS_GROUP((uint8 *)&DataEtherFrame_p->Hdr.da) )
      BcastFlag = TRUE;

    if ((!memcmp(DataEtherFrame_p->Hdr.da,macStaAddr,
                 sizeof(IEEEtypes_MacAddr_t))) || BcastFlag) {
      /* send the pkt to protocol stack */
      /* rcvdData is used by the w8100_eth_x functions */
      rcvdData = (uint8 *)DataEtherFrame_p;
      w8100_eth_RxEvent(Len);
      if (!BcastFlag) {
        wBufDescr.frameLen = Len;
        wBufDescr.framePtr = (uint8 *)DataEtherFrame_p;
        apio_free(etherHandle,&wBufDescr);

        continue;
      }
    }


    wBufDescr.frameLen = Len;
    wBufDescr.framePtr = (uint8*)DataEtherFrame_p;
    apio_write(wlanHandle,&wBufDescr);


  }
  return(noPkts);
}
/****************************************************************************** 
 * 
 * Name: ProcessLlcData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 * 
 *****************************************************************************/ 
uint32 ProcessLlcData(void)
{

  ether_hdr_t *EtherHdr_p, *EtherHdr2_p;

  Boolean BcastFlag;    
  IEEEtypes_MacAddr_t da,sa; 
  extStaDb_StaInfo_t StaInfo;
  uint32 Len;

  uint32 noPkts;
  ether_hdr_t EthHdr;

  dsQ_LlcData_t *LlcData_p;
  extStaDb_Status_e SearchResult;
  apio_bufdescr_t BufDescr;

  while ((   LlcData_p = os_QueueReadNoBlock(sysinfo_DS_LLC_DATA_Q))) {
    noPkts++;
    EtherHdr_p = (ether_hdr_t *)LlcData_p->data;
    Len = LlcData_p->totalLen;
    MACADDR_CPY(&da,&EtherHdr_p->da);
    MACADDR_CPY(&sa,&EtherHdr_p->sa);
    EthHdr.type = EtherHdr_p->type;
    BcastFlag = FALSE;

    if (IS_GROUP((uint8 *)&da))
      BcastFlag = TRUE;
    if (((SearchResult = extStaDb_GetStaInfo(&da,
                                             &StaInfo)) == STATE_SUCCESS && 
         StaInfo.State == ASSOCIATED) || BcastFlag) {

      if ((EtherHdr2_p =(ether_hdr_t *) apio_GetWlanTxBuf())
          == NULL) {
        /* free the llc msg */
        eth_drv_tx_done(sc1,LlcData_p->Key,0);
        pool_FreeBuf((uint8 *)LlcData_p);
        continue;
      }


      memcpy((uint8 *)EtherHdr2_p,(uint8*)EtherHdr_p,Len);
      BufDescr.frameLen = Len;
      BufDescr.framePtr = (uint8*)EtherHdr2_p;
      apio_write(wlanHandle,&BufDescr);


    }
    if (BcastFlag || (SearchResult != STATE_SUCCESS )) {
      /* to ethernet port */

      if ((EtherHdr_p =(ether_hdr_t *) apio_GetEtherTxBuf())
          == NULL) {
        /*call the tx done and free the llc msg */
        eth_drv_tx_done(sc1,LlcData_p->Key,0);
        pool_FreeBuf((uint8 *)LlcData_p);
        continue;
      }
      memcpy(EtherHdr_p,  LlcData_p->data,   LlcData_p->totalLen);


      BufDescr.frameLen = Len;
      BufDescr.framePtr = (uint8*)EtherHdr_p;
      apio_write(etherHandle,&BufDescr);

    }

    eth_drv_tx_done(sc1,LlcData_p->Key,0);
    pool_FreeBuf((uint8 *)LlcData_p);
  } /* while end */
  return 1;
}

#if 0
/******************************************************************************
 *
 * Name: dsQ_LlcDataWriteNoBlock
 *
 * Description:
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern int dsQ_LlcDataWriteNoBlock(dsQ_LlcData_t *LlcData_p )
{
  int  status;

  if ((status = os_QueueWriteNoBlock(sysinfo_DS_LLC_DATA_Q,
                                     LlcData_p)) == SUCCESS) {
    ds_SendEvent(ds_LLC_DATA_RCVD);

    return(SUCCESS);
  }
  else {
    return(FAIL);
  }
}
extern int dsQ_LllcDataWriteNoBlock(dsQ_LlcData_t *LlcData_p )
{
  int  status;

  if ((status = os_QueueWriteNoBlock(sysinfo_DS_LLC_DATA_Q,
                                     LlcData_p)) == SUCCESS) {
    ds_SendEvent(ds_LLC_DATA_RCVD);

    return(SUCCESS);
  }
  else {
    return(FAIL);
  }
}
/******************************************************************************
 *
 * Name: ds_GetLlcDataBuf
 *
 * Description:
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
dsQ_LlcData_t *ds_GetLlcDataBuf(void)
{
  return((dsQ_LlcData_t *)pool_GetBuf(llcDataPoolId));
}


void SetupPollingTimer()
{
  uint32 TimerMask;
  uint32 TimerControl;

  W81_READ_BYTE(TIMER_IMR, TimerMask);
  W81_WRITE_BYTE(TIMER_IMR, (TimerMask | 0x4));
  W81_READ_HWORD(TIMER_CONTROL, TimerControl);
  W81_WRITE_HWORD(TIMER_CONTROL,(TimerControl|0x0000));
  W81_WRITE_HWORD(TIMER3_LEN, 0x8ff); 
  W81_READ_HWORD(TIMER_CONTROL, TimerControl);
  W81_WRITE_HWORD(TIMER_CONTROL,(TimerControl|0xf00));

  W81_WRITE_BYTE(TIMER_SRC, 0x4);


}
#endif
