#include "msApiDefs.h"
#include "msApiPrototype.h"
#include "gtDrvSwRegs.h"
#include "MvCore.h"

#if 0
#define EAGLE_MAC_READ(x,y) *y = *(volatile unsigned int *) (0x80008000 + x)
#define EAGLE_MAC_WRITE(x,y) *(volatile unsigned int *) (0x80008000 + x) = y;
#else
void EAGLE_MAC_READ(unsigned x,unsigned *y);
void EAGLE_MAC_WRITE(unsigned x,unsigned y);
#endif

#define W81_WRITE_BYTE(x,y) * (unsigned char *) x = y;
#define W81_WRITE_HWORD(x,y) * (unsigned short *) x = y;
#define W81_WRITE_WORD(x,y) * (unsigned int *) x = y;
typedef unsigned int   uint32;
typedef unsigned int   SMI_REG;
typedef unsigned short uint16;
typedef unsigned char  uint8;

#define ETHER_SMI_REG 0x10
#define SMI_BUSY (1 << 28)
#define SMI_OP_CODE_BIT_READ                    1
#define SMI_OP_CODE_BIT_WRITE                   0
#define READ_VALID                              (1<<27)

GT_BOOL readMiiReg (GT_QD_DEV* dev, uint32 portNumber , uint32 MIIReg,
                        uint32* value);
GT_BOOL writeMiiReg (GT_QD_DEV* dev, uint32 portNumber , uint32 MIIReg,
                        uint32  value);

GT_STATUS gprtGetPortCtr
(
    IN  GT_QD_DEV       *dev,
    IN  GT_LPORT        port,
    OUT GT_PORT_STAT    *ctr
);
GT_STATUS hwGetGlobalRegField
(
    IN GT_QD_DEV *dev,
    IN  GT_U8    regAddr,
    IN  GT_U8    fieldOffset,
    IN  GT_U8    fieldLength,
    OUT GT_U16   *data
);

GT_PORT_STAT  stats[10];
GT_STATUS retVal;

void os_TaskDelay(int x);

GT_QD_DEV       etherDev;
GT_STATUS hwReadPortReg
(
    IN GT_QD_DEV *dev,
    IN  GT_U8    portNum,
    IN  GT_U8    regAddr,
    OUT GT_U16   *data
    );

void
getstats()
{
    int i;
   /* 6052 should reset with VLAN map set up with all ports accessible by */
   /* all other ports.  Set up after testing with Wan port mapped only to */  
   /* cpu port and not accessible by any other port.                      */
   for(i=0;i < etherDev.numOfPorts;i++) {
	   //retVal = gstpGetPortState(&etherDev, i, &stats[i]);
       retVal = gprtGetPortCtr(&etherDev, i, &stats[i]);
   }
}
void
etherswitch_init()
{
    int           i;
    uint16        deviceId;
    GT_STATUS     retVal; 
    uint16        ready;
    /* Functions return value.      */
    /* Initialize Marvell 88E6052/60 driver. */

   /* Power up using GPIO port bit 3.  Drive high-low-high for reset. */

   /* Set bit 3 to output mode on GPIO. Leave other GPIO unchanged. */
   W81_WRITE_HWORD(0x8000d000, 0xff7f);
   /* Program bit 3 to output mode. */
   W81_WRITE_HWORD(0x8000d008, 0xfff7);
   /* Set bit 3 output high then pause. */
   W81_WRITE_HWORD(0x8000d00c, 0xffff);
   os_TaskDelay(2);
   /* Set bit 3 output low then pause. */
   W81_WRITE_HWORD(0x8000d00c, 0xff7f);
   os_TaskDelay(2);
   /* Set bit 3 output high.  6052 should now be reset. */
   W81_WRITE_HWORD(0x8000d00c, 0xffff);

   /*
    *  Register all the required functions to QuarterDeck Driver.
   */

   etherDev.devNum       = 1;
   etherDev.devEnabled   = 1;
   etherDev.baseRegAddr  = 0x10;
   etherDev.numOfPorts   = 0x7;
   etherDev.cpuPortNum   = 0x5;
   etherDev.maxPhyNum    = 0x5;
   etherDev.stpMode      = 0;
   etherDev.devName      = 0;
   etherDev.atuRegsSem   = 0;
   etherDev.vtuRegsSem   = 0;
   etherDev.statsRegsSem = 0;
   etherDev.fgtReadMii   = readMiiReg;
   etherDev.fgtWriteMii  = writeMiiReg;
   etherDev.semCreate    = NULL;
   etherDev.semDelete    = NULL;
   etherDev.semTake      = NULL;
   etherDev.semGive      = NULL;
   etherDev.appData      = NULL;

   /* Now initialize 6052 chip for default setting - For now configure chip */
   /* to be a simple bridge with layer 2 switching between ports, all ports */
   /* can communicate with all other ports in this configuration (intended  */
   /* for testing purposes. */


   /* Get the Device Id and revision from the switch. */ 
   hwReadPortReg(&etherDev,1,QD_REG_SWITCH_ID,&deviceId);
   etherDev.revision = (uint8) (deviceId & 0x000F);
   etherDev.deviceId = (0x00FF) & (deviceId >> 4);

   /* Enable all ports. Ports 0-6 */

   for (i=0 ; i < etherDev.numOfPorts ; i++)
      gstpSetPortState(&etherDev, i, 0x3);  

   gprtClearAllCtr(&etherDev);

   getstats();
  
   /* use pin config. force control, unmodify ingress, no multicast limit, */
   /* normal trailer mode. */
   for (i=0 ; i < etherDev.numOfPorts ; i++)
   {
      gprtSetForceFc(&etherDev, i, GT_FALSE);
      gprtSetIngressMode(&etherDev, i, GT_UNMODIFY_INGRESS);
      gprtSetMcRateLimit(&etherDev, i, GT_MC_100_PERCENT_RL);
      gprtSetTrailerMode(&etherDev, i, GT_FALSE);
   }

    retVal = hwGetGlobalRegField(&etherDev,QD_REG_GLOBAL_STATUS,10,2,&ready);
#if 0
    if((retVal != GT_OK) || (ready != 3))
    {
	exit(1);
    }
#endif
}

extern void etherSwitch_Init( void )
{
    uint32 i;
    uint16 deviceId;

    /* Initialize Marvell 88E6052/60 driver. */

    /*
     *  Register all the required functions to QuarterDeck Driver.
    */

    etherDev.devNum = 1;
    etherDev.devEnabled = 1;
    etherDev.baseRegAddr = 0x10;
    etherDev.numOfPorts = 0x7;
    etherDev.cpuPortNum = 0x5;
    etherDev.maxPhyNum = 0x5;
    etherDev.stpMode = 0;
    etherDev.devName = 0;
    etherDev.atuRegsSem = 0;
    etherDev.vtuRegsSem = 0;
    etherDev.statsRegsSem = 0;
    etherDev.fgtReadMii = readMiiReg;
    etherDev.fgtWriteMii = writeMiiReg;
    etherDev.semCreate = NULL;
    etherDev.semDelete = NULL;
    etherDev.semTake = NULL;
    etherDev.semGive = NULL;
    etherDev.appData = NULL;

    /* Now initialize 6052 chip for default setting - For now configure chip */
    /* to be a simple bridge with layer 2 switching between ports, all ports */
    /* can communicate with all other ports in this configuration (intended  */
    /* for testing purposes. */


    /* Get the Device Id and revision from the switch. */
    hwReadPortReg(&etherDev, 1, QD_REG_SWITCH_ID, &deviceId);
    etherDev.revision = (uint8) (deviceId & 0x000F);
    etherDev.deviceId = (0x00FF) & (deviceId >> 4);
    etherDev.devName = etherDev.deviceId;

    /* Enable all ports. Ports 0-6 */

    for (i = 0 ; i < etherDev.numOfPorts ; i++)
        gstpSetPortState(&etherDev, i, 0x3);

    /* use pin config. force control, unmodify ingress, no multicast limit, */
    /* normal trailer mode. */
    for (i = 0 ; i < etherDev.numOfPorts ; i++)
    {
        gprtSetForceFc(&etherDev, i, GT_FALSE);
        gprtSetIngressMode(&etherDev, i, GT_UNMODIFY_INGRESS);
        gprtSetMcRateLimit(&etherDev, i, GT_MC_100_PERCENT_RL);
        gprtSetTrailerMode(&etherDev, i, GT_FALSE);
    }

    /* 6052 should reset with VLAN map set up with all ports accessible by */
    /* all other ports.  Set up after testing with Wan port mapped only to */
    /* cpu port and not accessible by any other port.                      */

    /* SohoGatewayVlan sets up a default VLAN where the Wan and cpu ports  */
    /* are in one VLan and all ports excluding the Wan but including the   */
    /* cpu port are in another Vlan.  In this configuration the Wan port   */
    /* can only communicate with the other ports through the cpu.          */
#ifdef VLAN
    SohoGatewayVlan(&etherDev, etherDev.numOfPorts, etherDev.cpuPortNum);
    /* Set Egress trailer mode for cpu port 5. */
    gprtSetTrailerMode(&etherDev, 5, GT_TRUE);
    /* Set Ingress trailer mode for cpu port 5. */
    gprtSetIngressMode(&etherDev, 5, GT_TRAILER_INGRESS);
#endif

}

/****************************************************************************** 
 * 
 * Name: readMIIReg
 * 
 * Description: 
 *   This routine reads the MII register for the specified port.
 * 
 * Conditions For Use: 
 *   There is a device connected to the MII interface. 
 * 
 * Arguments: 
 *   portNumber - Port number.
 *   MIIReg     - MII Register to read.
 *   value      - pointer passed that MII register content is stored.
 * 
 * Return Value: 
 *   Boolean  True successful read.
 *            False unsuccessful read.
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *    
 * END PDL 
 * 
 *****************************************************************************/ 

GT_BOOL readMiiReg (GT_QD_DEV* dev, uint32 portNumber, uint32 MIIReg,
                        uint32* value)
{
    SMI_REG smiReg;
    uint32 phyAddr;
    uint32 timeOut = 10; /* timeOut for Error condition. */


    /* first check that it is not busy */
    EAGLE_MAC_READ (ETHER_SMI_REG,(uint32*)&smiReg);
    while (smiReg & SMI_BUSY) 
    {
        os_TaskDelay(2);
        EAGLE_MAC_READ (ETHER_SMI_REG,(uint32*)&smiReg);
        if (timeOut-- <1 )
            return GT_FALSE; 
    }

    /* not busy */

    phyAddr = portNumber;

    smiReg =  (phyAddr << 16) | (SMI_OP_CODE_BIT_READ << 26) | (MIIReg << 21) |
         SMI_OP_CODE_BIT_READ<<26;

    EAGLE_MAC_WRITE (ETHER_SMI_REG,*((uint32*)&smiReg));
    timeOut = 10; /* Reinitialize the time out. */
    EAGLE_MAC_READ (ETHER_SMI_REG,(uint32*)&smiReg);
    while (!(smiReg & READ_VALID)) 
    {
        os_TaskDelay(2);
        EAGLE_MAC_READ (ETHER_SMI_REG,(uint32*)&smiReg);
        if (timeOut-- <1 )
            return GT_FALSE; 
    }

    *value = (uint32)(smiReg & 0xffff);

//    printf("readMii %d %d = %x\n", portNumber, MIIReg, smiReg & 0xffff);
    
    return GT_TRUE;


}


/****************************************************************************** 
 * 
 * Name: writeMIIReg
 * 
 * Description: 
 *   This routine writes data to the MII register for the specified port.
 * 
 * Conditions For Use: 
 *   There is a device connected to the MII interface. 
 * 
 * Arguments: 
 *   portNumber - Port number.
 *   MIIReg     - MII Register to write data.
 *   value      - data to be written to MII Register.
 * 
 * Return Value: 
 *   Boolean  True successful write.
 *            False unsuccessful write.
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *    
 * END PDL 
 * 
 *****************************************************************************/

GT_BOOL writeMiiReg (GT_QD_DEV* dev, uint32 portNumber, uint32 MIIReg,
                       uint32 value)
{
    SMI_REG smiReg;
    uint32 phyAddr;
    uint32 timeOut = 10; /* timeOut for Error condition. */

//    printf("writeMii %d %d %x\n", portNumber, MIIReg, value);
    /* first check that it is not busy */
    EAGLE_MAC_READ (ETHER_SMI_REG,(uint32*)&smiReg);
    while (smiReg & SMI_BUSY) 
    {
        os_TaskDelay(2);
        EAGLE_MAC_READ (ETHER_SMI_REG,(uint32*)&smiReg);
        if (timeOut-- < 1 )
            return GT_FALSE; 
    }

    /* not busy */

    phyAddr = portNumber;

    smiReg = 0;
    smiReg = smiReg | (phyAddr << 16) | (SMI_OP_CODE_BIT_WRITE << 26) |
             (MIIReg << 21) | (value & 0xffff);

    EAGLE_MAC_WRITE (ETHER_SMI_REG,*((uint32*)&smiReg));

    return GT_TRUE;
}
