/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtDesc.h  
*
* DESCRIPTION:                                                                
*       Descriptors header file.
*
* DEPENDENCIES:
*       gtEther.c/.h
*       gtMpsc.c/.h
*       gtSdma.c/.h
*
*******************************************************************************/

#ifndef __INCgtDesch 
#define __INCgtDesch 

/* includes */
#include "MvCore.h"
#include "gtMpsc.h"

/* typedefs */
typedef enum _descInitStatus {DESC_INIT_SUCCESS = 0,DESCS_BASE_NOT_ALLINGED=1, 
                              BUFFERS_BASE_NOT_ALLINGED = 2, BUFFER_TOO_BIG = 4,
                              WRONG_NUM_OF_BUFFERS_PER_PACKET = 8
                             } DESC_INIT_STATUS;
typedef enum _descOwner   {DESC_OWNER_IS_GT, DESC_OWNER_IS_CPU
                          } DESC_OWNER;
typedef enum _descCommand {CREATE_CYCLIC_CHAIN = 1, NO_CYCLIC_CHAIN = 2
                          } DESC_COMMAND;

typedef struct _rxDescriptor
{
  volatile unsigned int    	cmd_sts; 
  volatile 	unsigned short		bytecnt;
  volatile unsigned short		bufsize;
  volatile 	unsigned int		buf_ptr; 
  volatile unsigned int  		next_desc_ptr; 

} RX_DESC;
 
typedef struct _txDescriptor
{   
    volatile unsigned int    	    cmd_sts;
    volatile unsigned short		    bytecnt;
  volatile unsigned short		    shadow;
    volatile unsigned int		    buf_ptr;
    volatile unsigned int    	    next_desc_ptr; 
} TX_DESC;

/* defines */
#define DESC_SIZE                                   0x10
#define DESC_RX_BUF_OVERHEAD_FOR_CRC                4

/* Command/Status word bits, common for eth/mpsc */ 
#define DESC_OWNED_BY_GT			                BIT31
#define DESC_AUTO_MODE        		                BIT30
#define DESC_ENABLE_INTERRUPT	         	        BIT23
#define DESC_FIRST				          	        BIT17                                
#define DESC_LAST				          	        BIT16
#define DESC_ERROR_SUMMARY	           		        BIT15
#define DESC_DATA_OVERRUN                           BIT6    

/* command/Status word bits for tx: */
#define DESC_GENERATE_ERROR_CODE_TX		            BIT22 /* eth, hdlc(CRC)/ 
                                                             bisync,
                                                             transparent
                                                             (BCC,LRC) */
#define DESC_ADD_PADDING     		          		BIT18 /*  eth */
#define DESC_TX_COLLISION_OCCURRED  		        BIT9  /*  eth, hdlc */
#define DESC_RETRANSMIT_LIMIT_ERROR 		        BIT8  /*  eth, hdlc */
#define DESC_DATA_UNDERRUN                          BIT6  /*  eth, hdlc, 
                                                              transparent */
#define DESC_LATE_COLLISION_ERROR                   BIT5  /*  eth */
#define DESC_TRANSMISSION_DEFERRED                  BIT3  /*  hdlc */ 
#define DESC_CTS_LOSS                               BIT1  /*  hdlc, bisync,
                                                              Uart, 
                                                              transparent */
#define DESC_RESET_BCC                              BIT21 /*  bisync */
#define DESC_BCE_BCC_ENABLE                         BIT20 /*  bisync */
#define DESC_TRANSMIT_DLE_BEFORE_BUFFER             BIT19 /*  bisync */
#define DESC_BYSINC_IN_TRANSPARENT_MODE             BIT18 /*  bisync */
#define DESC_NO_STOP_BIT                            BIT20 /*  Uart */ 
#define DESC_UART_ADDRESS_BIT                       BIT19 /*  Uart */
#define DESC_UART_PREAMBLE                          BIT18 /*  Uart */
 
/******************************************/

/* command/Status word bits for rx: */
#define DESC_IGMP_PACKET      		          		BIT14 /*  eth */
#define DESC_ETH_HASH_TABLE_EXPIRED  		        BIT13 /*  eth */ 
#define DESC_ETH_MISSED_FRAME    		          	BIT12 /*  eth */ 
#define DESC_ETH_FRAME_TYPE    		                BIT11 /*  eth */ 
#define DESC_SHORT_FRAME_ERROR 		          	    BIT8  /*  eth, hdlc */
#define DESC_MAX_FRAME_LENGTH_ERROR          		BIT7  /*  eth, hdlc */
#define DESC_ETH_RX_COLLISION                       BIT4  /*  eth */ 
#define DESC_CRC_ERROR                              BIT0  /*  eth, hdlc */
#define DESC_RESIDUE_2                              BIT6  /*  hdlc */
#define DESC_RESIDUE_1                              BIT5  /*  hdlc */
#define DESC_RESIDUE_0                              BIT4  /*  hdlc */
#define DESC_ABORT_SEQUENCE                         BIT4  /*  hdlc */
#define DESC_NON_OCTET_FRAME                        BIT3  /*  hdlc */
#define DESC_DECODING_ERROR                         BIT2  /*  hdlc, bisync,
                                                              transparent */
#define DESC_CD_LOSS                                BIT1  /*  hdlc, bisync, Uart
                                                              ,transparent */
#define DESC_LAST_BYTES_IN_BUFFER_ARE_BCC           BIT14 /*  bisync */
#define DESC_LAST_BYTES_IN_BUFFER_CONTROL_CHARS     BIT13 /*  bisync */
#define DESC_TRANSPARENT_BUFFER                     BIT11 /*  bisync */
#define DESC_PAD_REPORT                             BIT9  /*  bisync */ 
#define DESC_BYSINC_PARITY_ERROR                    BIT4  /*  bisync */ 
#define DESC_DLE_ERROR                              BIT3  /*  bisync */ 
#define DESC_CRC_LRC_ERROR                          BIT0  /*  bisync,
                                                              transparent */
#define DESC_LAST_BYTE_IS_CONTROL_CHAR              BIT14 /*  Uart */ 
#define DESC_TRANSPARENCY_CONTROL_OCTET             BIT13 /*  Uart */ 
#define DESC_ADDRESS_MATCH                          BIT12 /*  Uart */ 
#define DESC_FIRST_BYTE_IS_ADDRESS                  BIT11 /*  Uart */ 
#define DESC_MAX_IDLE_TIMER_EXPIRATION              BIT10 /*  Uart */ 
#define DESC_BREAK_RECEIVED                         BIT9  /*  Uart */ 
#define DESC_FRAMING_ERROR                          BIT3  /*  Uart */ 
#define DESC_UART_PARITY_ERROR                      BIT0  /*  Uart */ 

/******************************************/

/* functions */

DESC_INIT_STATUS gtDescInitRxChain(unsigned int numOfDesc, 
                                   unsigned int firstRxDescAddr,
                                   unsigned int firstRxBufferAddr,
                                   unsigned int rxBufferSize,
                                   unsigned int cmdStsWord,
								   DESC_COMMAND command);
DESC_INIT_STATUS gtDescInitTxChain(unsigned int numOfDesc, 
                                   unsigned int firstTxDescAddr,
                                   unsigned int firstTxBufferAddr,
                                   unsigned int txBufferSize,
                                   unsigned int cmdStsWord,
								   DESC_COMMAND command, 
                                   unsigned int numOfBufferPacket);
DESC_INIT_STATUS gtDescInitTxChainForBisync(unsigned int bisyncMode,
											unsigned int numOfDesc, 
                                            unsigned int firstTxDescAddr,
                                            unsigned int firstTxBufferAddr,
                                            unsigned int txBufferSize,
                                            unsigned int cmdStsWord,
								            DESC_COMMAND command);
DESC_INIT_STATUS gtDescInitTxChainForBisyncHeader(unsigned int bisyncMode,
											      unsigned int numOfDesc, 
                                                  unsigned int firstTxDescAddr,
                                                  unsigned int firstTxBuffAddr,
                                                  unsigned int txBufferSize,
                                                  unsigned int cmdStsWord,
								                  DESC_COMMAND command);
DESC_OWNER       gtDescGetTxOwnership(unsigned int descAddr);
DESC_OWNER       gtDescGetRxOwnership(unsigned int descAddr);
void             gtDescSetTxOwnedByGT(unsigned int firstTxDescAddr,
                                      unsigned int numOfDesc);
void             gtDescSetRxOwnedByGT(unsigned int firstRxDescAddr, 
                                      unsigned int numOfDesc);
bool             gtDescIsOverRunSet(unsigned int firstRxDescAddr, 
                                    unsigned int numOfDesc);
bool             gtDescIsUnderRunSet(unsigned int firstTxDescAddr, 
                                     unsigned int numOfDesc);
 


#endif /* __INCgtDesch */






