/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtEther.h 
*
* DESCRIPTION:                                                                 
*       Ethernet functions header file.
*
* DEPENDENCIES:
*
*******************************************************************************/

#ifndef __INCgtEtherh 
#define __INCgtEtherh 

/* includes */
#include "MvCore.h"
#include "EagleReg.h"

/* typedefs */
typedef enum _ethPort {ETH0,ETH1
#ifdef   INCLUDE_ETH_PORT_2
                       ,ETH2
#endif /* INCLUDE_ETH_PORT_2 */
                      }ETH_PORT;
/* Lists the 4 Rx priority Queues */ 
typedef enum _ethRXPrio {ETH_RX_PRIO0, ETH_RX_PRIO1, ETH_RX_PRIO2, ETH_RX_PRIO3
                        } ETH_RX_PRIO;
/* Lists the 2 Tx priority Queues */ 
typedef enum _ethTXPrio {ETH_TX_PRIO0, ETH_TX_PRIO1} ETH_TX_PRIO;
typedef enum _ethHashMode {ETH_HASH_MODE_0, ETH_HASH_MODE_1} ETH_HASH_MODE;
typedef enum _ethHashTableSize {_8K_ADDRESSES_TABLE, _512_ADDRESSES_TABLE
                               } ETH_HASH_TABLE_SIZE;
typedef enum _ethHashTableStatus {HASH_TABLE_SUCCESS = 0, 
                                  HASH_TABLE_BASE_NOT_ALIGNED = 1,
                                  UNABLE_TO_ADD_ADDRESS = 2, 
                                  ADDRESS_NOT_IN_TABLE = 4
                                 } ETH_HASH_TABLE_STATUS; 
typedef enum _ethRDBit{DISCARD_UPON_MATCH, RECEIVE_UPON_MATCH} ETH_RD_BIT;

/* defines */

/* SDMA command register - SDCMRx */            
#define ETH_ABORT_RX                            BIT15
#define ETH_STOP_TRANSMIT_PRIO0                 BIT17
#define ETH_STOP_TRANSMIT_PRIO1                 BIT16
#define ETH_ENABLE_RX                           BIT7
#define ETH_START_TX_PRIO0                      BIT24
#define ETH_START_TX_PRIO1                      BIT23
#define ETH_ABORT_TX                            BIT31

/* Port configuration register - PCR */
#define ETH_PROMISCUOUS_MODE                    BIT0
#define ETH_REJECT_BROADCAST                    BIT1
#define ETH_PASS_BAD_FRAMES                     BIT2
#define ETH_PORT_ENABLE                         BIT7
#define ETH_INTERNAL_LOOPBACK                   BIT8
#define ETH_EXTERNAL_LOOPBACK                   BIT9
#define ETH_HASH_SIZE_256KB                     0
#define ETH_HASH_SIZE_16KB                      BIT12
#define ETH_HASH_SIZE_BIT                       BIT12
#define ETH_HASH_FUNCTION_0                     0 
#define ETH_HASH_FUNCTION_1                     BIT13 
#define ETH_HASH_FUNCTION_BIT                   BIT13 
#define ETH_FULL_DUPLEX                         BIT15

/* port configuration extend reg - PCXR */
#define ETH_IGMP_CAPTURE_ENABLE                 BIT0
#define ETH_SPAN_CAPTURE_ENABLE                 BIT1
#define ETH_PARTITION_ENABLE                    BIT2
#define ETH_DISABLE_AUTO_NEG_FOR_DUPLEX_MODE    BIT9
#define ETH_OVERRIDE_PRIORITY_FOR_RX_PACKETS    BIT8
#define ETH_DISABLE_AUTO_NEG_FOT_FLOW_CONTROL   BIT10
#define ETH_MAX_FRAME_LENGTH_1536B              BIT14
#define ETH_MAX_FRAME_LENGTH_1518B              0
#define ETH_MAX_FRAME_LENGTH_2KB                BIT15
#define ETH_MAX_FRAME_LENGTH_64KB               (BIT14|BIT15)
#define ETH_MIB_NO_COUNTERS_CLEAR_MODE          BIT16
#define ETH_PORT_SPEED_10                       0
#define ETH_PORT_SPEED_100                      BIT18
#define ETH_DISABLE_AUTO_NEG_FOR_SPEED          BIT19
#define ETH_RMII_ENABLE                         BIT20

/* eth interrupt cause reg (ICR) */

#define ETH_RX_BUFFER_RETURN                    BIT0
#define ETH_TX_BUFFER_HIGH_RETURN               BIT2
#define ETH_TX_BUFFER_LOW_RETURN                BIT3
#define ETH_TX_END_HIGH                         BIT6
#define ETH_TX_END_LOW                          BIT7
#define ETH_RX_RESOURCE_ERROR                   BIT8
#define ETH_TX_HIGH_RESOURCE_ERROR              BIT10
#define ETH_TX_LOW_RESOURCE_ERROR               BIT11
#define ETH_RX_OVERRUN                          BIT12
#define ETH_TX_UNDERRUN                         BIT13
#define ETH_RX_BUFFER_RETURN_QUEUE_0            BIT16
#define ETH_RX_BUFFER_RETURN_QUEUE_1            BIT17
#define ETH_RX_BUFFER_RETURN_QUEUE_2            BIT18
#define ETH_RX_BUFFER_RETURN_QUEUE_3            BIT19
#define ETH_RX_RESOURCE_ERROR_QUEUE_0           BIT20
#define ETH_RX_RESOURCE_ERROR_QUEUE_1           BIT21
#define ETH_RX_RESOURCE_ERROR_QUEUE_2           BIT22
#define ETH_RX_RESOURCE_ERROR_QUEUE_3           BIT23
#define ETH_MII_PHY_STATUS_CHANGE               BIT28
#define ETH_SMI_COMMAND_DONE                    BIT29
#define ETH_INTERRUPT_SUMMARY                   BIT31

/* sdma configuration reg - SDCR */
#define ETH_BURST_SIZE_8B                       0
#define ETH_BURST_SIZE_16B                      BIT12
#define ETH_BURST_SIZE_32B                      BIT13
#define ETH_BURST_SIZE_64B                      (BIT12|BIT13)
#define ETH_RECEIVE_INTERRUPT_ON_FRAME          BIT9
#define ETH_PCI_OVERRIDE                        BIT8
#define ETH_BLMT                                BIT7
#define ETH_BLMR                                BIT6

/* SMI definitions (SMIR) */
#define ETH_SMI_READ                            BIT26
#define ETH_SMI_WRITE                           0
#define ETH_SMI_READ_VALID                      BIT27
#define ETH_SMI_IS_BUSY                         BIT28


/**************** hash table definitions ******************/
#define ETH_HASH_HOP_NUMBER                     12

#define ETH_HASH_VALID_BIT                      BIT0
#define ETH_HASH_SKIP_BIT                       BIT1
#define ETH_HASH_RD_BIT                         BIT2


typedef struct _hashTableEntryData
{
    unsigned int    macLow;     /* bits 31:0 */
    unsigned int    macHigh;    /* bits 47:32 (16 bits) */
    ETH_RX_PRIO     prio;
    ETH_RD_BIT      rdBit;
} HASH_TABLE_ENTRY_DATA;

#ifdef LE
typedef struct _hashTableEntry
{
    unsigned int    lowWord;     /* bits 31:0 */
    unsigned int    highWord;    /* bits 63:32 */    
} HASH_TABLE_ENTRY;

#else /* BE */
typedef struct _hashTableEntry
{
    unsigned int    highWord;    /* bits 63:32 */     
    unsigned int    lowWord;     /* bits 31:0 */ 
} HASH_TABLE_ENTRY;

/* macros in use of hash table functions */
/* flip lower 6 bits (bits 5:0 -> 0:5), reset bits 31:6 */
#define INVERSE_6_BITS(wordToInverse) (((wordToInverse << 5) & 0x20) |    \
                                       ((wordToInverse << 3) & 0x10) |    \
                                       ((wordToInverse << 1) & 0x8)  |    \
                                       ((wordToInverse >> 1) & 0x4)  |    \
                                       ((wordToInverse >> 3) & 0x2)  |    \
                                       ((wordToInverse >> 5) & 0x1)) 
                                 
/* flip lower 9 bits (bits 8:0 -> 0:8), reset bits 31:9 */
#define INVERSE_9_BITS(wordToInverse) (((wordToInverse << 8) & 0x100) |   \
                                       ((wordToInverse << 6) & 0x80)  |   \
                                       ((wordToInverse << 4) & 0x40)  |   \
                                       ((wordToInverse << 2) & 0x20)  |   \
                                       ((wordToInverse) & 0x10)       |   \
                                       ((wordToInverse >> 2) & 0x8)   |   \
                                       ((wordToInverse >> 4) & 0x4)   |   \
                                       ((wordToInverse >> 6) & 0x2)   |   \
                                       ((wordToInverse >> 8) & 0x1))      \
/* flip lower 4 bits (bits 3:0 -> 0:3), reset bits 31:4 */              
#define INVERSE_NIBBLE(wordToInverse) (((wordToInverse << 3) & 0x8) |     \
                                       ((wordToInverse << 1) & 0x4) |     \
                                       ((wordToInverse >> 1) & 0x2) |     \
                                       ((wordToInverse >> 3) & 0x1))
                                       
#define SWAP_NIBBLES(byteToSwap) (((byteToSwap << 4) & 0xf0) |            \
                                  ((byteToSwap >> 4) & 0x0f))  
                                                            
#endif



/**************** end of hash table definitions ******************/

/* functions */
void                  gtEtherStartTransmit(ETH_PORT port, 
                                           unsigned int firstTxDescAddr,
                                           ETH_TX_PRIO prio);
void                  gtEtherEnableReception(ETH_PORT port, 
                                             unsigned int firstRxDescAddr, 
                                             ETH_RX_PRIO prio);
void                  gtEtherStartTx(ETH_PORT port, ETH_TX_PRIO prio);
void                  gtEtherEnableRx(ETH_PORT port, ETH_RX_PRIO prio);
void                  gtEtherStopTx(ETH_PORT port, ETH_TX_PRIO prio);
void                  gtEtherAbortTx(ETH_PORT port);
void                  gtEtherAbortRx(ETH_PORT port);
bool                  gtEtherIsAbortTxEnded(ETH_PORT port);
bool                  gtEtherIsAbortRxEnded(ETH_PORT port);
ETH_HASH_TABLE_STATUS gtEtherInitHashTable(ETH_PORT port, ETH_HASH_MODE mode, 
                                           ETH_HASH_TABLE_SIZE size, 
                                           unsigned int hashTableBase, 
                                           unsigned int numOfEntries, 
                                           HASH_TABLE_ENTRY_DATA* entries);
ETH_HASH_TABLE_STATUS gtEtherAddEntryToHash(ETH_PORT port, 
                                            HASH_TABLE_ENTRY_DATA* entryData);
ETH_HASH_TABLE_STATUS gtEtherRemoveEntryFromHash(ETH_PORT port, 
                                              HASH_TABLE_ENTRY_DATA* entryData);
bool                  gtEtherReadPHYReg(ETH_PORT port, unsigned int PHYReg, 
                                        unsigned int* value);
bool                  gtEtherWritePHYReg(ETH_PORT port, unsigned int PHYReg, 
                                         unsigned int value);
void                  gtEtherDefaultInit(ETH_PORT port);
void                  gtEtherSetPhyAddress(ETH_PORT port,
                                                       unsigned int newAddress);
unsigned int          gtEtherGetPhyAddress(ETH_PORT port);

#endif /* __INCgEtherh */





