/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtSdma.c  
*
* DESCRIPTION:
*       Mpscs' sdma channels related functions.
*
* DEPENDENCIES:
*       gtMpsc.c/.h
*       gtDesc.c/.h
*
*******************************************************************************/

/* includes */
#include "gtMpsc.h"
#include "MvCore.h"
#include "gtSdma.h"

/*******************************************************************************
* gtSdmaStartTransmit - start transmit
*
* DESCRIPTION:
*       Write first descriptor address in the given port's first and current 
*       descriptor registers and start transmit. 
*
* INPUT:
*      port - the port that will transmit
*      group - the sdma group												 
*      firstTxDescAddr - first tx desc in chain
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtSdmaStartTransmit(SDMA_GROUP group, SDMA_CHANNEL channel, 
                         unsigned int firstTxDescAddr)
{
    /* write the first  tx desc address to  CDP and FDP */
    GT_REG_WRITE(SDMA_CURR_TX_DESC_PTR(group, channel),
                                               PHY_TO_VIRTUAL(firstTxDescAddr));
    GT_REG_WRITE(SDMA_1ST_TX_DESC_PTR(group, channel), 
                 PHY_TO_VIRTUAL(firstTxDescAddr));
    GT_SET_REG_BITS(SDMA_COMM_REG(group, channel), SDMA_TX_DEMAND);
}

/*******************************************************************************
* gtSdmaEnableReception - enable reception
*
* DESCRIPTION:
*       Write first descriptor address in the given port's current descriptor 
*       register and enable reception through this port. 
*
* INPUT:
*      port - the port that will transmit
*      firstRxDescAddr - first rx desc in chain
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtSdmaEnableReception(SDMA_GROUP group, SDMA_CHANNEL channel,  
                           unsigned int firstRxDescAddr)
{
/* write the first rx descriptor address to CDP */

    GT_REG_WRITE(SDMA_CURR_RX_DESC_PTR(group, channel),
                 PHY_TO_VIRTUAL(firstRxDescAddr));
    GT_SET_REG_BITS(SDMA_COMM_REG(group, channel), SDMA_ENABLE_RX);
}

/*******************************************************************************
* gtAbortSdma - abort sdma channel of the given mpsc port
*
* DESCRIPTION:
*      Abort sdma channel of the given mpsc port, both for rx and tx.
*      For aborting for tx or rx only - use macros GT_SDMA_ABORT_RX and
*      GT_SDMA_ABORT_TX.
*
* INPUT:
*      port - mpsc prot whose sdma channel will be aborted.
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtAbortSdma(SDMA_GROUP group, SDMA_CHANNEL channel)
{
	GT_SDMA_ABORT_TX(group, channel);
	GT_SDMA_ABORT_RX(group, channel);
}

/*******************************************************************************
* gtSdmaTxDemand - set tx demand bit
*
* DESCRIPTION:
*       Set tx demand bit for a given sdma. User should initialize current and
*       first tx descriptor registers before calling this function. 
*
* INPUT:
*      sdma - the transmiting sdma channel 
*      group - the sdma group												 
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtSdmaTxDemand(SDMA_GROUP group, SDMA_CHANNEL channel)
{
    GT_SET_REG_BITS(SDMA_COMM_REG(group, channel), SDMA_TX_DEMAND);
}

/*******************************************************************************
* gtSdmaRxEnable - set rx enable bit
*
* DESCRIPTION:
*       Set rx enable bit for a given channel. User should initialize current rx
*       descriptor register before calling this function. 
*
* INPUT:
*      channel - the receiving sdma channel 
*      group - the sdma group												 
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtSdmaRxEnable(SDMA_GROUP group, SDMA_CHANNEL channel)
{
    GT_SET_REG_BITS(SDMA_COMM_REG(group, channel), SDMA_ENABLE_RX);
}






