

#include "board.h"
/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

unsigned flashsize=0;
unsigned flashbase=0xffff0000;
unsigned flashNsect;
unsigned manu;
unsigned part;
int split_sector = -1;
int split_sector_data[]={
    0,
    0x8000,
    0xa000,
    0xc000,
};
int split[2][4]={
  { 16, 8, 8, 32}, 
  { 32, 8, 8, 16}
};
void 
makesplit(unsigned ss)
{
  int top = 0;
  int start = 0;
  int i;
  split_sector = ss;
  if (ss) {
    top = 1;
  }
  for (i=0;i < 4;i++) {
    split_sector_data[i] = start;
    start += split[top][i] * 1024;
  }
}
void amdPreamble()
{
#ifdef SST_FLASH
    _REG16(flashbase | 0xAAAA) = 0x00AA;
    _REG16(flashbase | 0x5554) = 0x0055;
    _REG16(flashbase | 0xAAAA) = 0x0080;
    _REG16(flashbase | 0xAAAA) = 0x00AA;
    _REG16(flashbase | 0x5554) = 0x0055;
#else
    _REG16(flashbase | 0xAAA) = 0x00AA;
    _REG16(flashbase | 0x555) = 0x0055;
    _REG16(flashbase | 0xAAA) = 0x0080;
    _REG16(flashbase | 0xAAA) = 0x00AA;
    _REG16(flashbase | 0x555) = 0x0055;
#endif
}
void amdEraseChip(int sblk, int nblks)
{
    unsigned addr;
    int i;
    int j;
    int done;
    for(i=sblk;i < (sblk+nblks);i++) {
	if(i == split_sector) {
	    unsigned start = (i << (16));
	    for(j=0;j < sizeof(split_sector_data)/sizeof(int);j++) {
		addr= flashbase + start + split_sector_data[j];
		amdPreamble();
#ifdef SST_FLASH
		_REG16(addr) = 0x0050;
#else
		_REG16(addr) = 0x0030;
#endif
		for(done=0;!done;) {
		    if(_REG16(addr) == _REG16(addr)) {
		    //if(_REG16(addr) == 0xffff) {
				done = 1;
	    	}
    	}
//		while(!(_REG(0x80006000)&0x1));
	    }
	} else {
	    addr= flashbase + (i << (16));
	    amdPreamble();
#ifdef SST_FLASH
		_REG16(addr) = 0x0050;
#else
		_REG16(addr) = 0x0030;
#endif
	    for(done=0;!done;) {
			if(_REG16(addr) == _REG16(addr)) {
			//if(_REG16(addr) == 0xffff) {
		    	done = 1;
			}
	    }
//	    while(!(_REG(0x80006000)&0x1));
	}
   }
}

void amdFlashWriteWord( int offset, unsigned short dat)
{
    int done;
    unsigned addr = flashbase+offset;
#ifdef SST_FLASH
   _REG16(flashbase | 0xAAAA) = 0x00AA;  // word address 555
   _REG16(flashbase | 0x5554) = 0x0055;  // word address 2AA
   _REG16(flashbase | 0xAAAA) = 0x00A0;  // word address 555
#else
   _REG16(flashbase | 0xAAA) = 0x00AA;  // word address 555
   _REG16(flashbase | 0x555) = 0x0055;  // word address 2AA
   _REG16(flashbase | 0xAAA) = 0x00A0;  // word address 555
#endif

   _REG16(addr) = dat;  // word address 0xFC000000
   do {
       done = 0;
       if(_REG16(addr) == dat) {
	   done++;
       }
       if(_REG16(addr) == dat) {
	   done++;
       }
   } while(done != 2);
//   while(_REG16(addr) != dat);
}
void
interrogate()
{
  _REG16(flashbase) = 0xf0; // reset
#ifdef SST_FLASH
   _REG16(flashbase | 0xAAAA) = 0x00AA;
   _REG16(flashbase | 0x5554) = 0x0055;
   _REG16(flashbase | 0xAAAA) = 0x0090;
#else
   _REG16(flashbase | 0xAAA) = 0x00AA;
   _REG16(flashbase | 0x555) = 0x0055;
   _REG16(flashbase | 0xAAA) = 0x0090;
#endif
   manu = _REG16(flashbase);     // Manu
   part = _REG16(flashbase + 2); // part ID.
  _REG16(flashbase) = 0xf0; // reset

  // is it a manufacturer we know?
  switch (manu) {
  case AMD:
    // Have we heard of the part?
    switch (part) {
    case 0x22c4: // 160BT
      flashsize = 2*1024*1024;
      flashNsect = 32;
      makesplit(31);
      break;
    case 0x2249: // 160BB
      flashsize = 2*1024*1024;
      flashNsect = 32;
      makesplit(0);
      break;
    case 0x22da: // LV800BT
      flashsize = 1*1024*1024;
      flashNsect = 16;
      makesplit(15);
      break;
    case 0x225b: // LV800BB
      flashsize = 1*1024*1024;
      flashNsect = 16;
      makesplit(0);
      break;
    default:
      printf("UNKNOWN device %x\n", part);
      exit(1);
    }
    break;
  case HYUNDAI: // Hyundai
    switch (part) {
    case 0xD5: // 29F080
      flashsize = 1*1024*1024;
      flashNsect = 16;
      break;
    default:
      printf("UNKNOWN device %x\n", part);
      exit(1);
    }
    break;
    case SST: // SST
        switch(part)
        {
        case 0x2781: // SST39VF800A
            flashsize = 1*1024*1024;
            flashNsect = 16;
            makesplit(15);
            break;
        default:
            return;
        }
        break;
  case MICRONIX:
    // Have we heard of the part?
    switch (part) {
    case 0x22c4: // 160BT
      flashsize = 2*1024*1024;
      flashNsect = 32;
      makesplit(31);
      break;
    default:
      printf("UNKNOWN device %x\n", part);
      exit(1);
    }
    break;
  default:
    printf("UNKNOWN manu %x\n", manu);
    exit(1);
    break;
  }
   
  printf("manu %d part 0x%x flashsize %d split sector @%d\n",
         manu, part, flashsize, split_sector);

  flashbase = (0 - flashsize);
}
void
program_image()
{
    int sblk, nblks;
    int i;
    unsigned short val;
    unsigned short *addr;
    unsigned len;
    int saddr;

    // get info from the flash
    interrogate();

	sblk = 0;
	nblks = flashNsect - 2;


    printf("manu %d part 0x%x flashsize %d split sector @%d\n",
	   manu,part, flashsize, split_sector);
    printf("erasing\n");
    amdEraseChip(sblk, nblks);
    addr = (unsigned short *)0xd0000000;

    printf("Programming\n");
    len = nblks * (64*1024);
    saddr = (sblk * (64*1024));
    for(i=0;i < len;i+= 2) {
	val = *addr++;
	amdFlashWriteWord(saddr+i, val);
    }

  addr = (unsigned short *)0xd0000000;
  for (i=0;i < len;i+= 2, addr++) {
    unsigned short val = *(unsigned short *)(saddr+i+flashbase);
    if (*addr != val) {
      printf("Mismatch @%x %x != %x\n", 
             (saddr+i+flashbase), 
             (unsigned)addr, val);
    }
  }
}

