/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/mem_init.h,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    Defualt definition for verious board parameters
 *	  Memory controller initialization function declerations
 *
 * Public Procedures:
 *    boot                    Main entry for the C part of the boot.
 *
 * Private Procedures:
 *    clock_enable            enable Eagle clocks needed to run the boot.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

#ifndef _MEM_INIT_H_
#define _MEM_INIT_H_

#include "88E6218.h"

/* Memory Controller registers addresses */
#define MEMORY_CNTL_BASE_ADDR			0X80006000
#define CSR_ADDR						(MEMORY_CNTL_BASE_ADDR | 0x0)
#define CFGR0_ADDR						(MEMORY_CNTL_BASE_ADDR | 0x4)
#define TIMER00_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x8)
#define TIMER01_ADDR					(MEMORY_CNTL_BASE_ADDR | 0xC)
#define CFGR1_ADDR						(MEMORY_CNTL_BASE_ADDR | 0x10)
#define TIMER10_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x14)
#define TIMER11_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x18)
#define CFGR2_ADDR						(MEMORY_CNTL_BASE_ADDR | 0x1C)
#define TIMER20_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x20)
#define TIMER21_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x24)
#define CFGR3_ADDR						(MEMORY_CNTL_BASE_ADDR | 0x28)
#define TIMER30_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x2C)
#define TIMER31_ADDR					(MEMORY_CNTL_BASE_ADDR | 0x30)

/* SDRAM Defualt values */

/* CSR Register */
#define REFRESH_INTERVAL				((UINT32)0x157 << 18)
#define BRFRCycle						((UINT32)0x3 << 16)
#define SCLKO_DLY						0 /* TBD */
#define SCLK1_DLY						0 /* TBD */
/* NOTE: until properly adjust - read modify write this register */
#define CSR		(UINT32)(REFRESH_INTERVAL | BRFRCycle | SCLKO_DLY | SCLK1_DLY) 

/* DEVICE 0 Defualt Registers */
/* Device 0 is assumed to be sdram */

/* Configuration Register */
#define BAR0							((UINT32)0xff << 24)	/* Defualt base address */
#define BARMK0							((UINT32)0xfe << 16)  /* Base address mask register - window size = 8MByte*/			
#define KRO0							((UINT32)0x0 << 12) /* keep row closed */
#define PEN0							((UINT32)0x0 << 11) /* parity error disable */
#define PACK_EN0						((UINT32)0x1 << 7) /* support x8, x16, x32 bus sizes*/
#define MEMORY_SZ0						((UINT32)0x1 << 5) /* Memory size for single SDRAM - 64Mbit */						
#define MEMORY_TYPE0					((UINT32)0x2 << 3) /* SDRAM */						
#define BUS_WIDTH0						((UINT32)0x2 << 1) /* 16 bit bus width for single debvice */						
#define CS0En							((UINT32)0x1) /* Chip Select 0 is Enabled */						

#ifdef RD_88E6208
#define WS0								((UINT32)0x2 << 13) /* Total bus width = x16 bit*/
#else /*DB_88E6218 */
#define WS0								((UINT32)0x3 << 13) /* Total bus width = x32 bit*/
#endif /* RD_88E6208 */

#define CFGR0	(UINT32)(BAR0 | BARMK0 | KRO0 | PEN0 | PACK_EN0 | MEMORY_SZ0 | MEMORY_TYPE0 | BUS_WIDTH0 | CS0En | WS0)   

/* TIMR00 register - SDRAM*/
#define CL0								((UINT32)0x3 << 4)  /* Cas latency = 3*/
#define BLEN0							((UINT32)0x2)	/* Burst length = 4 */

#define TIMR00	(UINT32)(CL0 | BLEN0)

/* TIMR01 register - SDRAM*/ 
#define TMRD0							((UINT32)0x2 << 12) /* 2 Clock cycles delay needed @ all Frequencies*/   
#define TWR0							((UINT32)0x2)  /* 2 Clock cycles delay needed @ 100Mhz and @ 125Mhz (15ns)*/   

#ifdef RD_88E6208
#define TRFC0							((UINT32)0x7 << 24) /* 7 clock cycles delay needed @ 100Mhz (66ns)*/ 
#define TRP0							((UINT32)0x2 << 20) /* 2 Clock cycles delay needed @ 100Mhz (20ns)*/   
#define TXSR0							((UINT32)0x8 << 16) /* 8 Clock cycles delay needed @ 100Mhz (75ns)*/   
#define TRAS0							((UINT32)0x5 << 8)  /* 5 Clock cycles delay needed @ 100Mhz (44ns)*/   
#define TRCD0							((UINT32)0x2 << 4)  /* 2 Clock cycles delay needed @ 100Mhz (20ns)*/   

#else /*DB_88E6218 */
#define TRFC0							((UINT32)0x9 << 24)  /* 9 clock cycles delay needed @ 125Mhz (70ns)*/ 
#define TRP0							((UINT32)0x3 << 20)  /* 3 Clock cycles delay needed @ 125Mhz (20ns)*/   
#define TXSR0							((UINT32)0x10 << 16) /* 10 Clock cycles delay needed @ 125Mhz (80ns)*/   
#define TRAS0							((UINT32)0x7 << 8)   /* 6 Clock cycles delay needed @ 125Mhz (50ns)*/   
#define TRCD0							((UINT32)0x3 << 4)  /* 3 Clock cycles delay needed @ 125Mhz (20ns)*/   
#endif /* RD_88E6208 */

#define TIMR01	(UINT32)(TMRD0 | TWR0 | TRFC0 | TRP0 | TXSR0 | TRAS0 | TRCD0)

/* DEVICE 1 Defualt Registers */
/* Device 1 is assumed to be Q-Flash in the DB_88E6218 */
/* Device 1 is Unplaced in the RD_88E6208 */

/* Configuration Register */
#define BAR1							((UINT32)0xef << 24)	/* Defualt base address */
#define BARMK1							((UINT32)0xff << 16)  /* Base address mask register - window size = 4MByte*/			
#define KRO1							((UINT32)0x0 << 12) /* keep row closed */
#define PEN1							((UINT32)0x0 << 11) /* parity error disable */
#define PACK_EN1						((UINT32)0x1 << 7) /* Enable pack to 32b*/
#define MEMORY_TYPE1					((UINT32)0x0 << 3) /* FLASH */						

#ifdef RD_88E6208
#define WS1								((UINT32)0x1 << 13) /* Total bus width = x8 bit*/
#define BYTE_MODE1						((UINT32)0x0 << 9) /* byte mode = x8b - Flash only*/
#define BUS_WIDTH1						((UINT32)0x1 << 1) /* 8 bit bus width for single debvice */						
#define CS1En							((UINT32)0x0) /* Chip Select 1 is Disable */						
#else /*DB_88E6218 */
#define WS1								((UINT32)0x3 << 13) /* Total bus width = x32 bit*/
#define BYTE_MODE1						((UINT32)0x1 << 9) /* byte mode = x16b - Flash only*/
#define BUS_WIDTH1						((UINT32)0x3 << 1) /* 32 bit bus width for single debvice */						
#define CS1En							((UINT32)0x1) /* Chip Select 1 is Enabled */						
#endif /* RD_88E6208 */

#define CFGR1 (UINT32)(BAR1 | BARMK1 | KRO1 | PEN1 |PACK_EN1 | MEMORY_TYPE1 | CS1En | WS1 | BYTE_MODE1 | BUS_WIDTH1)


/* TIMR10 register - FLASH*/

#ifdef RD_88E6208
#define TWB1							((UINT32)0x9 << 24) /* 9 clock cycles delay needed @ 100Mhz (90ns)*/
#define TWPH1							((UINT32)0x3 << 20) /* 3 clock cycles delay needed @ 100Mhz (30ns)*/
#define TWH1							((UINT32)0x1 << 16) /* 1 clock cycles delay needed @ 100Mhz (10ns)*/
#define TWP1							((UINT32)0x7 << 12) /* 7 clock cycles delay needed @ 100Mhz (70ns)*/
#define TOD1							((UINT32)0x6 << 8)  /* 6 clock cycles delay needed @ 100Mhz (55ns)*/
#define TAA1							((UINT32)0xb) 		/* 11 clock cycles delay needed @ 100Mhz (110ns)*/

#else /* DB_88E6218 */
#define TWB1							((UINT32)0xc << 24) /* 12 clock cycles delay needed @ 125Mhz (90ns)*/
#define TWPH1							((UINT32)0x4 << 20) /* 3 clock cycles delay needed @ 125Mhz (30ns)*/
#define TWH1							((UINT32)0x2 << 16) /* 2 clock cycles delay needed @ 125Mhz (10ns)*/
#define TWP1							((UINT32)0x9 << 12) /* 9 clock cycles delay needed @ 125Mhz (70ns)*/
#define TOD1							((UINT32)0x7 << 8)  /* 7 clock cycles delay needed @ 125Mhz (55ns)*/
#define TAA1							((UINT32)0xe) 		/* 14 clock cycles delay needed @ 125Mhz (110ns)*/
#endif /* RD_88E6208 */

#define TIMR10 (UINT32)(TWB1 | TWPH1 | TWH1 | TWP1 | TOD1 |TAA1)

/* There is no need to config TIMR11 */

/* DEVICE 2 Defualt Registers */
/* Device 2 is assumed to be SuperIO / UART in the DB_88E6218 */
/* Device 2 is Unplaced in the RD_88E6208 */


/* NOTE - The parameters for device 2 are temporary */
/* Configuration Register */
#define BAR2							((UINT32)0x0 << 24)	/* Defualt base address */
#define BARMK2							((UINT32)0x0 << 16)  /* Base address mask register - window size = 4MByte*/			
#define WS2								((UINT32)0x0 << 13) /* Total bus width = x32 bit*/
#define KRO2							((UINT32)0x0 << 12) /* keep row closed */
#define PEN2							((UINT32)0x0 << 11) /* parity error disable */
#define BYTE_MODE2						((UINT32)0x0 << 9) /* byte mode = x16b - Flash only*/
#define PACK_EN2						((UINT32)0x0 << 7) /* Enable pack to 32b*/
#define MEMORY_TYPE2					((UINT32)0x0 << 3) /* FLASH */						
#define BUS_WIDTH2						((UINT32)0x0 << 1) /* 32 bit bus width for single debvice */						

#ifdef RD_88E6208
#define CS2En							((UINT32)0x0) /* Chip Select 2 is Disable */						
#else /*DB_88E6218 */
#define CS2En							((UINT32)0x1) /* Chip Select 2 is Enabled */						
#endif /* RD_88E6208 */

#define CFGR2 (UINT32)(BAR2 | BARMK2 | KRO2 | PEN2 |PACK_EN2 | MEMORY_TYPE2 | CS2En | WS2 | BYTE_MODE2 | BUS_WIDTH2)


/* TIMR20 register - FLASH*/

#ifdef RD_88E6208
#define TWB2							((UINT32)0x9 << 24) /* 9 clock cycles delay needed @ 100Mhz (90ns)*/
#define TWPH2							((UINT32)0x3 << 20) /* 3 clock cycles delay needed @ 100Mhz (30ns)*/
#define TWH2							((UINT32)0x1 << 16) /* 1 clock cycles delay needed @ 100Mhz (10ns)*/
#define TWP2							((UINT32)0x7 << 12) /* 7 clock cycles delay needed @ 100Mhz (70ns)*/
#define TOD2							((UINT32)0x6 << 8)  /* 6 clock cycles delay needed @ 100Mhz (55ns)*/
#define TAA2							((UINT32)0xb) 		/* 11 clock cycles delay needed @ 100Mhz (110ns)*/

#else /* DB_88E6218 */
#define TWB2							((UINT32)0xc << 24) /* 12 clock cycles delay needed @ 125Mhz (90ns)*/
#define TWPH2							((UINT32)0x4 << 20) /* 3 clock cycles delay needed @ 125Mhz (30ns)*/
#define TWH2							((UINT32)0x2 << 16) /* 2 clock cycles delay needed @ 125Mhz (10ns)*/
#define TWP2							((UINT32)0x9 << 12) /* 9 clock cycles delay needed @ 125Mhz (70ns)*/
#define TOD2							((UINT32)0x7 << 8)  /* 7 clock cycles delay needed @ 125Mhz (55ns)*/
#define TAA2							((UINT32)0xe) 		/* 14 clock cycles delay needed @ 125Mhz (110ns)*/
#endif /* RD_88E6208 */

#define TIMR20 (UINT32)(TWB2 | TWPH2 | TWH2 | TWP2 | TOD2 |TAA2)

/* There is no need to config TIMR21 */

/* DEVICE 3 Defualt Registers */
/* Device 3 is assumed to be Boot Flash in the DB_88E6218 */
/* Device 3 is Q-Flash in the RD_88E6208 */


/* Configuration Register */
#define BAR3							((UINT32)0xc0 << 24)	/* Defualt base address */
#define BARMK3							((UINT32)0xff << 16)  /* Base address mask register - window size = 4MByte*/	
													  /* Acctually in DB_88E6218 < 4MByte */		
#define WS3								((UINT32)0x1 << 13) /* Total bus width = x8 bit*/
#define KRO3							((UINT32)0x0 << 12) /* keep row closed */
#define PEN3							((UINT32)0x0 << 11) /* parity error disable */
#define BYTE_MODE3						((UINT32)0x0 << 9) /* byte mode = x8b - Flash only*/
#define PACK_EN3						((UINT32)0x1 << 7) /* Enable pack to 32b*/
#define MEMORY_TYPE3					((UINT32)0x0 << 3) /* FLASH */						
#define BUS_WIDTH3						((UINT32)0x1 << 1) /* 8 bit bus width for single debvice */						
#define CS3En							((UINT32)0x1) /* Chip Select 3 is Enabled */						

#define CFGR3 (UINT32)(BAR3 | BARMK3 | KRO3 | PEN3 |PACK_EN3 | MEMORY_TYPE3 | CS3En | WS3 | BYTE_MODE3 | BUS_WIDTH3)

/* TIMR30 register - Boot FLASH*/

#ifdef RD_88E6208
#define TWB3							((UINT32)0x9 << 24) /* 9 clock cycles delay needed @ 100Mhz (90ns)*/
#define TWPH3							((UINT32)0x3 << 20) /* 3 clock cycles delay needed @ 100Mhz (30ns)*/
#define TWH3							((UINT32)0x1 << 16) /* 1 clock cycles delay needed @ 100Mhz (10ns)*/
#define TWP3							((UINT32)0x7 << 12) /* 7 clock cycles delay needed @ 100Mhz (70ns)*/
#define TOD3							((UINT32)0x6 << 8)  /* 6 clock cycles delay needed @ 100Mhz (55ns)*/
#define TAA3							((UINT32)0xb) 		/* 11 clock cycles delay needed @ 100Mhz (110ns)*/

#else /* DB_88E6218 */
#define TWB3							((UINT32)0xff << 24) /* 1125 clock cycles delay needed @ 125Mhz (9us)*/
#define TWPH3							((UINT32)0x4 << 20) /* 4 clock cycles delay needed @ 125Mhz (30ns)*/
#define TWH3							((UINT32)0x0 << 16) /* 2 clock cycles delay needed @ 125Mhz (0ns)*/
#define TWP3							((UINT32)0x7 << 12) /* 7 clock cycles delay needed @ 125Mhz (50ns)*/
#define TOD3							((UINT32)0x4 << 8)  /* 4 clock cycles delay needed @ 125Mhz (30ns)*/
#define TAA3							((UINT32)0xf) 		/* 14 clock cycles delay needed @ 125Mhz (120ns)*/
#endif /* RD_88E6208 */

#define TIMR30 (UINT32)(TWB3 | TWPH3 | TWH3 | TWP3 | TOD3 |TAA3)

/* There is no need to config TIMR31 */

#define SDRAM_MODULE_I2C__BASE_ADDR						0xa4 /* 8'b10100100 */
#define SDRAM_MODULE_I2C__SDRAM_WIDTH_ADDR				13
#define SDRAM_MODULE_I2C__MODULE_BANK_DENSITY_ADDR		31

#define _8BIT_SDRAM_WIDTH								0x8
#define _16BIT_SDRAM_WIDTH								0x10
#define _16MB_MODULE_BANK_DENSITY						0x04
#define _32MB_MODULE_BANK_DENSITY						0x08
#define _64MB_MODULE_BANK_DENSITY						0x10

typedef struct 
	{
		UINT32 sdramWidth;
		UINT32 moduleBankDensity;
	} SDRAM_DATA;
		


//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================

/******************************************************************************
 *
 * Name: memoryInitDefualt
 *
 * Description:
 *   Initialize the memory controller with the SDRAM defualt values. 	 	
 *
 * Conditions For Use:
 *   Use this function with 88E6208 Reference design board or 88E6218 Development Board.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
 extern void memoryInitDefualt( void );
	
/******************************************************************************
 *
 * Name: memoryInit
 *
 * Description:
 *   Initialize the memory controller with the SDRAM DETECTED values. 	 	
 *
 * Conditions For Use:
 *   Use this function with  88E6218 Development Board.
 *
 * Arguments:
 *   SDRAM_DATA * - pointer to struct that will hold the deteced data.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
 void memoryInit(SDRAM_DATA * pSdramData);

/******************************************************************************
 *
 * Name: sdramDetect
 *
 * Description:
 *   Detects the SDRAM DIMM module parameters 	 	
 *
 * Conditions For Use:
 *   Use this function with 88E6218 Development Board.
 *
 * Arguments:
 *   SDRAM_DATA * - pointer to struct that will hold the deteced data.
 *
 * Return Value:
 *   UINT16 -   true upon detection.
 		   		false upon miss-detection
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/

 extern UINT16 sdramDetect(SDRAM_DATA *pSdramData);
 
//=============================================================================
//                    PRIVETE PROCEDURES (ANSI Prototypes)
//=============================================================================

/******************************************************************************
 *
 * Name: deviceInit
 *
 * Description:
 *   Initialize the memory controller devices with the  defualt values. 	 	
 *
 * Conditions For Use:
 *   Use this function with 88E6208 Reference design board or 88E6218 Development Board.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
 static void deviceInit( void );


#endif /* _MEM_INIT_H_ */