/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/test.c,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    Parse ARM's bist register format for each of the blocks
 *    (RAM,Tag,Cache)
 *
 * Public Procedures:
 *    test_boot               The routine that does the parse.
 *
 * Private Procedures:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================
#include "MvCore.h"
#include "gpio.h"
#include "test.h"

//=============================================================================
//                         CODED PUBLIC PROCEDURES
//=============================================================================
/******************************************************************************
 *
 * Name: test_boot
 *
 * Description:
 *   Code to handle any testing needed to be performed by the ARM
 *   itself. At the moment it is restricted to bist
 *   interpretation. All other ram blocks in the chip have bist logic
 *   connected through the scan chain.
 *
 * Conditions For Use:
 *   ARM946 specific.
 *   GPIO must be enabled before use.
 *
 * Arguments:
 *    Arg1 (u32 ): u32ramBist	- I&DSRAM bist result. (raw ARM mrc format).
 *    Arg2 (u32 ): u32tagBist	- I&DTAG  bist result.         "
 *    Arg2 (u32 ): u32cacheBist	- I&DCACHE  bist result.       "
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void
test_boot(UINT32 u32ramBist, UINT32 u32tagBist, UINT32 u32cacheBist)
{
    UINT32 u32failures = 0;
    if(u32ramBist & DxRAM_BIST_FAIL) {
	gpwr(DSRAM_MEM | DONEMASK | FAILMASK);
	u32failures++;
    } else {
	gpwr(DSRAM_MEM | DONEMASK);
    }
    if(u32ramBist & IxRAM_BIST_FAIL) {
	gpwr(ISRAM_MEM | DONEMASK | FAILMASK);
	u32failures++;
    } else {
	gpwr(ISRAM_MEM | DONEMASK);
    }
    if(u32tagBist & DxRAM_BIST_FAIL) {
	gpwr(DTAG_MEM | DONEMASK | FAILMASK);
	u32failures++;
    } else {
	gpwr(DTAG_MEM | DONEMASK);
    }
    if(u32tagBist & IxRAM_BIST_FAIL) {
	gpwr(ITAG_MEM | DONEMASK | FAILMASK);
	u32failures++;
    } else {
	gpwr(ITAG_MEM | DONEMASK);
    }
    if(u32cacheBist & DxRAM_BIST_FAIL) {
	gpwr(DCACHE_MEM | DONEMASK | FAILMASK);
	u32failures++;
    } else {
	gpwr(DCACHE_MEM | DONEMASK);
    }
    if(u32cacheBist & IxRAM_BIST_FAIL) {
	gpwr(ICACHE_MEM | DONEMASK | FAILMASK);
	u32failures++;
    } else {
	gpwr(ICACHE_MEM | DONEMASK);
    }
    if(u32failures) {
	gpwr(0xff);	// Tester will show ff on failure
    } else {
	gpwr(0xbf);     // ... and bf on success.
    }
}
