/*
 * Definitions for the 16550 UART device on the XT1000 board.
 */

/* $Id: uart_baruch.h,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $ */

/*
 * Customer ID=1126; Copyright 1999-2000 Tensilica Inc.
 * These coded instructions, statements, and computer programs are
 * Confidential Proprietary Information of Tensilica Inc. and may not be
 * disclosed to third parties or copied in any form, in whole or in part,
 * without the prior written consent of Tensilica Inc.
 */

extern void *UART_BASE;

/* address of serial ports on xt1000. One port is reserved for
   xmon.  The libgloss code should UART */
#define UART	((uart_dev_t *) &UART_BASE)

/* 16550 UART DEVICE REGISTERS
   The xt1000 board align each register to a word but the device only uses
   one byte of the word, the least-significant byte for little-endian and
   the most-significant for big-endian.  If you use byte-access then it
   won't matter.  We provide macros to do that.
*/
struct uart_dev_s {
  union {
    unsigned int rxb;		/* receive buffer, read-only */
    unsigned int txb;		/* transmit buffer, write-only */
    unsigned int dll;		/* divisor, least-significant byte latch,
				   write-only */
  } w0;				/* word 0 */
  union {
    unsigned int ier;		/* interrupt-enable register, write-only */
    unsigned int dlm;		/* divisor, most-significant byte latch,
				 write-only */
  } w1;

  union {
    unsigned int isr;		/* interrupt status register, read-only */
    unsigned int fcr;		/* FIFO control register, write-only */
  } w2;

  unsigned int lcr;		/* line control-register, write-only */
  unsigned int mcr;		/* modem control-regsiter, write-only */
  unsigned int lsr;		/* line status register, read-only */
  unsigned int msr;		/* modem status register, read-only */
  unsigned int scr;		/* scratch regsiter, read/write */
};

#define RXB(u) (0xff&((u)->w0.rxb))
#define TXB(u) (0xff&((u)->w0.txb))
#define DLL(u) (0xff&((u)->w0.dll))
#define IER(u) (0xff&((u)->w1.ier))
#define DLM(u) (0xff&((u)->w1.dlm))
#define ISR(u) (0xff&((u)->w2.isr))
#define FCR(u) (0xff&((u)->w2.fcr))
#define LCR(u) (0xff&((u)->lcr))
#define MCR(u) (0xff&((u)->mcr))
#define LSR(u) (0xff&((u)->lsr))
#define MSR(u) (0xff&((u)->msr))
#define SCR(u) (0xff&((u)->scr))

#define SET_RXB(u,v)(u)->w0.rxb=((unsigned char)(v))
#define SET_TXB(u,v)(u)->w0.txb=((unsigned char)(v))
#define SET_DLL(u,v)(u)->w0.dll=((unsigned char)(v))
#define SET_IER(u,v)(u)->w1.ier=((unsigned char)(v))
#define SET_DLM(u,v)(u)->w1.dlm=((unsigned char)(v))
#define SET_ISR(u,v)(u)->w2.isr=((unsigned char)(v))
#define SET_FCR(u,v)(u)->w2.fcr=((unsigned char)(v))
#define SET_LCR(u,v)(u)->lcr=((unsigned char)(v))
#define SET_MCR(u,v)(u)->mcr=((unsigned char)(v))
#define SET_LSR(u,v)(u)->lsr=((unsigned char)(v))
#define SET_MSR(u,v)(u)->msr=((unsigned char)(v))
#define SET_SCR(u,v)(u)->scr=((unsigned char)(v))

typedef volatile struct uart_dev_s uart_dev_t;

/* IER bits */
#define RCVR_HOLD_REG_INTENABLE 0x01
#define XMIT_HOLD_REG_INTENABLE    0x02
#define RCVR_STATUS_INTENABLE   0x04
#define MODEM_STATUS_INTENABLE     0x08

/* FCR bits */
#define FIFO_ENABLE      0x01
#define RCVR_FIFO_RESET  0x02
#define XMIT_FIFO_RESET  0x04
#define DMA_MODE_SELECT  0x08
#define RCVR_TRIGGER_LSB 0x40
#define RCVR_TRIGGER_MSB 0x80

/* ISR bits */
#define INT_STATUS(r)   ((r)&1)
#define INT_PRIORITY(r) (((r)>>1)&0x7)

/* LCR bits */
#define WORD_LENGTH(n)  (((n)-5)&0x3)
#define STOP_BIT_ENABLE 0x04
#define PARITY_ENABLE   0x08
#define EVEN_PARITY     0x10
#define FORCE_PARITY    0x20
#define XMIT_BREAK      0x40
#define DLAB_ENABLE     0x80

/* MCR bits */
#define DTR 0x01
#define RTS 0x02
#define OP1 0x04
#define OP2 0x08
#define LOOP_BACK 0x10

/* LSR Bits */
#define RCVR_DATA_READY 0x01
#define OVERRUN_ERROR   0x02
#define PARITY_ERROR    0x04
#define FRAMING_ERROR   0x08
#define BREAK_INTERRUPT 0x10
#define XMIT_HOLD_EMPTY 0x20
#define XMIT_EMPTY      0x40
#define FIFO_ERROR      0x80
#define RCVR_READY(n)   (LSR(u)&RCVR_DATA_READY)
#define XMIT_READY(n)   (LSR(u)&XMIT_HOLD_EMPTY)

/* MSR bits */
#define RDR       0x01
#define DELTA_DSR 0x02
#define DELTA_RI  0x04
#define DELTA_CD  0x08
#define CTS       0x10
#define DSR       0x20
#define RI        0x40
#define CD        0x80

/*
 * Local Variables:
 * mode:c
 * fill-column: 79
 * comment-column: 0
 * c-file-style: "gnu"
 * End:
 */
