/*
** program to flash factory default settings
** 
*/
/*----- Types and Constants Definition ----------------------*/
#define PACK   __attribute__ ((packed))

typedef unsigned char        uint8;
typedef signed   char        int8;
typedef unsigned short       uint16;
typedef signed   short       int16;
typedef unsigned int         uint32;
typedef signed   int         int32;

#define MIB_AP_VERSION 2


/* AP configuration data structure */
typedef struct mib_AP_t
{
   uint8   mibVersion;      // MIB version number 1..N
   uint8   PrivInvoked;     // Boolean
   uint8   PrivOption;   
   uint8   DesiredSsId[32]; // 32 byte string
   uint8   OpRateSet[126];  // 126 byte string
   uint8   DtimPeriod;      // 1 to 255
   //16 bit alignment before unit16
   uint16  BcnPeriod;       // 1 to 65535
   uint16  RtsThresh;       // 0 to 2347
   uint16  FragThresh;      // 256 to 2346
   uint8   RtsRetryLim;     // 1 to 255
   uint8   AuthType;
   uint8   AuthEnable;      // boolean
   uint8   CurrRFChan;      // 0 to 14
   //32 bit alignment before uint32
   uint32  WepType; //0=not_set, 1=40_bit, 2=104_bit
   uint8   WepDefaultKeyValue1[13]; 
   uint8   WepDefaultKeyValue2[13];
   uint8   WepDefaultKeyValue3[13];
   uint8   WepDefaultKeyValue4[13];
   uint8   DefaultWepKeyindex;
   uint8   CurrTxPwrLevel;    // 1 to 8
   //alignment
   uint16  TxPwrLevel1;       // 0 to 10000
   uint16  TxPwrLevel2;       // 0 to 10000
   uint16  TxPwrLevel3;       // 0 to 10000
   uint16  TxPwrLevel4;       // 0 to 10000
   uint16  TxPwrLevel5;       // 0 to 10000
   uint16  TxPwrLevel6;       // 0 to 10000
   uint16  TxPwrLevel7;       // 0 to 10000
   uint16  TxPwrLevel8;       // 0 to 10000
   uint8   CurrTxAnt;         // 1 to 255
   uint8   CurrRxAnt;         // 1 to 255
   uint8   rxAntenna;
   uint8   txAntenna;
   uint8   PreAmble;
   uint8   broadcastssid;     // SSID blanking, true or false
   uint8   EnableDHCP;        // True or False
   uint8   txDataRate;
   uint8   mib_wep;           //special mode
   uint8   wipodMode;         // WiPod 5-in-1 mode 
   //alignment
   uint32  IPAddr;            // IP address
   uint32  SubnetMask;        // Subnet Mask
   uint32  GwyAddr;           // Gateway address
   uint8   EthMACAddr[6];     // Eth MAC addr
   uint8   RFMACAddr[6];      // RF MAC addr
   //alignment
   uint8   username[32];      // User name
   uint8   password[32];      // password
   uint8   devName[16];
   uint8   apmode;
}PACK mib_AP_t;  

/* mfg data struct*/
typedef struct mfg_data_t
{
    uint8   BrdDscrpt[8];      // 8 byte ASCII to descript the type and version of the board
    uint8   Rev;
    uint8   PAOptions;
    uint8   ExtPA;
    uint8   Ant;
    uint16  IntPA[14];
    uint8   CCA[8];
    uint16  Domain;            // Wireless domain
    uint16  CstmrOpts;
    uint8   Reserved_1[10];
    uint16  ChkSum;
    uint8   MfgMacAddr[6];     // Mfg mac address
    uint8   Reserved_2[4];
    uint8   PID[58];           // Production ID
}PACK mfg_data_t;
/* Make sizeof(mib_AP_t) 16-bit aligned to simplify writing to flash */
#define SIZEOF_MIB_AP (((sizeof(mib_AP_t)+1)/2)*2)

/*--------------------------------------------*/
/* Various sizes used in IEEE 802.11 messages */
/*--------------------------------------------*/
#define IEEEtypes_ADDRESS_SIZE         6
#define IEEEtypes_BITMAP_SIZE          251
#define IEEEtypes_CHALLENGE_TEXT_SIZE  253
#define IEEEtypes_CHALLENGE_TEXT_LEN   128
#define IEEEtypes_MAX_DATA_RATES       8
#define IEEEtypes_MAX_DATA_BODY_LEN    2312
#define IEEEtypes_MAX_MGMT_BODY_LEN    2312
#define IEEEtypes_SSID_SIZE            32
#define IEEEtypes_TIME_STAMP_SIZE      8
#define IEEEtypes_MAX_CHANNELS         14
#define IEEEtypes_MAX_BSS_DESCRIPTS    10

typedef enum {
  AUTH_OPEN_SYSTEM = 0,
  AUTH_SHARED_KEY,
  AUTH_NOT_SUPPORTED
} IEEEtypes_AuthType_e;

#define WEP_TYPE_40 1
#define WEP_TYPE_104 2

typedef enum {
  AP_ANT_DIVERSITY,
  AP_ANT_A,
  AP_ANT_B
}ANTENNA_TYPE;

typedef enum {
  PREAMBLE_SHORT,
  PREAMBLE_LONG,
  PREAMBLE_AUTO_SELECT
}PREAMBLE_TYPE;

typedef enum
{
	AP_MODE_B_ONLY,
	AP_MODE_G_ONLY,
	AP_MODE_MIXED,
}AP_ERP_MODE;
#define TRUE 1
#define FALSE 0

char *DefaultSSID = "MarvellAP3X";

char *DefaultUsername = "admin";
char *DefaultPassword = "password";

char Eth_MAC_Address[6] = {
  0x00, 0x12, 0x34, 0x56, 0x78, 0x9A
};

char RF_MAC_Address[6] = {
  0x00, 0x21, 0x43, 0x65, 0x87, 0xA9
};

uint8
util_CopyList(uint8 *Dest_p, uint8 *Src_p, uint32 MaxSize)
{
  uint32  i = 0;

  while (i < MaxSize  &&  Src_p[i] != '\0') {
    Dest_p[i] = Src_p[i];
    i++;
  }

  if (i < MaxSize) {
    Dest_p[i] = '\0';
  }

  return(i);

} // End util_CopyList()

int
SetFactoryConfig(void *factorySettings)
{
  mib_AP_t *mib;
  int i;
  int rc;

  mib = (mib_AP_t *)factorySettings;

      memset((char *)mib, 0, SIZEOF_MIB_AP);

      mib->mibVersion = MIB_AP_VERSION;
  mib->apmode = AP_MODE_MIXED;
      mib->PrivOption = 0;
      mib->PrivInvoked = 0;

      util_CopyList((uint8 *)(mib->DesiredSsId), (uint8 *)DefaultSSID, IEEEtypes_SSID_SIZE);
      util_CopyList((uint8 *)(mib->username), (uint8 *)DefaultUsername, 32);
      util_CopyList((uint8 *)(mib->password), (uint8 *)DefaultPassword, 32);
      mib->OpRateSet[0] = 0x82;
      mib->OpRateSet[1] = 0x84;
      mib->OpRateSet[2] = 0x8B;
      mib->OpRateSet[3] = 0x96;
      mib->OpRateSet[4] = 12;
      mib->OpRateSet[5] = 18;
      mib->OpRateSet[6] = 24;
      mib->OpRateSet[7] = 36;
      mib->OpRateSet[8] = 48;
      mib->OpRateSet[9] = 72;
      mib->OpRateSet[10] = 96;
      mib->OpRateSet[11] = 108;
      mib->OpRateSet[12] = 0;
      mib->OpRateSet[13] = 0;

      mib->BcnPeriod = 100;
      mib->DtimPeriod = 2;
      mib->RtsThresh = 0x92B;
      mib->RtsRetryLim = 0x03;
      mib->FragThresh = 0x92a;
      mib->AuthEnable = 0;
      mib->AuthType = AUTH_OPEN_SYSTEM;

      mib->CurrRFChan = 6;

  mib->WepType = 0;

      mib->CurrTxPwrLevel = 0;
      mib->TxPwrLevel1 = 0;
      mib->TxPwrLevel2 = 0;
      mib->TxPwrLevel3 = 0;
      mib->TxPwrLevel4 = 0;
      mib->TxPwrLevel5 = 0;
      mib->TxPwrLevel6 = 0;
      mib->TxPwrLevel7 = 0;
      mib->TxPwrLevel8 = 0;

      mib->CurrTxAnt = AP_ANT_B; // Ant A or B
      mib->CurrRxAnt = AP_ANT_B; // diversity

  mib->PreAmble = PREAMBLE_AUTO_SELECT;

      mib->broadcastssid = TRUE;

      mib->EnableDHCP = FALSE;
      mib->IPAddr = 0xC0A8000A; /* 192.168.0.10 */
      mib->SubnetMask = 0xFFFFFF00; /* 255.255.255.0 */
      mib->GwyAddr = 0;

      /* default in auto state */
  mib->txDataRate = 0;

      for (i = 0; i < 6; ++i) {
        mib->EthMACAddr[i] = Eth_MAC_Address[i];
        mib->RFMACAddr[i] = RF_MAC_Address[i];
      }

      mib->devName[0] = 0xff;
      mib->devName[15] = 0xff;

      mib->apmode = 2; /* mixed mode */      

  return rc;
}
void MfgInit(void *mfgSetting)
{	
	static int mfgInit = 0;
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	if(!mfgInit)
	{
  		memset((char *)mfg, 0xff, sizeof(mfg_data_t));
		mfgInit = 1;
	}
}

void MfgSetCalibData(void *mfgSetting, char *calibData)
{
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	MfgInit(mfgSetting);
	memcpy(mfg->BrdDscrpt, calibData, 12*8);
}

void MfgSetMacAddr(void *mfgSetting, char *macAddr)
{
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	MfgInit(mfgSetting);
	memcpy(mfg->MfgMacAddr, macAddr, 6);
}

void MfgSetDomain(void *mfgSetting, uint16 domain)
{
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	MfgInit(mfgSetting);
	mfg->Domain = domain;
}