/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/src/api.c,v 1.43 2004/06/15 20:12:15 jshen Exp $
*
* Purpose:
*    This file contains the implementations of the AP API.
*   
*
* Public Procedures:
*
* Private Procedures:
*  
*
* Notes:
*    None.
*
*****************************************************************************/

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "osif.h"
#include "host.h"
#include "sysinfo.h"
#include "timer.h"
#include "ap_mib.h"
#include "smeMain.h"
#include "api2.h"
#include "ap_api.h"
#include "util.h"
#include "ap_mib.h"
#include "memmap.h"
#include "tkip.h"
#include "StaDb.h"
#include "rx80211.h"
#include "wl_macros.h"
#include "stats.h"

#ifndef NO_ECOS_TCP_IP_STACK
#include <network.h>
#endif /* ECOS_TCP_IP_STACK */

#include "oem.h"
#ifdef DOMAIN_SUPPORTED
    #include "domain.h"
#endif

#include "apcfg.h"
#include "apflash.h"

#include "bufmgt.h"
extern QHANDLE mgtCmdQ;
/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */

/* GPIO is accessible through the LBU register space */

#if 1
    #define GPIO_REG(offset) (WL_LBU_GPU_CFG_BASE | offset)

    #define GPIO_REG_SELECT_0_7     WL_LBU_GPU_CFG_BASE
    #define GPIO_REG_SELECT_8_15    GPIO_REG(0x04)
    #define GPIO_REG_OUTPUT_ENABLE  GPIO_REG(0x08)
    #define GPIO_REG_OUTPUTS        GPIO_REG(0x0C)
    #define GPIO_REG_INPUTS         GPIO_REG(0x10)
    #define GPIO_REG_IER			GPIO_REG(0x14)
    #define GPIO_REG_IMR			GPIO_REG(0x18)
    #define GPIO_REG_ISR			GPIO_REG(0x20)
    #define SW_LED_OUTPUT_ENABLE GPIO_REG(0x24)
    #define SW_LED_CYCLE_3_0  GPIO_REG(0x28)
    #define SW_LED_CYCLE_7_4 GPIO_REG(0x2c)
    #define SW_LED_CYCLE_11_8 GPIO_REG(0x30)
    #define SW_LED_CYCLE_15_12 GPIO_REG(0x34)
    #define SW_LED_DUTY_CYCLE_3_0 GPIO_REG(0x38)
    #define SW_LED_DUTY_CYCLE_7_4 GPIO_REG(0x3C)
    #define SW_LED_DUTY_CYCLE_11_8 GPIO_REG(0x40)
    #define SW_LED_DUTY_CYCLE_15_12 GPIO_REG(0x44)
    #define LED_OUTPUT_ENABLE_FOR_STATUS_SIGNALS GPIO_REG(0x48)
    #define LED_STRETCH_DURATION_FOR_STATUS_SIGNALS GPIO_REG(0x4c)
    #define PCLK_CLOCK_DIVIDER_TO_GENERATE_LED_CLK GPIO_REG(0x50)
#endif

#define ETHER_HANDLE 1
#define WLAN_HANDLE 2

#define DS_EVENTS     (APIO_POLL_ETH_DATA_RDY|APIO_POLL_RADIO_DATA_RDY | \
                       APIO_POLL_PROT_STACK_DATA_RDY)

/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */

#ifdef WATCHDOG_SIMULATION
extern UINT8 tmp_cnt;
#endif

extern UINT32 apio_wlanRead(apio_bufdescr_t *ppBufDescr);
extern UINT32 apio_wlanWrite(apio_bufdescr_t *ppBufDescr);
extern UINT32 * apio_GetWlanTxBuf(void);
extern UINT32 * apio_GetEtherTxBuf(void);
extern UINT32 apio_FreeWlanRxBuf(apio_bufdescr_t *pBufDescr);
extern UINT32 apio_FreeEtherRxBuf(apio_bufdescr_t *pBufDescr);
extern UINT32 apio_etherWrite( apio_bufdescr_t *pBufDescr);
extern UINT32 apio_etherRead(apio_bufdescr_t *pBufDescr);
extern UINT16 extStaDb_entries( void );
extern UINT16 extStaDb_list( UINT8 *buf, UINT8 get );
extern WL_STATUS SendDeauthCmd(IEEEtypes_MacAddr_t *MacAddr_p);
extern unsigned long inet_address(unsigned long addr);
extern void initiate_dhcp(int force);
extern void SendResetCmd(int);
extern UINT32 SendStartCmd(void);
/*extern int set_sta_aging_time(int minutes); */
extern void WaitForFlashReady(void);

void Reset(void);

extern char* __version__;

/* Timer alarm */
static short activeAlarmCnt = 0;

#define MAX_TIMER_ALARM 3

/* Memory for the timer alarm object */
typedef struct userTimer
{
    Timer timer;
    UINT8 inuse;
}userTimer;

static userTimer timerAlarm[MAX_TIMER_ALARM];


/* Global interrupt enable status */
static int intStatus = 0;

extern unsigned char *enaddr;
extern UINT8 WepType[];
extern UINT32 WepTypeAll;

#ifdef AP_SITE_SURVEY
/* For Site Survey */
#define SITE_SURVEY_ENTRY_MAX   20
#define DEFAULT_CHAN_MIN        1
#define DEFAULT_CHAN_MAX        14
extern API_SURVEY_ENTRY  siteSurveyInfo[SITE_SURVEY_ENTRY_MAX];
extern UINT8  busyScanning;

extern void SendScanCmd(UINT32 channelMin, UINT32 channelMax);
#endif /* AP_SITE_SURVEY */



/******************************************************************************
 *
 * Name: smeMain_SendEvent
 *
 * Description:
 *   This routine provides the mechanism to trigger an event indicating to
 *   the api called Task that a message has been written to one of its queues.
 *
 * Conditions For Use: 
 *   
 *
 * Arguments:
 *   Arg1 (i  ): Event - value indicating which queue has received a message
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS api_SendEvent( UINT8 Event )
{


    if (Event == api_RESP_RCVD )
    {
#ifdef USE_NEW_OSIF
        os_EventTrigger(apiQH, Event);
#else
        os_EventTrigger(sysinfo_API_RESP_Q_EVENT, Event);
#endif
        return(OS_SUCCESS);
    }
    else
    {
        return(OS_FAIL);
    }

}

/******************************************************************************
 *
 * Name: api_CmdWriteNoBlock
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   Arg1 (i  ): Cmd_p - a pointer to the message to be placed on the
 *                         queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS api_CmdWriteNoBlock( api_Cmd_t *Cmd_p )
{
#if 0
    WL_STATUS status;
#ifdef USE_NEW_OSIF
    if ((status = os_QueueWriteNoBlock(smeQH,
#else
    if ((status = os_QueueWriteNoBlock(sysinfo_SME_HOST_CMD_Q,
#endif
                                       Cmd_p)) == OS_SUCCESS)
    {
        smeMain_SendEvent(smeMain_CMD_RCVD);
        return(OS_SUCCESS);
    }
    else
    {
        return(OS_FAIL);
    }
#endif
}

/******************************************************************************
 *
 * Name: smeQ_ApiRestWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where API responses
 *    are placed for Command originator. If writing
 *   to the queue cannot immediately occur, then the routine returns with
 *   a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): ApiResp_p - a pointer to the message to be placed on the
 *                         queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS api_RespWriteNoBlock( api_Cmd_t *ApiResp_p )
{
#if 0
    WL_STATUS status;
#ifdef USE_NEW_OSIF
    if ((status = os_QueueWriteNoBlock(apiQH,
#else
    if ((status = os_QueueWriteNoBlock(sysinfo_AP_API_RESP_Q,
#endif
                                       ApiResp_p)) == OS_SUCCESS)
    {
        smeMain_SendEvent(api_RESP_RCVD);
        return(OS_SUCCESS);
    }
    else
    {
        return(OS_FAIL);
    }
#endif
}

/******************************************************************************
 *
 * Name: smeQ_ApiRestReadNoBlock
 *
 * Description:
 
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern api_Cmd_t *api_RespReadBlock(void)
{
#if 0
#ifdef USE_NEW_OSIF
    return((api_Cmd_t *) os_QueueReadNoBlock(apiQH));
#else
    return((api_Cmd_t *) os_QueueReadNoBlock(sysinfo_AP_API_RESP_Q));
#endif
#endif
}
/******************************************************************************
 *
 * Name:  
 *
 * Description:
 *
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern api_Cmd_t *api_GetCmdBuf(void)
{


    return((api_Cmd_t *)msi_wl_AllocPktBuf(mgtCmdQ));


}

#define API_CMD_BSS_START  1
extern void api_CmdProcess(UINT8 *cmd)
{
    api_Cmd_t *ApiCmd_p = (api_Cmd_t *)cmd;
    switch (ApiCmd_p->MsgHdr.Msg)
    {
    case API_CMD_BSS_START:
        SendResetCmd(0);
        break;
    default:
        break;
    }

}


void
SetSdramBurstMode(UINT32 mode)
{
    UINT32 val;

    WL_READ_WORD(SDRAM_BURST_REG,val);
    val = (val & 0xfffffff0) | mode;
    WL_WRITE_WORD(SDRAM_BURST_REG, val);
}


void setBasicRate(UINT8 Mode)
{
    int i;
    if (Mode == AP_MODE_G_ONLY)
    {
        for (i = 0; i< IEEEtypes_MAX_DATA_RATES_G; i++)
        {
            if(mib_StaCfg_p->OpRateSet[i] == 12 || 
               mib_StaCfg_p->OpRateSet[i] == 24 ||
               mib_StaCfg_p->OpRateSet[i] == 48 )
                mib_StaCfg_p->OpRateSet[i] |= 0x80;
        }
    }
    else
    {
        for (i = 0; i< IEEEtypes_MAX_DATA_RATES_G; i++)
        {
            if(mib_StaCfg_p->OpRateSet[i] == (12| 0x80) || 
               mib_StaCfg_p->OpRateSet[i] == (24| 0x80) ||
               mib_StaCfg_p->OpRateSet[i] == (48| 0x80) )
                mib_StaCfg_p->OpRateSet[i] &= 0x7f;
        }
    }
}
/******************************************************************************
 *
 * Name:  apctl
 *
 * Description: This function executes the command indicated in the requested structure
 * 
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/

WL_STATUS apctl(APICMDBUF *apicmdbuf)
{

    api_Cmd_t *apiCmdQueue;
    unsigned int i;
#ifdef DOMAIN_SUPPORTED

    UINT8 minChannel, maxChannel, defaultChannel;
#endif

    static unsigned int set_auth_priv_cnt = 0;

    switch (apicmdbuf->cmdCode)
    {
    case APCMD_BSS_START:
	
        SendResetCmd(0);
/*
        if ((apiCmdQueue = api_GetCmdBuf()) != NULL)
        {
            apiCmdQueue->MsgHdr.Msg = APCMD_BSS_START;
            if (api_CmdWriteNoBlock(apiCmdQueue) == OS_FAIL)
            {
                msi_wl_FreePktBuf(apiCmdQueue);
                apicmdbuf->cmdStatus = APCTL_ERROR;
                return OS_FAIL;
            }
            else
            {*/

                UserConfigModified();
                apicmdbuf->cmdStatus = APCTL_OK;
/*            }
        }
        else
        {
            apicmdbuf->cmdStatus = APCTL_ERROR;
            return OS_FAIL;
        }*/

        break;
    case APCMD_GET_SSID:
        if (apicmdbuf->cmdBodyLen < IEEEtypes_SSID_SIZE)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = IEEEtypes_SSID_SIZE;
            return OS_FAIL;
        }
        else
        {
            util_CopyList(apicmdbuf->cmdBody, mib_StaCfg_p->DesiredSsId, IEEEtypes_SSID_SIZE);
            apicmdbuf->cmdBody[IEEEtypes_SSID_SIZE] = '\0';
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_SSID:
        if (apicmdbuf->cmdBodyLen < IEEEtypes_SSID_SIZE)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = IEEEtypes_SSID_SIZE;
            return OS_FAIL;
        }
        else
        {
            util_CopyList( mib_StaCfg_p->DesiredSsId, apicmdbuf->cmdBody, IEEEtypes_SSID_SIZE);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_GET_OPEN_SSID_EN_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_OPEN_SSID_EN_STATE* apicmdbuf_get_open_ssid_en_state;
            apicmdbuf_get_open_ssid_en_state = (APICMDBUF_GET_OPEN_SSID_EN_STATE *)apicmdbuf;
            apicmdbuf_get_open_ssid_en_state->cmdCode = APCMD_GET_OPEN_SSID_EN_STATE_REPLY;
            apicmdbuf_get_open_ssid_en_state->openSSIDEnabled = mib_broadcastssid;
            apicmdbuf_get_open_ssid_en_state->cmdStatus = APCTL_OK;
        }

        break;



    case APCMD_SET_OPEN_SSID_EN_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_OPEN_SSID_EN_STATE* apicmdbuf_set_open_ssid_en_state;
            apicmdbuf_set_open_ssid_en_state = (APICMDBUF_SET_OPEN_SSID_EN_STATE *)apicmdbuf;
            apicmdbuf_set_open_ssid_en_state->cmdCode = APCMD_SET_OPEN_SSID_EN_STATE_REPLY;
            mib_broadcastssid = apicmdbuf_set_open_ssid_en_state->openSSIDEnabled;
            apicmdbuf_set_open_ssid_en_state->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_GET_RF_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RF_MACADDR* apicmdbuf_get_rf_macaddr;
            apicmdbuf_get_rf_macaddr = (APICMDBUF_GET_RF_MACADDR *)apicmdbuf;
            apicmdbuf_get_rf_macaddr->cmdCode = APCMD_GET_RF_MAC_ADDRESS_REPLY;
            memcpy(apicmdbuf_get_rf_macaddr->physAddr, mib_OpData_p->StaMacAddr, 6);
            apicmdbuf_get_rf_macaddr->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_RF_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RF_MACADDR* apicmdbuf_set_rf_macaddr;
            apicmdbuf_set_rf_macaddr = (APICMDBUF_SET_RF_MACADDR *)apicmdbuf;
            apicmdbuf_set_rf_macaddr->cmdCode = (short)APCMD_SET_RF_MAC_ADDRESS_REPLY;
            memcpy(mib_OpData_p->StaMacAddr, apicmdbuf_set_rf_macaddr->physAddr, 6);
            //memcpy(enaddr, apicmdbuf_set_rf_macaddr->physAddr, 6);
            apicmdbuf_set_rf_macaddr->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_GET_RF_CHANNEL:
        if (apicmdbuf->cmdBodyLen < 18)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 18;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RF_CHANNEL *apicmdbuf_get_rf_channel;
            apicmdbuf_get_rf_channel = (APICMDBUF_GET_RF_CHANNEL *)apicmdbuf;
            apicmdbuf_get_rf_channel->cmdCode = APCMD_GET_RF_CHANNEL_REPLY;
            apicmdbuf_get_rf_channel->rfModulationType = 2;
            apicmdbuf_get_rf_channel->curChanNum = PhyDSSSTable_p->CurrChan; /*RfGetChannel(); */
            apicmdbuf_get_rf_channel->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_RF_CHANNEL:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RF_CHANNEL* apicmdbuf_set_rf_channel;
            apicmdbuf_set_rf_channel = (APICMDBUF_SET_RF_CHANNEL *)apicmdbuf;
#ifdef DOMAIN_SUPPORTED

            if (domainGetInfo(&minChannel, &maxChannel, &defaultChannel))
            {
                apicmdbuf_set_rf_channel->cmdCode = APCMD_SET_RF_CHANNEL_REPLY;
                PhyDSSSTable_p->CurrChan = apicmdbuf_set_rf_channel->curChanNum;
                apicmdbuf_set_rf_channel->cmdStatus = APCTL_OK;
            }
            else
            {
                if ( (apicmdbuf_set_rf_channel->curChanNum > maxChannel) ||
                     (apicmdbuf_set_rf_channel->curChanNum < minChannel) )
                {
                    apicmdbuf_set_rf_channel->cmdCode = APCMD_SET_RF_CHANNEL_REPLY;
                    apicmdbuf_set_rf_channel->cmdStatus = APCTL_ERROR;
                }
                else
                {
                    apicmdbuf_set_rf_channel->cmdCode = APCMD_SET_RF_CHANNEL_REPLY;
                    PhyDSSSTable_p->CurrChan = apicmdbuf_set_rf_channel->curChanNum;
                    apicmdbuf_set_rf_channel->cmdStatus = APCTL_OK;
                }
            }
#else
            apicmdbuf_set_rf_channel->cmdCode = APCMD_SET_RF_CHANNEL_REPLY;
            PhyDSSSTable_p->CurrChan = apicmdbuf_set_rf_channel->curChanNum;
            apicmdbuf_set_rf_channel->cmdStatus = APCTL_OK;
#endif

        }

        break;

    case APCMD_GET_BASIC_DATA_RATES:
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_BASIC_DATA_RATES * apicmdbuf_get_basic_data_rates;
            apicmdbuf_get_basic_data_rates = (APICMDBUF_GET_BASIC_DATA_RATES *)apicmdbuf;
            apicmdbuf_get_basic_data_rates->cmdCode = APCMD_GET_BASIC_DATA_RATES;
            apicmdbuf_get_basic_data_rates->basicDataRates[0] = mib_StaCfg_p->OpRateSet[0];
            apicmdbuf_get_basic_data_rates->basicDataRates[1] = mib_StaCfg_p->OpRateSet[1];
            apicmdbuf_get_basic_data_rates->basicDataRates[2] = mib_StaCfg_p->OpRateSet[2];
            apicmdbuf_get_basic_data_rates->basicDataRates[3] = mib_StaCfg_p->OpRateSet[3];
            apicmdbuf_get_basic_data_rates->basicDataRates[4] = mib_StaCfg_p->OpRateSet[4];
            apicmdbuf_get_basic_data_rates->basicDataRates[5] = mib_StaCfg_p->OpRateSet[5];
            apicmdbuf_get_basic_data_rates->basicDataRates[6] = mib_StaCfg_p->OpRateSet[6];
            apicmdbuf_get_basic_data_rates->basicDataRates[7] = mib_StaCfg_p->OpRateSet[7];
            apicmdbuf_get_basic_data_rates->basicDataRates[8] = mib_StaCfg_p->OpRateSet[8];
            apicmdbuf_get_basic_data_rates->basicDataRates[9] = mib_StaCfg_p->OpRateSet[9];
            apicmdbuf_get_basic_data_rates->basicDataRates[10] = mib_StaCfg_p->OpRateSet[10];
            apicmdbuf_get_basic_data_rates->basicDataRates[11] = mib_StaCfg_p->OpRateSet[11];
            apicmdbuf_get_basic_data_rates->basicDataRates[12] = mib_StaCfg_p->OpRateSet[12];
            apicmdbuf_get_basic_data_rates->basicDataRates[13] = mib_StaCfg_p->OpRateSet[13];

            apicmdbuf_get_basic_data_rates->cmdStatus = APCTL_OK;

        }

        break;
    case APCMD_SET_BASIC_DATA_RATES:
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_BASIC_DATA_RATES * apicmdbuf_set_basic_data_rates;
            apicmdbuf_set_basic_data_rates = (APICMDBUF_SET_BASIC_DATA_RATES *)apicmdbuf;

            apicmdbuf_set_basic_data_rates->cmdCode = APCMD_SET_BASIC_DATA_RATES_REPLY;
            mib_StaCfg_p->OpRateSet[0] = apicmdbuf_set_basic_data_rates->basicDataRates[0];
            mib_StaCfg_p->OpRateSet[1] = apicmdbuf_set_basic_data_rates->basicDataRates[1];
            mib_StaCfg_p->OpRateSet[2] = apicmdbuf_set_basic_data_rates->basicDataRates[2];
            mib_StaCfg_p->OpRateSet[3] = apicmdbuf_set_basic_data_rates->basicDataRates[3];
            mib_StaCfg_p->OpRateSet[4] = apicmdbuf_set_basic_data_rates->basicDataRates[4];
            mib_StaCfg_p->OpRateSet[5] = apicmdbuf_set_basic_data_rates->basicDataRates[5];
            mib_StaCfg_p->OpRateSet[6] = apicmdbuf_set_basic_data_rates->basicDataRates[6];
            mib_StaCfg_p->OpRateSet[7] = apicmdbuf_set_basic_data_rates->basicDataRates[7];
            mib_StaCfg_p->OpRateSet[8] = apicmdbuf_set_basic_data_rates->basicDataRates[8];
            mib_StaCfg_p->OpRateSet[9] = apicmdbuf_set_basic_data_rates->basicDataRates[9];
            mib_StaCfg_p->OpRateSet[10] = apicmdbuf_set_basic_data_rates->basicDataRates[10];
            mib_StaCfg_p->OpRateSet[11] = apicmdbuf_set_basic_data_rates->basicDataRates[11];
            mib_StaCfg_p->OpRateSet[12] = apicmdbuf_set_basic_data_rates->basicDataRates[12];
            mib_StaCfg_p->OpRateSet[13] = apicmdbuf_set_basic_data_rates->basicDataRates[13];
#ifdef CONDOR
            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_ERROR;
#else
            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_OK;
#endif
        }

        break;
    case APCMD_GET_BASIC_DATA_RATES_G:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_BASIC_DATA_RATES_G * apicmdbuf_get_basic_data_rates;
            apicmdbuf_get_basic_data_rates= (APICMDBUF_GET_BASIC_DATA_RATES_G  *)apicmdbuf;
            apicmdbuf_get_basic_data_rates->cmdCode = APCMD_GET_BASIC_DATA_RATES_G;
            for ( i = 0; i <16; i++)
            {
                apicmdbuf_get_basic_data_rates->basicDataRates[i]=
                mib_StaCfg_p->OpRateSet[i];
            }
            apicmdbuf_get_basic_data_rates->cmdStatus = APCTL_OK; 

        }

        break;
    case APCMD_SET_BASIC_DATA_RATES_G:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_BASIC_DATA_RATES_G * apicmdbuf_set_basic_data_rates;
            apicmdbuf_set_basic_data_rates= (APICMDBUF_SET_BASIC_DATA_RATES_G  *)apicmdbuf;
#if 0
            apicmdbuf_set_basic_data_rates->cmdCode = APCMD_SET_BASIC_DATA_RATES_G_REPLY;
            if (apicmdbuf_set_basic_data_rates->basicDataRates[0]==0x82)
                mib_StaCfg.OpRateSet[0]|=0x80;
            if (apicmdbuf_set_basic_data_rates->basicDataRates[1]==0x84)
                mib_StaCfg.OpRateSet[1]|=0x80;
            if (apicmdbuf_set_basic_data_rates->basicDataRates[2]==0x0B)
                mib_StaCfg.OpRateSet[2]|=0x80;
            if (apicmdbuf_set_basic_data_rates->basicDataRates[3]==0x16)
                mib_StaCfg.OpRateSet[3]|=0x80;
            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_OK; 
#else
            apicmdbuf_set_basic_data_rates->cmdCode = APCMD_SET_BASIC_DATA_RATES_G_REPLY;
            for (i=0;i < 16; i++)
            {
                mib_StaCfg_p->OpRateSet[i]=apicmdbuf_set_basic_data_rates->basicDataRates[i];
            }

            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_OK; 
#endif
        }

        break;
    case APCMD_GET_SUP_DATA_RATES:
#if 0
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_GET_SUP_DATA_RATE * apicmdbuf_get_sup_data_rates;
            apicmdbuf_get_sup_data_rates = (APICMDBUF_GET_SUP_DATA_RATE *)apicmdbuf;
            apicmdbuf_get_sup_data_rates->cmdCode = APCMD_GET_SUP_DATA_RATES_REPLY;

            apicmdbuf_get_sup_data_rates->txDataRate[0] = mib_StaCfg_p->OpRateSet[0];
            apicmdbuf_get_sup_data_rates->txDataRate[1] = mib_StaCfg_p->OpRateSet[1];
            apicmdbuf_get_sup_data_rates->txDataRate[2] = mib_StaCfg_p->OpRateSet[2];
            apicmdbuf_get_sup_data_rates->txDataRate[3] = mib_StaCfg_p->OpRateSet[3];
            apicmdbuf_get_sup_data_rates->txDataRate[4] = mib_StaCfg_p->OpRateSet[4];
            apicmdbuf_get_sup_data_rates->txDataRate[5] = mib_StaCfg_p->OpRateSet[5];
            apicmdbuf_get_sup_data_rates->txDataRate[6] = mib_StaCfg_p->OpRateSet[6];
            apicmdbuf_get_sup_data_rates->txDataRate[7] = mib_StaCfg_p->OpRateSet[7];
            apicmdbuf_get_sup_data_rates->cmdStatus = APCTL_OK;

        }
#endif
        break;
    case APCMD_SET_SUP_DATA_RATES:
#if 0

        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {


            APICMDBUF_SET_SUP_DATA_RATE * apicmdbuf_set_sup_data_rates;

            apicmdbuf_set_sup_data_rates = (APICMDBUF_SET_SUP_DATA_RATE *)apicmdbuf;

            apicmdbuf_set_sup_data_rates->cmdCode = APCMD_SET_SUP_DATA_RATES_REPLY;
            mib_StaCfg_p->OpRateSet[0] = apicmdbuf_set_sup_data_rates->txDataRate[0];
            mib_StaCfg_p->OpRateSet[1] = apicmdbuf_set_sup_data_rates->txDataRate[1];
            mib_StaCfg_p->OpRateSet[2] = apicmdbuf_set_sup_data_rates->txDataRate[2];
            mib_StaCfg_p->OpRateSet[3] = apicmdbuf_set_sup_data_rates->txDataRate[3];
            mib_StaCfg_p->OpRateSet[4] = apicmdbuf_set_sup_data_rates->txDataRate[4];
            mib_StaCfg_p->OpRateSet[5] = apicmdbuf_set_sup_data_rates->txDataRate[5];
            mib_StaCfg_p->OpRateSet[6] = apicmdbuf_set_sup_data_rates->txDataRate[6];
            mib_StaCfg_p->OpRateSet[7] = apicmdbuf_set_sup_data_rates->txDataRate[7];



            apicmdbuf_set_sup_data_rates->cmdStatus = APCTL_OK;
        }
#endif
        break;
    case APCMD_GET_TX_POWER_LEVEL:
        if (apicmdbuf->cmdBodyLen < 9)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 9;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_POWER_LEVEL * apicmdbuf_get_tx_power_level;
            apicmdbuf_get_tx_power_level = (APICMDBUF_GET_TX_POWER_LEVEL *)apicmdbuf;
            apicmdbuf_get_tx_power_level->cmdCode = APCMD_GET_TX_POWER_LEVEL_REPLY;
            apicmdbuf_get_tx_power_level->txPowerLevel = mib_PhyTxPwrData_p->CurrTxPwrLevel;
            apicmdbuf_get_tx_power_level->powerLevelList[0] = mib_PhyTxPwrData_p->TxPwrLevel1;
            apicmdbuf_get_tx_power_level->powerLevelList[1] = mib_PhyTxPwrData_p->TxPwrLevel2;
            apicmdbuf_get_tx_power_level->powerLevelList[2] = mib_PhyTxPwrData_p->TxPwrLevel3;
            apicmdbuf_get_tx_power_level->powerLevelList[3] = mib_PhyTxPwrData_p->TxPwrLevel4;
            apicmdbuf_get_tx_power_level->powerLevelList[4] = mib_PhyTxPwrData_p->TxPwrLevel5;
            apicmdbuf_get_tx_power_level->powerLevelList[5] = mib_PhyTxPwrData_p->TxPwrLevel6;
            apicmdbuf_get_tx_power_level->powerLevelList[6] = mib_PhyTxPwrData_p->TxPwrLevel7;
            apicmdbuf_get_tx_power_level->powerLevelList[7] = mib_PhyTxPwrData_p->TxPwrLevel8;
            apicmdbuf_get_tx_power_level->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_TX_POWER_LEVEL:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_POWER_LEVEL * apicmdbuf_set_tx_power_level;
            apicmdbuf_set_tx_power_level = (APICMDBUF_SET_TX_POWER_LEVEL *)apicmdbuf;
            apicmdbuf_set_tx_power_level->cmdCode = APCMD_SET_TX_POWER_LEVEL_REPLY;
            mib_PhyTxPwrData_p->CurrTxPwrLevel = apicmdbuf_set_tx_power_level->txPowerLevel;
            apicmdbuf_set_tx_power_level->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_RF_PREAMBLE_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMBUF_GET_RF_PREAMBLE_OPTION * apicmdbuf_get_rf_preamble_option;
            apicmdbuf_get_rf_preamble_option = (APICMBUF_GET_RF_PREAMBLE_OPTION *)apicmdbuf;
            apicmdbuf_get_rf_preamble_option->cmdCode = APCMD_GET_RF_PREAMBLE_OPTION_REPLY;
            apicmdbuf_get_rf_preamble_option->rfPreambleOption = mib_preAmble;
            apicmdbuf_get_rf_preamble_option->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_RF_PREAMBLE_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMBUF_SET_RF_PREAMBLE_OPTION * apicmdbuf_set_rf_preamble_option;
            apicmdbuf_set_rf_preamble_option = (APICMBUF_SET_RF_PREAMBLE_OPTION *)apicmdbuf;
            apicmdbuf_set_rf_preamble_option->cmdCode = APCMD_SET_RF_PREAMBLE_OPTION_REPLY;
            mib_preAmble = apicmdbuf_set_rf_preamble_option->rfPreambleOption;
            apicmdbuf_set_rf_preamble_option->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_ANTENNA_MODE:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_ANTENNA_MODE* apicmdbuf_get_antenna_mode;
            apicmdbuf_get_antenna_mode = (APICMDBUF_GET_ANTENNA_MODE *)apicmdbuf;
            apicmdbuf_get_antenna_mode->cmdCode = APCMD_GET_ANTENNA_MODE_REPLY;
            apicmdbuf_get_antenna_mode->rxAntennaMode = mib_rxAntenna;
            apicmdbuf_get_antenna_mode->txAntennaMode = mib_txAntenna;
            apicmdbuf_get_antenna_mode->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_ANTENNA_MODE:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_ANTENNA_MODE* apicmdbuf_set_antenna_mode;
            apicmdbuf_set_antenna_mode = (APICMDBUF_SET_ANTENNA_MODE *)apicmdbuf;
            apicmdbuf_set_antenna_mode->cmdCode = APCMD_SET_ANTENNA_MODE_REPLY;
            if (apicmdbuf_set_antenna_mode->whichAnt == AP_RX_ANTENNA)
            {
                mib_rxAntenna = apicmdbuf_set_antenna_mode->antennaMode;
            }
            else
            {
                mib_txAntenna = apicmdbuf_set_antenna_mode->antennaMode;
            }
            apicmdbuf_set_antenna_mode->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_FRAG_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_FRAG_THRESHOLD* apicmdbuf_get_frag_threshold;
            apicmdbuf_get_frag_threshold = (APICMDBUF_GET_FRAG_THRESHOLD *)apicmdbuf;
            apicmdbuf_get_frag_threshold->cmdCode = APCMD_GET_FRAG_THRESHOLD_REPLY;
            apicmdbuf_get_frag_threshold->fragThreshold = mib_OpData_p->FragThresh;
            apicmdbuf_get_frag_threshold->cmdStatus = APCTL_OK;

        }
        break;

    case APCMD_SET_FRAG_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_FRAG_THRESHOLD* apicmdbuf_set_frag_threshold;
            apicmdbuf_set_frag_threshold = (APICMDBUF_SET_FRAG_THRESHOLD *)apicmdbuf;
            apicmdbuf_set_frag_threshold->cmdCode = APCMD_SET_FRAG_THRESHOLD_REPLY;
            if(apicmdbuf_set_frag_threshold->fragThreshold < 256)
            {
                mib_OpData_p->FragThresh = 256;
            }
            else if (apicmdbuf_set_frag_threshold->fragThreshold > 2346)
            {
                mib_OpData_p->FragThresh = 2346;
            }
            else
            {
                mib_OpData_p->FragThresh = apicmdbuf_set_frag_threshold->fragThreshold;
            }
            apicmdbuf_set_frag_threshold->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_RTS_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RTS_THRESHOLD* apicmdbuf_get_rts_threshold;
            apicmdbuf_get_rts_threshold = (APICMDBUF_GET_RTS_THRESHOLD *)apicmdbuf;
            apicmdbuf_get_rts_threshold->cmdCode = APCMD_GET_RTS_THRESHOLD_REPLY;
            apicmdbuf_get_rts_threshold->rtsThreshold = mib_OpData_p->RtsThresh;
            apicmdbuf_get_rts_threshold->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_RTS_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RTS_THRESHOLD* apicmdbuf_set_rts_threshold;
            apicmdbuf_set_rts_threshold = (APICMDBUF_SET_RTS_THRESHOLD *)apicmdbuf;
            apicmdbuf_set_rts_threshold->cmdCode = APCMD_SET_RTS_THRESHOLD_REPLY;
            if(apicmdbuf_set_rts_threshold->rtsThreshold < 0)
            {
                mib_OpData_p->RtsThresh = 0;
            }
            else if (apicmdbuf_set_rts_threshold->rtsThreshold > 2347)
            {
                mib_OpData_p->RtsThresh = 2347;
            }
            else
            {
                mib_OpData_p->RtsThresh = apicmdbuf_set_rts_threshold->rtsThreshold;
            }
            apicmdbuf_set_rts_threshold->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_RTS_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RTS_RETRY_LIMIT * apicmdbuf_get_rts_retry_limit;
            apicmdbuf_get_rts_retry_limit = (APICMDBUF_GET_RTS_RETRY_LIMIT *)apicmdbuf;
            apicmdbuf_get_rts_retry_limit->cmdCode = APCMD_GET_RTS_THRESHOLD_REPLY;
            apicmdbuf_get_rts_retry_limit->rtsRetryLimit = mib_OpData_p->ShortRetryLim;
            apicmdbuf_get_rts_retry_limit->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_RTS_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RTS_RETRY_LIMIT * apicmdbuf_set_rts_retry_limit;
            apicmdbuf_set_rts_retry_limit = (APICMDBUF_SET_RTS_RETRY_LIMIT *)apicmdbuf;
            apicmdbuf_set_rts_retry_limit->cmdCode = APCMD_SET_RTS_THRESHOLD_REPLY;
            mib_OpData_p->ShortRetryLim = apicmdbuf_set_rts_retry_limit->rtsRetryLimit;
            apicmdbuf_set_rts_retry_limit->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_DATA_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            apicmdbuf->cmdStatus = APCTL_OK;
            /*Todo (UINT16 *)apicmdbuf->cmdBody =  mib_OpData_p->; */

        }
        break;

    case APCMD_SET_DATA_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            apicmdbuf->cmdStatus = APCTL_OK;
            /*Todo mib_StaCfg.BcnPeriod = (UINT16 *)apicmdbuf->cmdBody; */
        }
        break;

    case APCMD_GET_BEACON_PERIOD:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_BEACON_PERIOD* apicmdbuf_get_beacon_period;
            apicmdbuf_get_beacon_period = (APICMDBUF_GET_BEACON_PERIOD *)apicmdbuf;
            apicmdbuf_get_beacon_period->cmdCode = APCMD_GET_BEACON_PERIOD_REPLY;
            apicmdbuf_get_beacon_period->beaconPeriod = mib_StaCfg_p->BcnPeriod;
            apicmdbuf_get_beacon_period->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_BEACON_PERIOD:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_BEACON_PERIOD* apicmdbuf_set_beacon_period;
            apicmdbuf_set_beacon_period = (APICMDBUF_SET_BEACON_PERIOD *)apicmdbuf;
            apicmdbuf_set_beacon_period->cmdCode = APCMD_SET_BEACON_PERIOD_REPLY;
            if(apicmdbuf_set_beacon_period->beaconPeriod < 1)
            {
                mib_StaCfg_p->BcnPeriod = 1;
            }
            else if (apicmdbuf_set_beacon_period->beaconPeriod > 65535)
            {
                mib_StaCfg_p->BcnPeriod = 65535;
            }
            else
            {
                mib_StaCfg_p->BcnPeriod = apicmdbuf_set_beacon_period->beaconPeriod;
            }
            apicmdbuf_set_beacon_period->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_DTIM_PERIOD:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_DTIM_PERIOD* apicmdbuf_get_dtim_period;
            apicmdbuf_get_dtim_period = (APICMDBUF_GET_DTIM_PERIOD *)apicmdbuf;
            apicmdbuf_get_dtim_period->cmdCode = APCMD_GET_DTIM_PERIOD_REPLY;
            apicmdbuf_get_dtim_period->dtimPeriod = mib_StaCfg_p->DtimPeriod;
            apicmdbuf_get_dtim_period->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_DTIM_PERIOD:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_DTIM_PERIOD* apicmdbuf_set_dtim_period;
            apicmdbuf_set_dtim_period = (APICMDBUF_SET_DTIM_PERIOD *)apicmdbuf;
            apicmdbuf_set_dtim_period->cmdCode = APCMD_SET_DTIM_PERIOD_REPLY;
            if( apicmdbuf_set_dtim_period->dtimPeriod <= 0)
                mib_StaCfg_p->DtimPeriod = 1;
            else if(apicmdbuf_set_dtim_period->dtimPeriod > 255)
                mib_StaCfg_p->DtimPeriod = 255;
            else
                mib_StaCfg_p->DtimPeriod = apicmdbuf_set_dtim_period->dtimPeriod;
            
            apicmdbuf_set_dtim_period->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_802_11_STATS:
        if (apicmdbuf->cmdBodyLen < 60)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 60;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_802_11_STATS* apicmdbuf_get_802_11_stats;
            apicmdbuf_get_802_11_stats = (APICMDBUF_GET_802_11_STATS *)apicmdbuf;
            apicmdbuf_get_802_11_stats->cmdCode = APCMD_GET_802_11_STATS_REPLY;
            apicmdbuf_get_802_11_stats->rxFrmCnt = Stats.DataRecvCnt;
            apicmdbuf_get_802_11_stats->txFrmCnt = Stats.DataTxSuccessCnt;
            Stats.DataRecvCnt = 0;
            Stats.DataTxSuccessCnt = 0;
            apicmdbuf_get_802_11_stats->totaltxBytes = Stats.WlTxDataBytes;
            Stats.WlTxDataBytes = 0;
            apicmdbuf_get_802_11_stats->totalRxBytes = (*(volatile unsigned long *)(0x8000a220));
            /*todo */
            apicmdbuf_get_802_11_stats->cntMcastPackets = 0;
            apicmdbuf_get_802_11_stats->cntUcastPackets = 0; /* count of uniticastpackets */
            apicmdbuf_get_802_11_stats->cntTxBad = 0;
            apicmdbuf_get_802_11_stats->cntRetry = 0;
            apicmdbuf_get_802_11_stats->rtsBadCnt = 0;
            apicmdbuf_get_802_11_stats->ackBadCnt = 0;
            apicmdbuf_get_802_11_stats->rxMcastOkCnt = 0; /*mib_Counters. */
            apicmdbuf_get_802_11_stats->fcsErrorCnt = 0;
            apicmdbuf_get_802_11_stats->duplicateCnt = 0; /*todo */
            apicmdbuf_get_802_11_stats->wepDecryptBadCnt = 0; /*todo */
            apicmdbuf_get_802_11_stats->cmdStatus = APCTL_OK;

        }
        break;

    case APCMD_ADD_WLAN_FILTER_ENTRY:
        /*Todo */
        break;
    case APCMD_GET_STA_LIST:
        {
            UINT16 nostation = extStaDb_entries();
            UINT16 minbodysize = 2 + extStaDb_entries() * sizeof(STA_INFO);
            if (apicmdbuf->cmdBodyLen < minbodysize)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdCode = APCMD_GET_STA_LIST_REPLY;
                apicmdbuf->cmdBodyLen = minbodysize;
                return OS_FAIL;
            }
            else
            {
                /*Number of associated station */
#if defined(AP31E) || defined(AP31CLIENT) || defined (AP_PCI)
                apicmdbuf->cmdBody[1] = nostation / 0x100;
                apicmdbuf->cmdBody[0] = nostation % 0x100;
#else
                apicmdbuf->cmdBody[0] = nostation / 0x100;
                apicmdbuf->cmdBody[1] = nostation % 0x100;
#endif

#ifdef STA_INFO_DB
                extStaDb_list(&apicmdbuf->cmdBody[2], 1);
#else
                extStaDb_list(&apicmdbuf->cmdBody[2], 0);
#endif
                apicmdbuf->cmdCode = APCMD_GET_STA_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_STA_INFO:
        /*Todo */
        break;
    case APCMD_DEAUTH_STA:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_DEAUTH_STA* apicmdbuf_deauth_sta;
            apicmdbuf_deauth_sta = (APICMDBUF_DEAUTH_STA *)apicmdbuf;
            apicmdbuf_deauth_sta->cmdCode = APCMD_DEAUTH_STA_REPLY;
            SendDeauthCmd((IEEEtypes_MacAddr_t *)apicmdbuf_deauth_sta->staMACAddress);
            apicmdbuf_deauth_sta->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_DISASSOC_STA:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_DISASSOC_STA* apicmdbuf_disassoc_sta;
            apicmdbuf_disassoc_sta = (APICMDBUF_DISASSOC_STA *)apicmdbuf;
            apicmdbuf_disassoc_sta->cmdCode = APCMD_DISASSOC_STA_REPLY;
            SendDeauthCmd((IEEEtypes_MacAddr_t *)apicmdbuf_disassoc_sta->staMACAddress);
            apicmdbuf_disassoc_sta->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_ADD_ASSOC_FILTER_ENTRY:
        /*Todo */
        break;
    case APCMD_DEL_ASSOC_FILTER_ENTRY:
        /*Todo */
        break;
    case APCMD_GET_MAX_NUM_STA:
        /*Todo */
        break;
    case APCMD_SET_MAX_NUM_STA:
        /*Todo */
        break;
    case APCMD_GET_DEFAULT_WEP_KEYS:
        if (apicmdbuf->cmdBodyLen < 74)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 74;
            return OS_FAIL;
        }
        else
        {


            APICMDBUF_GET_DEFAULT_WEP_KEYS* apicmdbuf_get_default_wep_keys;
            apicmdbuf_get_default_wep_keys = (APICMDBUF_GET_DEFAULT_WEP_KEYS *)apicmdbuf;
            apicmdbuf_get_default_wep_keys->cmdCode = APCMD_GET_DEFAULT_WEP_KEYS_REPLY;
            apicmdbuf_get_default_wep_keys->keyindex = mib_defaultkeyindex;
            /*check if it is 40 or 104 bit */

            apicmdbuf_get_default_wep_keys->keyType0 = WepType[0];
            memset(apicmdbuf_get_default_wep_keys->keyValue0, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue0, mib_WepDefaultKeys_p[0].WepDefaultKeyValue, 13);

            apicmdbuf_get_default_wep_keys->keyType1 = WepType[1];
            memset(apicmdbuf_get_default_wep_keys->keyValue1, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue1, mib_WepDefaultKeys_p[1].WepDefaultKeyValue, 13);

            apicmdbuf_get_default_wep_keys->keyType2 = WepType[2];
            memset(apicmdbuf_get_default_wep_keys->keyValue2, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue2, mib_WepDefaultKeys_p[2].WepDefaultKeyValue, 13);

            apicmdbuf_get_default_wep_keys->keyType3 = WepType[3];
            memset(apicmdbuf_get_default_wep_keys->keyValue3, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue3, mib_WepDefaultKeys_p[3].WepDefaultKeyValue, 13);


            apicmdbuf_get_default_wep_keys->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_DEFAULT_WEP_KEYS:
        if (apicmdbuf->cmdBodyLen < 74)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 74;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_DEFAULT_WEP_KEYS* apicmdbuf_set_default_wep_keys;
            apicmdbuf_set_default_wep_keys = (APICMDBUF_SET_DEFAULT_WEP_KEYS *)apicmdbuf;
            apicmdbuf_set_default_wep_keys->cmdCode = APCMD_SET_DEFAULT_WEP_KEYS_REPLY;

            mib_defaultkeyindex = apicmdbuf_set_default_wep_keys->keyindex;

            memcpy(mib_WepDefaultKeys_p[0].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue0, 13);
            memcpy(mib_WepDefaultKeys_p[1].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue1, 13);
            memcpy(mib_WepDefaultKeys_p[2].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue2, 13);
            memcpy(mib_WepDefaultKeys_p[3].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue3, 13);
            WepType[0] = apicmdbuf_set_default_wep_keys->keyType0;
            WepType[1] = apicmdbuf_set_default_wep_keys->keyType1;
            WepType[2] = apicmdbuf_set_default_wep_keys->keyType2;
            WepType[3] = apicmdbuf_set_default_wep_keys->keyType3;

            WepTypeAll = WepType[0] | WepType[1] | WepType[2] | WepType[3];
            apicmdbuf_set_default_wep_keys->cmdStatus = APCTL_OK;

            /* Check for no. of keys */
        }
        break;
    case APCMD_SET_STA_WEP_KEY:
        /*todo */
        break;
    case APCMD_GET_PRIVACY_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_PRIVACY_OPTION* apicmdbuf_get_privacy_option;
            apicmdbuf_get_privacy_option = (APICMDBUF_GET_PRIVACY_OPTION*)apicmdbuf;
            apicmdbuf_get_privacy_option->cmdCode = APCMD_GET_PRIVACY_OPTION_REPLY;
            apicmdbuf_get_privacy_option->dataEncrypt = mib_StaCfg_p->PrivOption;
            apicmdbuf_get_privacy_option->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_PRIVACY_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_PRIVACY_OPTION* apicmdbuf_set_privacy_option;
            apicmdbuf_set_privacy_option = (APICMDBUF_SET_PRIVACY_OPTION *)apicmdbuf;
            apicmdbuf_set_privacy_option->cmdCode = APCMD_SET_PRIVACY_OPTION_REPLY;
            mib_StaCfg_p->PrivOption = apicmdbuf_set_privacy_option->dataEncrypt;
            mib_PrivacyTable_p->PrivInvoked = apicmdbuf_set_privacy_option->dataEncrypt;

            set_auth_priv_cnt++;

            if (mib_PrivacyTable_p->PrivInvoked)
            {
                if (set_auth_priv_cnt & 1) /*if this coming before set_auth_mode */
                {
                    ;
                }
                else
                {
                    if (mib_wep==0xff)
                    {
                        mib_AuthAlg_p->Enable = 1;
                        mib_AuthAlg_p->Type = Strict_Open_Mode()?AUTH_OPEN_SYSTEM:AUTH_SHARED_KEY;
                        mib_wep = 3;
                    }
                }
                mib_PrivacyTable_p->RSNEnabled = 0;/*Disable WPA*/
            }
            apicmdbuf_set_privacy_option->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_PSK_PASSPHRASE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_PSK_PASSPHRASE* apicmdbuf_set_psk_passph;
            apicmdbuf_set_psk_passph = (APICMDBUF_SET_PSK_PASSPHRASE *)apicmdbuf;
            apicmdbuf_set_psk_passph->cmdCode = APCMD_SET_PSK_PASSPHRASE_REPLY;
            strcpy(mib_RSNConfig_p->PSKPassPhrase, apicmdbuf_set_psk_passph->passphrase);

            apicmdbuf_set_psk_passph->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_GRP_REKEY_TIME:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_GRP_REKEY_TIME* apicmdbuf_set_grp_rekey_time;
            apicmdbuf_set_grp_rekey_time = (APICMDBUF_SET_GRP_REKEY_TIME *)apicmdbuf;
            apicmdbuf_set_grp_rekey_time->cmdCode = APCMD_SET_GRP_REKEY_TIME_REPLY;
            mib_RSNConfig_p->GroupRekeyTime = apicmdbuf_set_grp_rekey_time->rekey_time;

            apicmdbuf_set_grp_rekey_time->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AUTH_MODE:
        if (apicmdbuf->cmdBodyLen < 1) /** size of UINT16 **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AUTH_MODE* apicmdbuf_get_auth_mode;
            apicmdbuf_get_auth_mode = (APICMDBUF_GET_AUTH_MODE *)apicmdbuf;
            apicmdbuf_get_auth_mode->cmdCode = APCMD_GET_AUTH_MODE_REPLY;
            if (mib_wep == 1)
            {
                apicmdbuf_get_auth_mode->authMode = OPEN_SYSTEM_AND_SHARED_KEY;
            }
            else if (mib_wep == 3)
            {
                apicmdbuf_get_auth_mode->authMode = OPEN_SYSTEM_ONLY;
            }
            else if ( mib_AuthAlg_p->Enable == 0x0 && mib_AuthAlg_p->Type == AUTH_OPEN_SYSTEM)
            {
                apicmdbuf_get_auth_mode->authMode = OPEN_SYSTEM_ONLY;
            }
            else if (mib_AuthAlg_p->Enable == 0x1 && mib_AuthAlg_p->Type == AUTH_SHARED_KEY)
            {
                apicmdbuf_get_auth_mode->authMode = SHARED_KEY_ONLY;
            }
            apicmdbuf_get_auth_mode->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_AUTH_MODE:
        if (apicmdbuf->cmdBodyLen < 1) /** size of UINT16 **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AUTH_MODE* apicmdbuf_set_auth_mode;
            apicmdbuf_set_auth_mode = (APICMDBUF_SET_AUTH_MODE *)apicmdbuf;
            apicmdbuf_set_auth_mode->cmdCode = APCMD_SET_AUTH_MODE_REPLY;

            set_auth_priv_cnt++;

            if (apicmdbuf_set_auth_mode->authMode == OPEN_SYSTEM_ONLY)
            {
                mib_AuthAlg_p->Enable = 0;
                mib_AuthAlg_p->Type = AUTH_OPEN_SYSTEM;
                if (set_auth_priv_cnt & 1)
                { /*if this coming before set_privacy_option */
                    mib_wep = 0xff;
                }
                else if (mib_PrivacyTable_p->PrivInvoked)
                {
                    mib_wep = 3;
                    mib_AuthAlg_p->Enable = 1;
                    mib_AuthAlg_p->Type = Strict_Open_Mode()?AUTH_OPEN_SYSTEM:AUTH_SHARED_KEY;
                }
            }
            else if (apicmdbuf_set_auth_mode->authMode == SHARED_KEY_ONLY)
            {
                mib_AuthAlg_p->Enable = 1;
                mib_AuthAlg_p->Type = AUTH_SHARED_KEY;
                mib_wep = Strict_Share_Mode()?0:2; /*2 - not strict share mode */
                                                   /*0 -strict share mode */
            }
            else if (apicmdbuf_set_auth_mode->authMode == OPEN_SYSTEM_AND_SHARED_KEY)
            {
                mib_AuthAlg_p->Enable = 1;
                mib_AuthAlg_p->Type = 1; /*AUTH_OPEN_OR_SHARED_KEY; */
                mib_wep = 1;
            }
            apicmdbuf_set_auth_mode->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_GET_ETH_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_GET_ETH_MAC_ADDRESS* apicmdbuf_get_eth_mac_address;
            apicmdbuf_get_eth_mac_address = (APICMDBUF_GET_ETH_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_get_eth_mac_address->cmdCode = APCMD_GET_ETH_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                apicmdbuf_get_eth_mac_address->phyAddress[i] = mib_currentmacaddress[i];
            apicmdbuf_get_eth_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_ETH_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_SET_ETH_MAC_ADDRESS* apicmdbuf_set_eth_mac_address;
            apicmdbuf_set_eth_mac_address = (APICMDBUF_SET_ETH_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_set_eth_mac_address->cmdCode = APCMD_SET_ETH_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                mib_currentmacaddress[i] = apicmdbuf_set_eth_mac_address->phyAddress[i];
            apicmdbuf_set_eth_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_WAN_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_GET_WAN_MAC_ADDRESS* apicmdbuf_get_wan_mac_address;
            apicmdbuf_get_wan_mac_address = (APICMDBUF_GET_WAN_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_get_wan_mac_address->cmdCode = APCMD_GET_WAN_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                apicmdbuf_get_wan_mac_address->phyAddress[i] = mib_currentmacaddress[i];
            apicmdbuf_get_wan_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_WAN_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_SET_WAN_MAC_ADDRESS* apicmdbuf_set_wan_mac_address;
            apicmdbuf_set_wan_mac_address = (APICMDBUF_SET_WAN_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_set_wan_mac_address->cmdCode = APCMD_SET_WAN_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                mib_currentmacaddress[i] = apicmdbuf_set_wan_mac_address->phyAddress[i];
            apicmdbuf_set_wan_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_ETH_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 16) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_ETH_IP_CONFIG* apicmdbuf_get_eth_ip_config;
            apicmdbuf_get_eth_ip_config = (APICMDBUF_GET_ETH_IP_CONFIG *)apicmdbuf;
            apicmdbuf_get_eth_ip_config->cmdCode = APCMD_GET_ETH_IP_CONFIG_REPLY;
/*#ifndef AP22 */

            apicmdbuf_get_eth_ip_config->dhcpEnabled = mib_dhcpenable;
            apicmdbuf_get_eth_ip_config->subnetMask = inet_address(mib_DHCP_p->SubnetMask);
            apicmdbuf_get_eth_ip_config->ipAddress = inet_address(mib_DHCP_p->IPAddr);
            apicmdbuf_get_eth_ip_config->gatewayAddress = inet_address(mib_DHCP_p->GwyAddr);
/*#endif */

            apicmdbuf_get_eth_ip_config->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_ETH_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 16) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_ETH_IP_CONFIG* apicmdbuf_set_eth_ip_config;
            apicmdbuf_set_eth_ip_config = (APICMDBUF_SET_ETH_IP_CONFIG *)apicmdbuf;
            apicmdbuf_set_eth_ip_config->cmdCode = APCMD_SET_ETH_IP_CONFIG_REPLY;
/*#ifndef AP22 */

            mib_dhcpenable = apicmdbuf_set_eth_ip_config->dhcpEnabled;
            mib_DHCP_p->SubnetMask = inet_address(apicmdbuf_set_eth_ip_config->subnetMask);
            mib_DHCP_p->IPAddr = inet_address(apicmdbuf_set_eth_ip_config->ipAddress);
            mib_DHCP_p->GwyAddr = inet_address(apicmdbuf_set_eth_ip_config->gatewayAddress);
/*#endif */

            apicmdbuf_set_eth_ip_config->cmdStatus = APCTL_OK;
            mib_dhcpstatechange = TRUE;
            initiate_dhcp(1);
        }
        break;
    case APCMD_GET_ETH_PORT_STATUS:
        if (apicmdbuf->cmdBodyLen < 4) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_GET_ETH_PORT_STATUS* apicmdbuf_get_eth_port_status;

            int status;

            apicmdbuf_get_eth_port_status = (APICMDBUF_GET_ETH_PORT_STATUS *)apicmdbuf;
            apicmdbuf_get_eth_port_status->cmdCode = APCMD_GET_ETH_PORT_STATUS_REPLY;
#ifdef AP21
            apicmdbuf_get_eth_port_status->portnumber = 0;

            status = (WL_REGS32(0x80008418) & 0xff);
            if (status & 0x8)
            {
                apicmdbuf_get_eth_port_status->ethOperStatus = ETH_OPER_STATUS_UP;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethOperStatus = ETH_OPER_STATUS_DOWN;
            }
            if ( status & 0x01)
            {
                apicmdbuf_get_eth_port_status->ethLinkSpeed = 100;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethLinkSpeed = 10;
            }

            if (status & 0x02)
            {
                apicmdbuf_get_eth_port_status->ethDuplex = 1;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethDuplex = 0;
            }
#endif
            apicmdbuf_get_eth_port_status->cmdStatus = APCTL_OK;

        }
        break;

    case APCMD_SET_ETH_PORT_PARAMS:
        /*todo */
        break;
    case APCMD_GET_ICMP_EN_STATE:
        /*todo */
        break;
    case APCMD_SET_ICMP_EN_STATE:
        /*todo */
        break;
    case APCMD_GET_802_3_STATES:
        /*todo */
        break;
    case APCMD_GET_TX_DATA_RATE:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_DATA_RATE * apicmdbuf_get_tx_data_rate;
            apicmdbuf_get_tx_data_rate = (APICMDBUF_GET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_get_tx_data_rate->cmdCode = APCMD_GET_TX_DATA_RATE_REPLY;
            if (mib_enableFixedRateTx == FALSE)
            {
                apicmdbuf_get_tx_data_rate->txDataRate=0;
            }
            else
            {
                if (mib_txDataRate==0)
                    apicmdbuf_get_tx_data_rate->txDataRate=2;
                else if (mib_txDataRate==1)
                    apicmdbuf_get_tx_data_rate->txDataRate=4;
                else if (mib_txDataRate==2)
                    apicmdbuf_get_tx_data_rate->txDataRate=11;
                else if (mib_txDataRate==3)
                    apicmdbuf_get_tx_data_rate->txDataRate=22;
                else if (mib_txDataRate==4)
                    apicmdbuf_get_tx_data_rate->txDataRate=44;
            }
            apicmdbuf_get_tx_data_rate->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_TX_DATA_RATE:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_DATA_RATE * apicmdbuf_set_tx_data_rate;
            apicmdbuf_set_tx_data_rate = (APICMDBUF_SET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_set_tx_data_rate->cmdCode = APCMD_SET_TX_DATA_RATE_REPLY;
            apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_OK;
            switch (apicmdbuf_set_tx_data_rate->txDataRate)
            {
            case 0:
                mib_enableFixedRateTx = 0;
                break;
            case 2:
                mib_enableFixedRateTx = 1;
                mib_txDataRate=0;  /* Reg index for 1M */
                break;
            case 4:
                mib_enableFixedRateTx=1;
                mib_txDataRate=1;
                break;
            case 11:
                mib_enableFixedRateTx = 1;
                mib_txDataRate=2;
                break;
            case 22:
                mib_enableFixedRateTx = 1;
                mib_txDataRate=3;
                break;
            case 44:
                mib_enableFixedRateTx = 1;
                mib_txDataRate=4;
            default:
                apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_BAD_CMD;
            }
        }
        break;
    case APCMD_GET_TX_DATA_RATE_G:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_DATA_RATE_G * apicmdbuf_get_tx_data_rate;
            apicmdbuf_get_tx_data_rate= (APICMDBUF_GET_TX_DATA_RATE_G *)apicmdbuf;
            apicmdbuf_get_tx_data_rate->cmdCode =APCMD_GET_TX_DATA_RATE_G_REPLY;
            apicmdbuf_get_tx_data_rate->cmdStatus = APCTL_OK;
            if (mib_enableFixedRateTx == FALSE)
            {
                apicmdbuf_get_tx_data_rate->txDataRate=0;
            }
            else
            {
                switch (mib_txDataRateG)
                {
                case 0:
                    apicmdbuf_get_tx_data_rate->txDataRate=2;
                    break;
                case 1:
                    apicmdbuf_get_tx_data_rate->txDataRate=4;
                    break;
                case 2:
                    apicmdbuf_get_tx_data_rate->txDataRate=11;
                    break;
                case 3:
                    apicmdbuf_get_tx_data_rate->txDataRate=22;
                    break;
                case 4:
                    apicmdbuf_get_tx_data_rate->txDataRate=44;
                    break;
                case 5:
                    apicmdbuf_get_tx_data_rate->txDataRate=12;
                    break;
                case 6:
                    apicmdbuf_get_tx_data_rate->txDataRate=18;
                    break;
                case 7:
                    apicmdbuf_get_tx_data_rate->txDataRate=24;
                    break;
                case 8:
                    apicmdbuf_get_tx_data_rate->txDataRate=36;
                    break;
                case 9:
                    apicmdbuf_get_tx_data_rate->txDataRate=48;
                    break;
                case 10:
                    apicmdbuf_get_tx_data_rate->txDataRate=72;
                    break;
                case 11:
                    apicmdbuf_get_tx_data_rate->txDataRate=96;
                    break;
                case 12:
                    apicmdbuf_get_tx_data_rate->txDataRate=108;
                    break;
                case 13:
                    apicmdbuf_get_tx_data_rate->txDataRate=144;
                    break;
                default:
                    apicmdbuf_get_tx_data_rate->txDataRate=108;
                    break;
                }
            }
        }
        break;

    case APCMD_SET_TX_DATA_RATE_G:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_DATA_RATE_G * apicmdbuf_set_tx_data_rate;
            apicmdbuf_set_tx_data_rate= (APICMDBUF_SET_TX_DATA_RATE_G *)apicmdbuf;
            apicmdbuf_set_tx_data_rate->cmdCode =APCMD_SET_TX_DATA_RATE_G_REPLY;
            apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_OK;
            switch (apicmdbuf_set_tx_data_rate->txDataRate)
            {
            case 0:
                mib_enableFixedRateTx = 0;
                break;
            case 2:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=0;  /* Reg index for 1M */
                break;
            case 4:
                mib_enableFixedRateTx=1;
                mib_txDataRateG=1;
                break;
            case 11:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=2;
                break;
            case 22:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=3;
                break;
            case 44:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=4;
            case 12:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=5;  /* Reg index for 1M */
                break;
            case 18:
                mib_enableFixedRateTx=1;
                mib_txDataRateG=6;
                break;
            case 24:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=7;
                break;
            case 36:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=8;
                break;
            case 48:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=9;
                break;
            case 72:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=10;
                break;
            case 96:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=11;
                break;
            case 108:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=12;
                break;
            case 144:
                mib_enableFixedRateTx = 1;
                mib_txDataRateG=13;
                break;
            default:
                apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_BAD_CMD;
                break;
            }
        }
        break;
    case APCMD_READ_GPIO:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_READ_GPIO * apicmdbuf_read_gpio;
            apicmdbuf_read_gpio = (APICMDBUF_READ_GPIO *)apicmdbuf;
            apicmdbuf_read_gpio->cmdCode = APCMD_READ_GPIO_REPLY;
            /*todo; */
            apicmdbuf_read_gpio->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_WRITE_GPIO:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_WRITE_GPIO * apicmdbuf_write_gpio;
            apicmdbuf_write_gpio = (APICMDBUF_WRITE_GPIO *)apicmdbuf;
            apicmdbuf_write_gpio->cmdCode = APCMD_WRITE_GPIO_REPLY;
            /*	mib_txDataRate=apicmdbuf_set_tx_data_rate->txDataRate; */
            apicmdbuf_write_gpio->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_FW_SOFT_RESET:
        {
            apicmdbuf->cmdStatus = APCTL_OK;
            Reset();
        }

        break;

#ifdef MACACCESSTABLE

    case APCMD_GET_WLAN_FILTER_LIST:
        {
            UINT16 nostation = mib_wlanfilterno;
            UINT16 minbodysize = 2 + mib_wlanfilterno * 6;
            if (apicmdbuf->cmdBodyLen < minbodysize)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = minbodysize;
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_LIST_REPLY;
                return OS_FAIL;
            }
            else
            {
                /*Number of associated station */
#if defined(AP31E) || defined(AP31CLIENT)
                apicmdbuf->cmdBody[1] = 0; /*Cannot be more than 256 for now */
                apicmdbuf->cmdBody[0] = nostation % 0x100;
#else

                apicmdbuf->cmdBody[0] = 0; /*Cannot be more than 256 for now */
                apicmdbuf->cmdBody[1] = nostation % 0x100;
#endif

                memcpy(&apicmdbuf->cmdBody[2], mib_wlanfiltermac, mib_wlanfilterno*6);
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_WLAN_FILTER_LIST:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_LIST_REPLY;
                return OS_FAIL;
            }
            else
            {
                /*Number of associated station */
#if defined(AP31E) || defined(AP31CLIENT)
                mib_wlanfilterno = 0; /*Cannot be more than 256 for now */
                mib_wlanfilterno = apicmdbuf->cmdBody[0] % 0x100;
#else

                mib_wlanfilterno = 0; /*Cannot be more than 256 for now */
                mib_wlanfilterno = apicmdbuf->cmdBody[1] % 0x100;
#if !defined(WIPOD_NEW_GUI)
                mib_wlanfiltermode = 1;
#endif /* WIPOD_NEW_GUI */
#endif

                /*if(mib_wlanfiltermac!=NULL) */
                /*	{ */
                /*	free(mib_wlanfiltermac); */
                /*	} */
                /*malloc(mib_wlanfiltermac,(6*mib_wlanfilterno)); */
                memcpy(mib_wlanfiltermac, &apicmdbuf->cmdBody[2], mib_wlanfilterno*6);
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
#if defined(WIPOD_NEW_GUI)
    case APCMD_GET_WLAN_FILTER_MODE:
        {
            if (apicmdbuf->cmdBodyLen < sizeof(APICMDBUF_GET_FILTER_MODE) - 4*sizeof(unsigned short))
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = sizeof(APICMDBUF_GET_FILTER_MODE) - 4*sizeof(unsigned short);
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_GET_FILTER_MODE *apicmdbuf_get_filter_mode;
                apicmdbuf_get_filter_mode = (APICMDBUF_GET_FILTER_MODE *)apicmdbuf; 
                apicmdbuf_get_filter_mode->mode = mib_wlanfiltermode;
                apicmdbuf_get_filter_mode->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_get_filter_mode->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_WLAN_FILTER_MODE:
        {
            if (apicmdbuf->cmdBodyLen < sizeof(APICMDBUF_SET_FILTER_MODE) - 4*sizeof(unsigned short))
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = sizeof(APICMDBUF_SET_FILTER_MODE) - 4*sizeof(unsigned short);
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_SET_FILTER_MODE *apicmdbuf_set_filter_mode;
                apicmdbuf_set_filter_mode = (APICMDBUF_SET_FILTER_MODE *)apicmdbuf;
                mib_wlanfiltermode = apicmdbuf_set_filter_mode->mode;
                apicmdbuf_set_filter_mode->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_set_filter_mode->cmdStatus = APCTL_OK;
                if (mib_wlanfiltermode == WLAN_FILTER_BLOCK)
                {
                    SendResetCmd(0);
                }
            }
        }
        break;
#else /* WIPOD_NEW_GUI */
    case APCMD_GET_WLAN_FILTER_MODE:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
            else
            {
                apicmdbuf->cmdBody[0] = mib_wlanfiltermode;
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_WLAN_FILTER_MODE:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
            else
            {
                mib_wlanfiltermode = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
                if (mib_wlanfiltermode == WLAN_FILTER_BLOCK)
                {
                    SendResetCmd(0);
                }
            }
        }
        break;
#endif/* WIPOD_NEW_GUI */
#endif

    case APCMD_UPDATE_FIRMWARE:
        {
            APICMDBUF_UPDATE_FIRMWARE * apicmdbuf_update_fw;
            apicmdbuf_update_fw = (APICMDBUF_UPDATE_FIRMWARE *)apicmdbuf;
            if (apicmdbuf_update_fw->cmdBodyLen < 100*1024)
            {
                apicmdbuf_update_fw->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_update_fw->cmdStatus = APCTL_BUF_TOO_SMALL;
                return OS_FAIL;
            }
            else
            {
                FirmwareUpgrade(apicmdbuf_update_fw->imgPtr, apicmdbuf_update_fw->cmdBodyLen);
                apicmdbuf_update_fw->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_update_fw->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SAVE_USER_CONFIG:
        {
            apicmdbuf->cmdCode = APCMD_SAVE_USER_CONFIG_REPLY;
            apicmdbuf->cmdStatus = APCTL_OK;
            UserConfigModified();
        }
        break;
    case APCMD_SET_STA_AGING_TIME_IN_MINUTES:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_STA_AGING_TIME* apicmdbuf_set_sta_aging_time;
            apicmdbuf_set_sta_aging_time = (APICMDBUF_SET_STA_AGING_TIME *)apicmdbuf;
            apicmdbuf_set_sta_aging_time->cmdCode = APCMD_SET_STA_AGING_TIME_IN_MINUTES_REPLY;
            /*set_sta_aging_time(apicmdbuf_set_sta_aging_time->timeInMinutes); */
            apicmdbuf_set_sta_aging_time->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_GET_FIRMWARE_VERSION:
        if (apicmdbuf->cmdBodyLen < 50)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 50;
            return OS_FAIL;
        }
        else
        {
#ifdef WATCHDOG_SIMULATION
            tmp_cnt++;
#endif

            util_CopyList(apicmdbuf->cmdBody, __version__, 50);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_BEACON_TX_MODE:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_BEACON_TX_MODE_REPLY;
                return OS_FAIL;
            }
            else
            {
                mib_beaconDisable = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_BEACON_TX_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
#ifdef ENABLE_RATE_ADAPTATION
    case APCMD_SET_RATE_ADAPTATION_METHOD:
        {
            apicmdbuf->cmdCode++;   /* Make it a reply */
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return OS_FAIL;
            }
            else
            {
                switch ( apicmdbuf->cmdBody[0] )
                {
                case RATE_FOLLOW_STN:
                    txRateAdaptMethod = RATE_FOLLOW_STN;
                    break;
                case RATE_ADAPT_TX:
                default:
                    txRateAdaptMethod = RATE_ADAPT_TX;
#ifdef ENABLE_RATE_ADAPTATION_ON_TX_ERR
                    SetupTxRateProb();  /* Initialize the table */
#endif
                    break;
                }
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RATE_ADAPTATION_PARAMS:
        {
            apicmdbuf->cmdCode++;   /* Make it a reply */
            if (apicmdbuf->cmdBodyLen < 3)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return OS_FAIL;
            }
            else
            {
#ifdef ENABLE_RATE_ADAPTATION_ON_TX_ERR            
                APICMDBUF_RATE_ADAPTATION_PARAMS *p=
                (APICMDBUF_RATE_ADAPTATION_PARAMS *)apicmdbuf;
                p->tx_success_rate_limit = tx_success_rate_limit;
                p->tx_failure_rate_limit = tx_failure_rate_limit;
                p->tx_retry_as_failure = tx_retry_as_failure;
#endif
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_RATE_ADAPTATION_PARAMS:
        {
            apicmdbuf->cmdCode++;   /* Make it a reply */
            if (apicmdbuf->cmdBodyLen < 3)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_RATE_ADAPTATION_PARAMS *p=
                (APICMDBUF_RATE_ADAPTATION_PARAMS *)apicmdbuf;
#ifdef ENABLE_RATE_ADAPTATION_ON_TX_ERR
                tx_success_rate_limit = p->tx_success_rate_limit;
                tx_failure_rate_limit = p->tx_failure_rate_limit;
                tx_retry_as_failure = p->tx_retry_as_failure;
                SetupTxRateProb();  /* Update the table for all STNs */
#endif
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
#endif      /* ENABLE_RATE_ADAPTATION */
    case APCMD_SET_SDRAM_BURST_MODE:
        {
            apicmdbuf->cmdCode++;   /* Make it a reply */
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_SDRAM_BURST_MODE *p=
                (APICMDBUF_SDRAM_BURST_MODE *)apicmdbuf;
                if ( p->sdramBurstMode > SDRAM_BURST_8 )
                {
                    apicmdbuf->cmdStatus = APCTL_ERROR;
                    apicmdbuf->cmdBodyLen = 1;
                    return OS_FAIL;
                }
                SetSdramBurstMode(p->sdramBurstMode);
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_DEVICE_NAME:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
            apicmdbuf->cmdCode =APCMD_GET_DEVICE_NAME_REPLY;
            util_CopyList(apicmdbuf->cmdBody, mib_DeviceName, 16);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_DEVICE_NAME:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
            apicmdbuf->cmdCode =APCMD_SET_DEVICE_NAME_REPLY;
            util_CopyList( mib_DeviceName, apicmdbuf->cmdBody, 16);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
/* GET/SET Baseband Reg */
    case APCMD_GET_BB_REG:
        if (apicmdbuf->cmdBodyLen < 3)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_BB_REG *apicmdbuf_get_bb_reg;
            apicmdbuf_get_bb_reg = (APICMDBUF_GET_BB_REG *)apicmdbuf;
            apicmdbuf_get_bb_reg->cmdCode = APCMD_GET_BB_REG_REPLY;
            /*Read bb reg */
            msi_wl_ReadBBP (apicmdbuf_get_bb_reg->regNum, &apicmdbuf_get_bb_reg->regVal);
            apicmdbuf_get_bb_reg->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_BB_REG:
        if (apicmdbuf->cmdBodyLen < 3)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_BB_REG* apicmdbuf_set_bb_reg;
            apicmdbuf_set_bb_reg = (APICMDBUF_SET_BB_REG *)apicmdbuf;
            apicmdbuf_set_bb_reg->cmdCode = APCMD_SET_BB_REG_REPLY;

            /*Write reg */
            msi_wl_WriteBBP (apicmdbuf_set_bb_reg->regNum, apicmdbuf_set_bb_reg->regVal);

            apicmdbuf_set_bb_reg->cmdStatus = APCTL_OK;
        }

        break;
/* GET/SET Baseband Reg end */

    case APCMD_GET_CUR_RADIO_TX_RATE:
        if (apicmdbuf->cmdBodyLen < 1) /** size of UINT16 **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            extern int CurTxDataRate;
            APICMDBUF_GET_CUR_RADIO_TX_RATE* apicmdbuf_get_radio_tx_rate;
            apicmdbuf_get_radio_tx_rate = (APICMDBUF_GET_CUR_RADIO_TX_RATE *)apicmdbuf;
            apicmdbuf_get_radio_tx_rate->cmdCode = APCMD_GET_CUR_RADIO_TX_RATE_REPLY;
            apicmdbuf_get_radio_tx_rate->txRate = mib_PhySuppTxDataRates_p[CurTxDataRate].SuppDataRatesTxVal;
            apicmdbuf_get_radio_tx_rate->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_GET_AP_MODE:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AP_MODE* apicmdbuf_get_ap_mode;
            apicmdbuf_get_ap_mode = (APICMDBUF_GET_AP_MODE *)apicmdbuf;
            apicmdbuf_get_ap_mode->cmdCode =APCMD_GET_AP_MODE_REPLY;

            apicmdbuf_get_ap_mode->apMode = mib_ApMode;
            apicmdbuf_get_ap_mode->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_AP_MODE:
        /* currently supports only mixed mode */
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AP_MODE* apicmdbuf_set_ap_mode;
            apicmdbuf_set_ap_mode = (APICMDBUF_SET_AP_MODE *)apicmdbuf;
            apicmdbuf_set_ap_mode->cmdCode = APCMD_SET_AP_MODE_REPLY;
            mib_ApMode = apicmdbuf_set_ap_mode->apMode;
            if (mib_ApMode == AP_MODE_G_ONLY && (!G_Mode_ErpEnable()))
                mib_ErpProtEnabled = FALSE;
            else
                mib_ErpProtEnabled = TRUE;
            setBasicRate(mib_ApMode);
            apicmdbuf_set_ap_mode->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_PRIVACY_TBL_RSN_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED* apicmdbuf_get_privacy_tbl_rsn_enabled;
                apicmdbuf_get_privacy_tbl_rsn_enabled = (APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED*)apicmdbuf;
                apicmdbuf_get_privacy_tbl_rsn_enabled->cmdCode = APCMD_GET_PRIVACY_TBL_RSN_ENABLED_REPLY;
                apicmdbuf_get_privacy_tbl_rsn_enabled->Enabled = mib_PrivacyTable_p->RSNEnabled;
                apicmdbuf_get_privacy_tbl_rsn_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_PRIVACY_TBL_RSN_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED* apicmdbuf_set_privacy_tbl_rsn_enabled;
                apicmdbuf_set_privacy_tbl_rsn_enabled = (APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED*)apicmdbuf;
                apicmdbuf_set_privacy_tbl_rsn_enabled->cmdCode = APCMD_SET_PRIVACY_TBL_RSN_ENABLED_REPLY;
			    mib_PrivacyTable_p->RSNEnabled = apicmdbuf_set_privacy_tbl_rsn_enabled->Enabled; 
                if (mib_PrivacyTable_p->RSNEnabled)
                {
                    DoWPAAndSchedFrameFp = DoTKIPAndSchedFrameAP;
                    ProcessWPAPcktFp = ProcessTKIPPcktAP;
                    mib_PrivacyTable_p->PrivInvoked = FALSE; /*Disable WEP*/
                }
                apicmdbuf_set_privacy_tbl_rsn_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;
        case APCMD_SET_RSN_CFG_AUTH_SUITE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_AUTH_SUITE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_AUTH_SUITE* apicmdbuf_set_rsn_cfg_auth_suite;
                apicmdbuf_set_rsn_cfg_auth_suite =
                    (APICMDBUF_SET_RSN_CFG_AUTH_SUITE*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_auth_suite->cmdCode = APCMD_SET_RSN_CFG_AUTH_SUITE;

                memcpy(mib_RSNConfigAuthSuites_p->AuthSuites,
                       apicmdbuf_set_rsn_cfg_auth_suite->AuthSuites, 4);
                mib_RSNConfigAuthSuites_p->Enabled = apicmdbuf_set_rsn_cfg_auth_suite->Enabled;
                apicmdbuf_set_rsn_cfg_auth_suite->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_AUTH_SUITE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_AUTH_SUITE) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_AUTH_SUITE* apicmdbuf_get_rsn_cfg_auth_suite;
                apicmdbuf_get_rsn_cfg_auth_suite =
                    (APICMDBUF_GET_RSN_CFG_AUTH_SUITE*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_auth_suite->cmdCode =
                    APCMD_GET_RSN_CFG_AUTH_SUITE;
                memcpy(apicmdbuf_get_rsn_cfg_auth_suite->AuthSuites,
                       mib_RSNConfigAuthSuites_p->AuthSuites, 4);
                apicmdbuf_get_rsn_cfg_auth_suite->Enabled = mib_RSNConfigAuthSuites_p->Enabled;
                apicmdbuf_get_rsn_cfg_auth_suite->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_RSN_CFG_UNICAST_CIPHERS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_UNICAST_CIPHERS) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_UNICAST_CIPHERS* apicmdbuf_set_rsn_cfg_ucast_cipher;
                apicmdbuf_set_rsn_cfg_ucast_cipher =
                (APICMDBUF_SET_RSN_CFG_UNICAST_CIPHERS*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_ucast_cipher->cmdCode = APCMD_SET_RSN_CFG_UNICAST_CIPHERS;

                memcpy(mib_RSNConfigUnicastCiphers_p->UnicastCipher,
                       apicmdbuf_set_rsn_cfg_ucast_cipher->UnicastCipher, 4);
                mib_RSNConfigAuthSuites_p->Enabled = apicmdbuf_set_rsn_cfg_ucast_cipher->Enabled;
                apicmdbuf_set_rsn_cfg_ucast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_UNICAST_CIPHERS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_UNICAST_CIPHERS) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_UNICAST_CIPHERS* apicmdbuf_get_rsn_cfg_ucast_cipher;
                apicmdbuf_get_rsn_cfg_ucast_cipher =
                (APICMDBUF_GET_RSN_CFG_UNICAST_CIPHERS*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_ucast_cipher->cmdCode =
                                                APCMD_GET_RSN_CFG_UNICAST_CIPHERS;
                memcpy(apicmdbuf_get_rsn_cfg_ucast_cipher->UnicastCipher,
                       mib_RSNConfigUnicastCiphers_p->UnicastCipher, 4);
                apicmdbuf_get_rsn_cfg_ucast_cipher->Enabled = mib_RSNConfigAuthSuites_p->Enabled;
                apicmdbuf_get_rsn_cfg_ucast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_CFG_MULTICAST_CIPHERS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS* apicmdbuf_set_rsn_cfg_mcast_cipher;
                apicmdbuf_set_rsn_cfg_mcast_cipher =
                (APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_mcast_cipher->cmdCode =
                                                APCMD_SET_RSN_CFG_MULTICAST_CIPHERS;
                memcpy(mib_RSNConfig_p->MulticastCipher,
                       apicmdbuf_set_rsn_cfg_mcast_cipher->MulticastCipher,4);
                apicmdbuf_set_rsn_cfg_mcast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;

        case APCMD_GET_RSN_CFG:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG* apicmdbuf_get_rsn_cfg;
                apicmdbuf_get_rsn_cfg = (APICMDBUF_GET_RSN_CFG*)apicmdbuf;
                apicmdbuf_get_rsn_cfg->cmdCode = APCMD_GET_RSN_CFG;

                apicmdbuf_get_rsn_cfg->Version = mib_RSNConfig_p->Version;
                apicmdbuf_get_rsn_cfg->PairwiseKeysSupported = mib_RSNConfig_p->PairwiseKeysSupported;
                memcpy(apicmdbuf_get_rsn_cfg->MulticastCipher, mib_RSNConfig_p->MulticastCipher, 4);
                apicmdbuf_get_rsn_cfg->GroupRekeyMethod = mib_RSNConfig_p->GroupRekeyMethod;
                apicmdbuf_get_rsn_cfg->GroupRekeyTime = mib_RSNConfig_p->GroupRekeyTime;
                apicmdbuf_get_rsn_cfg->GroupRekeyPackets = mib_RSNConfig_p->GroupRekeyPackets;
                apicmdbuf_get_rsn_cfg->GroupRekeyStrict = mib_RSNConfig_p->GroupRekeyStrict;
                apicmdbuf_get_rsn_cfg->TSNEnabled = mib_RSNConfig_p->TSNEnabled;
                apicmdbuf_get_rsn_cfg->GroupMasterRekeyTime = mib_RSNConfig_p->GroupMasterRekeyTime;
                apicmdbuf_get_rsn_cfg->GroupUpdateTimeOut = mib_RSNConfig_p->GroupUpdateTimeOut;
                apicmdbuf_get_rsn_cfg->GroupUpdateCount = mib_RSNConfig_p->GroupUpdateCount;
                apicmdbuf_get_rsn_cfg->PairwiseUpdateTimeOut = mib_RSNConfig_p->PairwiseUpdateTimeOut;
                apicmdbuf_get_rsn_cfg->PairwiseUpdateCount = mib_RSNConfig_p->PairwiseUpdateCount;

                apicmdbuf_get_rsn_cfg->cmdStatus = APCTL_OK;
            }
        }
        break;
        case APCMD_GET_RSN_STATS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_STATS) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_STATS* apicmdbuf_get_rsn_stats;
                apicmdbuf_get_rsn_stats = (APICMDBUF_GET_RSN_STATS*)apicmdbuf;
                apicmdbuf_get_rsn_stats->cmdCode = APCMD_GET_RSN_STATS_REPLY;

                /*memcpy(apicmdbuf_get_rsn_stats->MacAddr, mib_RSNStats_p->MacAddr, 6);*/
                apicmdbuf_get_rsn_stats->Version = mib_RSNStats_p->Version;
                memcpy( apicmdbuf_get_rsn_stats->SelectedUnicastCipher, mib_RSNStats_p->SelectedUnicastCipher, 4);
                apicmdbuf_get_rsn_stats->TKIPICVErrors = mib_RSNStats_p->TKIPICVErrors;
                apicmdbuf_get_rsn_stats->TKIPLocalMICFailures = mib_RSNStats_p->TKIPLocalMICFailures;
                apicmdbuf_get_rsn_stats->TKIPRemoteMICFailures = mib_RSNStats_p->TKIPRemoteMICFailures;
                apicmdbuf_get_rsn_stats->TKIPCounterMeasuresInvoked = mib_RSNStats_p->TKIPCounterMeasuresInvoked;
                apicmdbuf_get_rsn_stats->CCMPFormatErrors = mib_RSNStats_p->CCMPFormatErrors;
                apicmdbuf_get_rsn_stats->CCMPReplays = mib_RSNStats_p->CCMPReplays;
                apicmdbuf_get_rsn_stats->CCMPDecryptErrors = mib_RSNStats_p->CCMPDecryptErrors;

                apicmdbuf_get_rsn_stats->cmdStatus = APCTL_OK;
            }
        }
        break;

#if defined(WIPOD)
        case APCMD_GET_WIPOD_MODE:
            if (apicmdbuf->cmdBodyLen < 
                sizeof(APICMDBUF_GET_WIPOD_MODE) - 4*sizeof(unsigned short))
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 
                    sizeof(APICMDBUF_GET_WIPOD_MODE) - 4*sizeof(unsigned short);
                apicmdbuf->cmdCode = APCMD_GET_WIPOD_MODE_REPLY;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_WIPOD_MODE *apicmdbuf_get_wipod_mode;
                apicmdbuf_get_wipod_mode = (APICMDBUF_GET_WIPOD_MODE *)apicmdbuf;
	    	    apicmdbuf_get_wipod_mode->cmdCode = APCMD_GET_WIPOD_MODE_REPLY;
                apicmdbuf_get_wipod_mode->mode = mib_wipodMode;
			    apicmdbuf_get_wipod_mode->cmdStatus = APCTL_OK;
            }
	        break;

        case APCMD_SET_WIPOD_MODE:
            if (apicmdbuf->cmdBodyLen < 
                sizeof(APICMDBUF_SET_WIPOD_MODE) - 4*sizeof(unsigned short))
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 
                    sizeof(APICMDBUF_SET_WIPOD_MODE) - 4*sizeof(unsigned short);
                apicmdbuf->cmdCode = APCMD_SET_WIPOD_MODE_REPLY;
                return FAIL;
            }
            else
            {
                #define IS_MAC_VALID(x) (((unsigned char *)x)[0]||((unsigned char *)x)[1]||((unsigned char *)x)[2] \
                                        ||((unsigned char *)x)[3]||((unsigned char *)x)[4]||((unsigned char *)x)[5])

                int tmpCount = 0;
                unsigned char *tmpMacAddr = NULL;

                APICMDBUF_SET_WIPOD_MODE *apicmdbuf_set_wipod_mode;
                apicmdbuf_set_wipod_mode = (APICMDBUF_SET_WIPOD_MODE *)apicmdbuf;
				apicmdbuf_set_wipod_mode->cmdCode = APCMD_SET_WIPOD_MODE_REPLY;
/* This is not used by new GUI */
//#define ENABLE_WDS_API /* Comment out this line if any problem */
#if defined(ENABLE_WDS_API)
                /* This piece of code need to be integrated and tested with new GUI */
                /* Individual WDS function, AP/WB are tested ok independently */
                switch(apicmdbuf_set_wipod_mode->mode)
                {
                    case WIPOD_MODE_CLIENT_BRIDGE:
                        break;
                    case WIPOD_MODE_AP:
                        // AP need to disable WDS ??
                        break;
                    case WIPOD_MODE_REPEATER:

                        tmpMacAddr = ((APICMDBUF_WIPOD_REPEATER_MODE *)apicmdbuf_set_wipod_mode)->macAddr1;
                        
                        mib_wdsEnable = 1;
                        mib_wdsno = 0;

                        if( IS_MAC_VALID(tmpMacAddr) ) {
                            
                            memcpy(mib_wdsmac,tmpMacAddr,6);
                            mib_wdsno ++; 
                        }
                        if( IS_MAC_VALID(tmpMacAddr+6) ) {
                            
                            memcpy(mib_wdsmac + (mib_wdsno*6),tmpMacAddr+6,6);
                            mib_wdsno ++; 
                        }

                        if(((APICMDBUF_WIPOD_REPEATER_MODE *)apicmdbuf_set_wipod_mode)->enableApFunc) {

                            mib_broadcastssid = TRUE;
                        } else {

                            mib_broadcastssid = FALSE;
                        }
                        break;
                    case WIPOD_MODE_PT_2_PT:

                        tmpMacAddr = ((APICMDBUF_WIPOD_P2P_MODE *)apicmdbuf_set_wipod_mode)->macAddr;

                        if( IS_MAC_VALID(tmpMacAddr) ) {

                            mib_wdsEnable = 1;
                            mib_wdsno = 1; 
                            memcpy(mib_wdsmac,tmpMacAddr,6);

                            mib_broadcastssid = FALSE;
                        } else {

                            apicmdbuf_set_wipod_mode->cmdStatus = APCTL_ERROR;
                            return OS_FAIL;
                        }
                        break;

                    case WIPOD_MODE_PT_2_MULTIPT:

                        if(((APICMDBUF_WIPOD_P2MP_MODE *)apicmdbuf_set_wipod_mode)->enableDynamic) {

                            mib_wdsEnable = 2;
                        } else {

                            mib_wdsEnable = 1;
                            mib_wdsno = 0; 

                            for( i = 0; i < 6; i++) {

                                tmpMacAddr = ((APICMDBUF_WIPOD_P2MP_MODE *)apicmdbuf_set_wipod_mode)->macAddr1 + (i*6);
                                if( IS_MAC_VALID(tmpMacAddr) ) {

                                    memcpy(mib_wdsmac + (mib_wdsno*6), tmpMacAddr, 6); 
                                    mib_wdsno ++;
                                }
                            }
                        }
                        mib_broadcastssid = FALSE;
                        break;

                    default:
                        apicmdbuf_set_wipod_mode->cmdStatus = APCTL_ERROR;
                        return OS_FAIL;
                }
                //SetUserConfig();
#endif /* ENABLE_WDS_API */

                mib_wipodMode = apicmdbuf_set_wipod_mode->mode;
				apicmdbuf_set_wipod_mode->cmdStatus = APCTL_OK;
            }
    		break;
#endif /* WIPOD */

#ifdef WDS_FEATURE
        case APCMD_GET_WDS_MODE:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WDS_MODE_REPLY;
                return FAIL;
            }
            else
            {
                apicmdbuf->cmdBody[0] = mib_wdsEnable;
		   		apicmdbuf->cmdCode = APCMD_GET_WDS_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
	      break;
        case APCMD_SET_WDS_MODE:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WDS_MODE_REPLY;
                return FAIL;
            }
            else
            {
                mib_wdsEnable = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_WDS_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		break;
        case APCMD_GET_WDS_LIST:
#if !defined(WIPOD_NEW_GUI)
            if (apicmdbuf->cmdBodyLen < 2 + mib_wdsno * 6)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 2 + mib_wdsno * 6;
                apicmdbuf->cmdCode = APCMD_GET_WDS_LIST_REPLY;
                return FAIL;
            }
#else
            if (apicmdbuf->cmdBodyLen < 2 + 6*6)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 2 + 6*6;
                apicmdbuf->cmdCode = APCMD_GET_WDS_LIST_REPLY;
                return FAIL;
            }
#endif /* WIPOD_NEW_GUI */
            else
            {
                if(mib_wdsEnable != 2)
                {
                    apicmdbuf->cmdBody[0] = mib_wdsno;

#if !defined(WIPOD_NEW_GUI)
                    memcpy(&apicmdbuf->cmdBody[2], mib_wdsmac, mib_wdsno*6);
#else
                    memcpy(&apicmdbuf->cmdBody[2], mib_wdsmac, 6*6);
#endif
                }
                else
                {
                    apicmdbuf->cmdBody[0] = getDynamicWdsMac(&apicmdbuf->cmdBody[2]); 
                }
                apicmdbuf->cmdCode = APCMD_GET_WDS_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		break;

        case APCMD_SET_WDS_LIST:
            if (apicmdbuf->cmdBodyLen < 2)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 2;
                apicmdbuf->cmdCode = APCMD_SET_WDS_LIST_REPLY;
                return FAIL;
            }
            else
            {
#if !defined(WIPOD_NEW_GUI)
                if(mib_wdsEnable == 1)
#endif
                {
                    mib_wdsno = apicmdbuf->cmdBody[0];
                    memcpy(mib_wdsmac, &apicmdbuf->cmdBody[2],mib_wdsno*6);
                }
                apicmdbuf->cmdCode = APCMD_SET_WDS_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
                SetUserConfig();
                return APCTL_OK;
            }
		break;

        case APCMD_GET_DISABLE_ASSOC:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_DISABLE_ASSOC_REPLY;
                return FAIL;
            }
            else
            {
                apicmdbuf->cmdBody[0] = mib_disableAssoc;
		   		apicmdbuf->cmdCode = APCMD_GET_DISABLE_ASSOC_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
	        break;

        case APCMD_SET_DISABLE_ASSOC:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_DISABLE_ASSOC_REPLY;
                return FAIL;
            }
            else
            {
                mib_disableAssoc = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_DISABLE_ASSOC_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		    break;

        case APCMD_GET_WDS_OPTION:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WDS_OPTION_REPLY;
                return FAIL;
            }
            else
            {
                apicmdbuf->cmdBody[0] = mib_wdsOption;
		   		apicmdbuf->cmdCode = APCMD_GET_WDS_OPTION_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
	        break;

        case APCMD_SET_WDS_OPTION:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WDS_OPTION_REPLY;
                return FAIL;
            }
            else
            {
                mib_wdsOption = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_WDS_OPTION_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		    break;

#endif /* WDS_FEATURE*/

    case APCMD_GET_FORCE_PROTECTION_MODE:
        if ( apicmdbuf->cmdBodyLen < 2 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_FORCE_PROTECTION_MODE * apicmdbuf_get_force_protection_mode;
            apicmdbuf_get_force_protection_mode = (APICMDBUF_GET_FORCE_PROTECTION_MODE *)apicmdbuf;
            apicmdbuf_get_force_protection_mode->cmdCode = APCMD_GET_FORCE_PROTECTION_MODE_REPLY;

            apicmdbuf_get_force_protection_mode->forceProtectMode = mib_forceProtectiondisable;
            apicmdbuf_get_force_protection_mode->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_FORCE_PROTECTION_MODE:
        /* currently supports only mixed mode */
        if ( apicmdbuf->cmdBodyLen < 2 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_FORCE_PROTECTION_MODE* apicmdbuf_set_force_protection_mode;
            apicmdbuf_set_force_protection_mode = (APICMDBUF_SET_FORCE_PROTECTION_MODE*)apicmdbuf;
            apicmdbuf_set_force_protection_mode->cmdCode = APCMD_SET_FORCE_PROTECTION_MODE_REPLY;
            mib_forceProtectiondisable= apicmdbuf_set_force_protection_mode->forceProtectMode;
            mib_ErpProtEnabled = (!mib_forceProtectiondisable);
            apicmdbuf_set_force_protection_mode->cmdStatus = APCTL_OK;
        }
        break;

#ifdef BURST_MODE
    case APCMD_GET_BURST_MODE:
        if (apicmdbuf->cmdBodyLen < 
            sizeof(APICMDBUF_GET_BURST_MODE) - 4*sizeof(unsigned short))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 
                sizeof(APICMDBUF_GET_BURST_MODE) - 4*sizeof(unsigned short);
            apicmdbuf->cmdCode = APCMD_GET_BURST_MODE_REPLY;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_BURST_MODE *apicmdbuf_get_burst_mode;
            apicmdbuf_get_burst_mode = (APICMDBUF_GET_BURST_MODE *)apicmdbuf;
	    	apicmdbuf_get_burst_mode->cmdCode = APCMD_GET_BURST_MODE_REPLY;
            apicmdbuf_get_burst_mode->burstMode = mib_BurstMode_p->mib_burstmode;
            apicmdbuf_get_burst_mode->burstRate = mib_BurstMode_p->mib_burstrate;
			apicmdbuf_get_burst_mode->cmdStatus = APCTL_OK;
        }
	    break;

    case APCMD_SET_BURST_MODE:
        if (apicmdbuf->cmdBodyLen < 
            sizeof(APICMDBUF_SET_BURST_MODE) - 4*sizeof(unsigned short))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 
                sizeof(APICMDBUF_SET_BURST_MODE) - 4*sizeof(unsigned short);
            apicmdbuf->cmdCode = APCMD_SET_BURST_MODE_REPLY;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_BURST_MODE *apicmdbuf_set_burst_mode;
            apicmdbuf_set_burst_mode = (APICMDBUF_SET_BURST_MODE *)apicmdbuf;
			apicmdbuf_set_burst_mode->cmdCode = APCMD_SET_BURST_MODE_REPLY;
            mib_BurstMode_p->mib_burstmode = apicmdbuf_set_burst_mode->burstMode;
            if( mib_BurstMode_p->mib_burstmode != 0 ) 
            {
                mib_BurstMode_p->mib_burstrate = apicmdbuf_set_burst_mode->burstRate;
            }
            apicmdbuf_set_burst_mode->cmdStatus = APCTL_OK;
        }
        break;
#endif /* BURST_MODE */

#ifdef AP_SITE_SURVEY
    case APCMD_SET_SURVEY_SCAN:
        {

            apicmdbuf->cmdCode = APCMD_SET_SURVEY_SCAN_REPLY;        
            memset((UINT8 *)&siteSurveyInfo, 0, sizeof(API_SURVEY_ENTRY) * SITE_SURVEY_ENTRY_MAX);
            if(!busyScanning)
            {
                UINT8  scanChannelMin;
                UINT8  scanChannelMax;
                /* get range to scan */
                #ifdef DOMAIN_SUPPORTED
                if (domainGetInfo(&scanChannelMin, &scanChannelMax, &defaultChannel) == OS_FAIL)
                #endif
                {
                    scanChannelMin = DEFAULT_CHAN_MIN;
                    scanChannelMax = DEFAULT_CHAN_MAX;
                }
                SendScanCmd(scanChannelMin, scanChannelMax);
            }
            apicmdbuf->cmdStatus = APCTL_OK;
        }
        break;
    
    case APCMD_GET_SURVEY_INFO:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_SURVEY_INFO* apicmdbuf_get_survey;
            apicmdbuf_get_survey = (APICMDBUF_GET_SURVEY_INFO *)apicmdbuf;
            apicmdbuf_get_survey->cmdCode =APCMD_GET_SURVEY_INFO_REPLY;

            if(busyScanning)
            {
                apicmdbuf_get_survey->busyScan = 1;
            }
            else
            {
                apicmdbuf_get_survey->busyScan = 0;
            }
            apicmdbuf_get_survey->maxEntry = SITE_SURVEY_ENTRY_MAX;
            apicmdbuf_get_survey->infoAddr = (UINT8 *)&siteSurveyInfo;
            apicmdbuf_get_survey->cmdStatus = APCTL_OK;
#ifdef ETH_DEBUG
            //eprintf("API busyScan = %x\n", apicmdbuf_get_survey->busyScan);
            //eprintf("API maxEntry = %x\n", apicmdbuf_get_survey->maxEntry);
            //eprintf("API addr = %x\n", apicmdbuf_get_survey->infoAddr);
#endif /* ETH_DEBUG */
        }
        break;
#endif /* AP_SITE_SURVEY */
#ifdef SELF_CTS
    case APCMD_GET_SELF_CTS:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_SELF_CTS * apicmdbuf_get_SELF_CTS;
            apicmdbuf_get_SELF_CTS = (APICMDBUF_GET_SELF_CTS *)apicmdbuf;
            apicmdbuf_get_SELF_CTS->cmdCode = APCMD_GET_SELF_CTS_REPLY;

            apicmdbuf_get_SELF_CTS->SelfCts = mib_StaCfg_p->SelfCts;
            apicmdbuf_get_SELF_CTS->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_SELF_CTS:
        /* currently supports only mixed mode */
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_SELF_CTS* apicmdbuf_set_SELF_CTS;
            apicmdbuf_set_SELF_CTS = (APICMDBUF_SET_SELF_CTS*)apicmdbuf;
            apicmdbuf_set_SELF_CTS->cmdCode = APCMD_SET_SELF_CTS_REPLY;
            mib_StaCfg_p->SelfCts= apicmdbuf_set_SELF_CTS->SelfCts;
            apicmdbuf_set_SELF_CTS->cmdStatus = APCTL_OK;
        }
        break;
#endif
    default:
        apicmdbuf->cmdStatus = APCTL_BAD_CMD;
        return OS_FAIL;
    }



    return OS_SUCCESS;


}
void Reset(void)
{
#ifndef AP31E
    union
    {
        int x;
        void (*f)(void);
    }u;

    WaitForFlashReady();
    msi_wl_PowerdownRF();
    u.x = 0xffff0000;
    u.f();
#endif    
}

static unsigned flashsizeApi = 0;
static unsigned flashbaseApi = 0xffff0000;


/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

#define EVAL_BOARD  0
#define AP21_BOARD  1
#define AP22_BOARD  2
#define AP22N_BOARD 3


static unsigned manuApi;
static unsigned partApi;
static int split_sectorApi = -1;
static int split_sector_dataApi[] = {
    0,
    0x8000,
    0xa000,
    0xc000,
};

static void amdPreamble()
{
    _REG16(flashbaseApi | 0xAAA) = 0x00AA;
    _REG16(flashbaseApi | 0x555) = 0x0055;
    _REG16(flashbaseApi | 0xAAA) = 0x0080;
    _REG16(flashbaseApi | 0xAAA) = 0x00AA;
    _REG16(flashbaseApi | 0x555) = 0x0055;
}
static void amdEraseChip(int sblk, int nblks)
{
    unsigned addr;
    int i;
    int j;
    for (i = sblk;i < (sblk + nblks);i++)
    {
        if (i == split_sectorApi)
        {
            unsigned start = (i << (16));
            for (j = 0;j < sizeof(split_sector_dataApi) / sizeof(int);j++)
            {
                addr = flashbaseApi + start + split_sector_dataApi[j];
                amdPreamble();
                _REG16(addr) = 0x0030;
                while (!(_REG(0x80006000)&0x1))
                    ;
            }
        }
        else
        {
            addr = flashbaseApi + (i << (16));
            amdPreamble();
            _REG16(addr) = 0x0030;
            while (!(_REG(0x80006000)&0x1))
                ;
        }
    }
}

static void amdFlashWriteWord( int offset, unsigned short dat)
{
    unsigned addr = flashbaseApi + offset;
    _REG16(flashbaseApi | 0xAAA) = 0x00AA;  /* word address 555 */
    _REG16(flashbaseApi | 0x555) = 0x0055;  /* word address 2AA */
    _REG16(flashbaseApi | 0xAAA) = 0x00A0;  /* word address 555 */

    _REG16(addr) = dat;  /* word address 0xFC000000 */
    while (_REG16(addr) != dat)
        ;
}
static void
interrogate()
{
    _REG16(flashbaseApi | 0xAAA) = 0x00AA;
    _REG16(flashbaseApi | 0x555) = 0x0055;
    _REG16(flashbaseApi | 0xAAA) = 0x0090;
    manuApi = _REG16(flashbaseApi);     /* Manu */
    partApi = _REG16(flashbaseApi + 2); /* part ID. */
}

BOOLEAN apctlFlashEraseSector(int sectorNum)
{
    flashsizeApi = 2 * 1024 * 1024;
    flashbaseApi = (0 - flashsizeApi);

    interrogate();
    switch (manuApi)
    {
    case 1:  /* AMD */
        break;
    default:
        /*PRINT("\r\nUnknown manufacturer %x\r\n", manuApi); */
        return FALSE;
        break;
    }
    /* Have we heard of the part? */
    switch (partApi)
    {
    case 0x22c4:  /* 160BT */
        flashsizeApi = 2 * 1024 * 1024;
        split_sectorApi = 31;
        break;
    case 0x225b:  /* LV800 */
        flashsizeApi = 1 * 1024 * 1024;
        break;
    default:
        /*PRINT("\r\nUnknown part %x\r\n", partApi); */
        return FALSE;
    }


    amdEraseChip(sectorNum, 1);
    return TRUE;
}
BOOLEAN apctlFlashWriteWord(int offset, UINT16 value)
{
    flashsizeApi = 2 * 1024 * 1024;
    flashbaseApi = (0 - flashsizeApi);

    switch (manuApi)
    {
    case 1:  /* AMD */
        break;
    default:
        /*PRINT("\r\nUnknown manufacturer %x\r\n", manuApi); */
        return FALSE;
        break;
    }

    /* Have we heard of the part? */
    switch (partApi)
    {
    case 0x22c4:  /* 160BT */
        flashsizeApi = 2 * 1024 * 1024;
        split_sectorApi = 31;
        break;
    case 0x225b:  /* LV800 */
        flashsizeApi = 1 * 1024 * 1024;
        break;
    default:
        /*PRINT("\r\nUnknown part %x\r\n", partApi); */
        return FALSE;
    }

    amdFlashWriteWord(offset, value);
    return TRUE;

}

void ConfigMDU(int boardtype)
{
    if (boardtype > AP22N_BOARD)
    {
        return ;
    }
    _REG(MDU_TIMR00) = 0x0442A20C;   /* FLASH timing */
    _REG(MDU_CSR) = 0x4190040;  /* for 88 Mhz clock cpu speed */
    _REG(MDU_TIMR01) = 0x00008080;
    _REG(MDU_CFGR0) = 0xFFFF4285;   /* 0xffc00000 FLASH configuration pack_en = 1*/
    switch (boardtype)
    {
    case EVAL_BOARD:
        _REG(MDU_TIMR20) = 0x00000022;
        _REG(MDU_TIMR21) = 0x03232222;
        _REG(MDU_CFGR2) = 0x40FE50B5; /* 0xd0000000 */
        break;
    case AP21_BOARD:
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x03232222;
        _REG(MDU_CFGR1) = 0x40FF5095; /* 0xd0000000 */
        break;
    case AP22_BOARD:
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x08383632;
        _REG(MDU_CFGR1) = 0x40FC70B5; /* 0xd0000000 */
        break;
    case AP22N_BOARD:
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x08383632;
        _REG(MDU_CFGR1) = 0x40FE70B7; /* 0xd0000000 */
        break;
    }
    /*disable devices 3 4 5 */
    _REG(MDU_CFGR3) = 0xC0FF0008;
    _REG(MDU_CFGR4) = 0x80FF0008;
    _REG(MDU_CFGR5) = 0x40FF0008;

}


/* Interrupt control API */

/**
 * Disable all interrupts
 * 
 * @return Interrupt status
 */
int apctlIntDisable(void)
{
    cyg_interrupt_disable();
    intStatus = -1;
    return intStatus;
}

/**
 * Enable all interrupts
 * 
 * @param status Status word returned by apctlIntDisable()
 * @return Interrupt status
 */
int apctlIntEnable(int status)
{
    cyg_interrupt_enable();
    intStatus = 0;
    return intStatus;
}

/**
 * Get interrupt enable status
 * 
 * @return Interrupt status
 */
int apctlGetIntStatus(void)
{
    return intStatus;
}

unsigned char apctlSwLedDisable(unsigned char gpioport)
{
    unsigned long dirMap = 0;
    unsigned long val;

    WL_READ_REGS32(GPIO_REG_OUTPUT_ENABLE, val);

    dirMap = 1 << gpioport;

    WL_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, val | dirMap);

    return 0;
}


void apctlSwLedEnable(unsigned char gpioport)
{

    unsigned long dirMap = 0;
    unsigned long val;

    WL_READ_REGS32(GPIO_REG_OUTPUT_ENABLE, val);
    dirMap = 1 << gpioport;

    WL_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, ~dirMap & val);

    if ( gpioport < 8)
        WL_WRITE_WORD(GPIO_REG_SELECT_0_7, 1 << (2*gpioport));
    else
        WL_WRITE_WORD(GPIO_REG_SELECT_8_15, 1 << (2*(gpioport - 8)));
}
void apctlSwLedSetCycle(unsigned char gpioport, unsigned int cycle, unsigned int duty)
{
    unsigned int cycle_b = (cycle << (gpioport/4)) & 0xffff;
    unsigned int duty_b = (duty << (gpioport/4)) & 0xffff;
    unsigned int mask = ~(0xf << (gpioport/4)); 
    switch (gpioport)
    {
    case 0:
    case 1:
    case 2:
    case 3:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_3_0) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_3_0) & mask) | duty_b );
        break;
    case 4:
    case 5:
    case 6:
    case 7:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_7_4) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_7_4) & mask) | duty_b );
        break;
    case 8:
    case 9:
    case 10:
    case 11:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_11_8) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_11_8) & mask) | duty_b );
        break;
    case 12:
    case 13:
    case 14:
    case 15:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_15_12) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_15_12) & mask) | duty_b );
        break;
    }
}
#ifdef AP21N
    #include "etherLow.h"

unsigned int packet_cnt[2];

int ether_port_status;
void apctlLedSet(unsigned char wlan, unsigned char status)
{
    unsigned char gpio[4] = {4, 5, 8, 9};

    switch (status)
    {
#ifdef LED_INVERTOR
    case 1:
#else

    case 0:   /*off */
#endif

        apctlGPIOWrite(gpio[wlan], 1);
        break;
#ifdef LED_INVERTOR

    case 0:
#else

    case 1:   /*on */
#endif

        apctlGPIOWrite(gpio[wlan], 0);
        if (wlan)
            apctlGPIOWrite(gpio[wlan + 2], 0);
        else
        {
            if (ether_port_status & 0x0008)
            {
                if (ether_port_status & 1)
                    apctlGPIOWrite(gpio[wlan + 2], 0);
                else
                    apctlGPIOWrite(gpio[wlan + 2], 1);
            }
            else
            {
                apctlGPIOWrite(gpio[wlan], 1);
            }
        }
        break;
    default:
        break;
    }

}
    #define MAX_CYCLES_PER_SECOND   20
void Led_Process(void)
{
    static int maxRate[2] = {MAX_CYCLES_PER_SECOND, MAX_CYCLES_PER_SECOND};
    static int brinking_start[2] = {0, 0};
    static int brinking_rate[2] = {0, 0};
    unsigned int i;
    unsigned char status;
    for ( i = 0; i < 2; i++)
    {
        if (brinking_start[i])
        {
            if (maxRate[i])
            {
                status = (maxRate[i] >> brinking_rate[i]) & 1;
                apctlLedSet(i, status);
                maxRate[i]--;
            }
            else
            {
                brinking_start[i] = 0;
                apctlLedSet(i, 1);
                packet_cnt[i] = 0;
                maxRate[i] = MAX_CYCLES_PER_SECOND;
            }
        }
        else
        {
            brinking_start[i] = 1;
            if (packet_cnt[i] > 6)
            {
                apctlGPIOWrite(13, apctlGPIORead(13) ? 0 : 1);
                brinking_rate[i] = 0;
            }
            else if (packet_cnt[i] > 3)
            {
                brinking_rate[i] = 1;
            }
            else if (packet_cnt[i])
            {
                brinking_rate[i] = 2;
            }
            else
                brinking_start[i] = 0;
        }
    }
}
void Ether_Status_Monitor(void)
{
    unsigned int status;
    apctlGPIOWrite(13, apctlGPIORead(13) ? 0 : 1);
    status = (WL_REGS32(ETHER0_PORT_STATUS_REG) & 0x0009);
    if (status != ether_port_status)
    {
        ether_port_status = status;
        apctlLedSet(0, 1);
    }
    if ((status & 0x0008) == 0)
        apctlGPIOWrite(4, 1);
}
#endif 
/* GPIO control API */

/**
 * Initialize the GPIO pin for output or input
 * 
 * @param dirMap The GPIO direction bit map.
 *               Bit set to 1: the corresponding GPIO pin is configured
 *               for input.
 *               Bit set to 0: the corresponding GPIO pin is configured
 *               for output
 */
void apctlGPIOInit(unsigned long dirMap)
{
#if 1
    UINT32 idx;
    UINT32 regVal;
    WL_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, dirMap & 0xFFFF);
    /* Have two GPIO output source select registers, so do two loops */
    regVal = 0;
    for (idx = 0; idx < 8; idx++)
    {
        if (((dirMap >> idx) & 0x01) == 0)
        {
            /* Set to output, so select output source */
            regVal |= (1 << (2 * idx));
        }
    }
    WL_WRITE_WORD(GPIO_REG_SELECT_0_7, regVal);
    dirMap = dirMap >> 8;

    regVal = 0;
    for (idx = 0; idx < 8; idx++)
    {
        if (((dirMap >> idx) & 0x01) == 0)
        {
            /* Set to output, so select output source */
            regVal |= (1 << (2 * idx));
        }
    }
    WL_WRITE_WORD(GPIO_REG_SELECT_8_15, regVal);
#endif
#ifdef AP21N

    WL_WRITE_WORD(SW_LED_OUTPUT_ENABLE, 0);
#endif
}

/**
 * Test if GPIO number is configured as output mode
 * 
 * @param gpioNum the GPIO pin number
 * @return TRUE: output mode
 *         FALSE: input mode
 */
inline int gpioInOutputMode(int gpioNum)
{
#if 1
    UINT32 dirMap;
    int retVal = FALSE;

    if (gpioNum < 8)
    {
        WL_READ_WORD(GPIO_REG_SELECT_0_7, dirMap);
    }
    else
    {
        WL_READ_WORD(GPIO_REG_SELECT_8_15, dirMap);
        /* transform gpio num to a number between 0 and 7 */
        gpioNum -= 8;
    }

    /* Test output mode */
    if (((dirMap >> (gpioNum * 2)) & 0x03) == 0x01)
    {
        retVal = TRUE;
    }

    return retVal;
#endif
}

/**
 * Read the state of a GPIO pin
 * 
 * @param gpioNum the GPIO pin number
 * @return 0: GPIO state is "low"
 *         1: GPIO state is "high"
 *        -1: Error: invalid GPIO number
 */
int apctlGPIORead(int gpioNum)
{
    UINT32 regVal;
#if 1

    if (gpioNum > MAX_GPIO_NUM)
    {
        return -1;
    }

    if (gpioInOutputMode(gpioNum))
    {
        /* GPIO pin is output */
        WL_READ_WORD(GPIO_REG_OUTPUTS, regVal);
        regVal = (regVal >> gpioNum) & 0x01;
    }
    else
    {
        /* GPIO pin is input */
        WL_READ_WORD(GPIO_REG_INPUTS, regVal);
        regVal = (regVal >> gpioNum) & 0x01;
    }
#endif

    return regVal;
}

/**
 * Set the state of a GPIO pin. The new GPIO state
 * after write is returned.
 * 
 * @param gpioNum The GPIO pin number
 * @param state   The state to set:
 *                0: set to "low"
 *                1: set to "high"
 * @return 0: the GPIO state is "low"
 *         1: the GPIO state is "high"
 *        -1: Error: invalid GPIO number
 *        -2: Error: GPIO pin is in input mode
 */
int apctlGPIOWrite(int gpioNum, int state)
{
    UINT32 regVal;
#if 1

    if (gpioNum > MAX_GPIO_NUM)
    {
        return -1;
    }

    if (gpioInOutputMode(gpioNum))
    {
        /* GPIO pin is output */

        /* Read the current setting and preserve what we do not touch */
        WL_READ_WORD(GPIO_REG_OUTPUTS, regVal);

        /* Set the output pin according to state */
        if (state)
        {
            /* Must return state value of 1 or 0 */
            state = 1;
            regVal |= (1 << gpioNum);
        }
        else
        {
            regVal &= ~(1 << gpioNum);
        }

        WL_WRITE_WORD(GPIO_REG_OUTPUTS, regVal);
    }
    else
    {
        /* GPIO pin is in input mode */
        return( -2);
    }
#endif

    return state;
}

/* Timer API */
/**
 * Create a periodic timer. The callback function will run in the
 * Delayed Service Routine of the real-time clock interrupt.
 * 
 * @param period  The period in 100 milliseconds
 * @param timerCbFunc The callback function
 * @return 0: failed to create timer because exceed maximum
 *         number of timers supported
 *         >0: the timer index, to be used in
 *         subsequent call to apctlTimerDelete()
 */
unsigned long apctlTimerCreate(int period, void (*timerCbFunc)(UINT8 *))
{
    int i;
    if (activeAlarmCnt > MAX_TIMER_ALARM - 1)
    {
        return 0;
    }
    for(i = 0; i <MAX_TIMER_ALARM; i++)
    {
        if(timerAlarm[i].inuse == FALSE)
        {
            TimerInit(&(timerAlarm[i].timer));
            TimerFireEvery(&(timerAlarm[i].timer), TRUE, timerCbFunc, NULL, period );
            activeAlarmCnt++;
            timerAlarm[i].inuse = TRUE;
            break;
        }
    }
    return i;
}

/**
 * Delete a timer
 * 
 * @param timerIndex The timer index returned by apctlTimerCreate()
 * @return TRUE: success
 *         FALSE: failure because timer index is invalid
 */
int apctlTimerDelete(unsigned long timerIndex)
{
    if (timerIndex > MAX_TIMER_ALARM)
    {
        return FALSE;
    }
    --activeAlarmCnt;
    TimerDisarm(&(timerAlarm[timerIndex].timer));
    timerAlarm[timerIndex].inuse = FALSE;
    return TRUE;
}
/*
*********************************************************************************************************
*                                          API EVENT CONTROL BLOCK
*********************************************************************************************************
*/
typedef struct
{
    void (*evtCbFunc)(ApEvtMsg_t *);      /* pointer to the event call back function */
    UINT8 apiEvtTbl[API_EVENT_MAX_EVENT]; /* List of events waiting for calling callback */
    UINT8 apiEvtIDAssigned;               /* id assigned flag */
}
API_EVT_CTRL;

API_EVT_CTRL apiEvtCtrl[API_EVENT_MAX_SUB];

/*
 ** apEvtInit
 *
 *  FILENAME: D:\jshen\802.11_MAC-DRIVER_SW\AP\Src\api.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS:
 *
 */
void apEvtInit(void)
{
    return ;
}
/*
 ** apEvtSub
 *
 *  FILENAME: D:\jshen\802.11_MAC-DRIVER_SW\AP\Src\api.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS: TRUE/FALSE
 *
 */
BOOLEAN apEvtSub(unsigned long eventSet, void (*evtCbFunc)(ApEvtMsg_t *), unsigned int *pSubID)
{
    int i;
    static int apEvtInitFlag = 0;
    UINT16 val;
    /*self contained initialization */
    if ( !apEvtInitFlag )
    {
        apEvtInit();
        apEvtInitFlag++;
    }
    /*safe guard */
    if (!evtCbFunc || !eventSet)
        return FALSE;
    if ( eventSet == APEVT_RST_BTN_PRESSED)
    {
        /*    	WL_WRITE_WORD(GPIO_REG_OUTPUT_ENABLE, GPIO_PIN_SW_RESET); */
        WL_READ_WORD(GPIO_REG_INPUTS, val);
        if (val & GPIO_PIN_SW_RESET )
            WL_WRITE_WORD(GPIO_REG_IER, (GPIO_PIN_SW_RESET)<<3);
        else
            WL_WRITE_WORD(GPIO_REG_IER, 0);
        WL_WRITE_WORD(GPIO_REG_IMR, (GPIO_PIN_SW_RESET)<<3);
    }

    for ( i = 0; i < API_EVENT_MAX_SUB; i++)
    {
        if ( !apiEvtCtrl[i].apiEvtIDAssigned)
        {
            /*found an available id space */
            break;
        }
    }
    if ( i == API_EVENT_MAX_SUB )   /* out of id space */
        return FALSE;

    apiEvtCtrl[i].evtCbFunc = evtCbFunc;
    apiEvtCtrl[i].apiEvtIDAssigned = 1;
    *pSubID = i;

    /* set events in list table to active */
    for (i = 0; i < API_EVENT_MAX_EVENT; i++)
    {
        if ( eventSet & (1 << i) )
        {
            apiEvtCtrl[*pSubID].apiEvtTbl[i] = 1;
        }
    }
    return TRUE;
}

/*
 ** apEvtUnsub
 *
 *  FILENAME: D:\jshen\802.11_MAC-DRIVER_SW\AP\Src\api.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS: TRUE/FALSE
 *
 */
BOOLEAN apEvtUnsub(unsigned long eventSet, unsigned int subID)
{
    int i, left = 0;
    if (subID >= API_EVENT_MAX_SUB)
        return FALSE;

    for (i = 0; i < API_EVENT_MAX_EVENT; i++)
    {
        if ( eventSet & (1 << i) )
        {
            /*deactivate the event */
            apiEvtCtrl[subID].apiEvtTbl[i] = 0;
        }
        if (apiEvtCtrl[subID].apiEvtTbl[i])
        {
            /*count active events left */
            left++;
        }
    }
    if (!left)
    {
        /*no active events left, release the id space */
        apiEvtCtrl[subID].evtCbFunc = NULL;
        apiEvtCtrl[subID].apiEvtIDAssigned = 0;
    }
    return TRUE;
}
/*
 ** apEvtRun
 *
 *  FILENAME: D:\jshen\802.11_MAC-DRIVER_SW\AP\Src\api.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS:
 *
 */
void apEvtRun(UINT16 evtCode, UINT8 *body, UINT8 status)
{
    int i;
    APEVT_STA_ASSOC_TYPE apevtmsg;
    UINT16 evtIndex = 0;
    apevtmsg.evtCode = evtCode;
    apevtmsg.evtTimeStamp = cyg_current_time();
    switch (evtCode)
    {
    case APEVT_STA_ASSOC:
        apevtmsg.evtBodyLen = 7;
        memcpy(apevtmsg.staMACAddress, body, 6);
        apevtmsg.assocType = status;
        break;
    case APEVT_RST_BTN_PRESSED:
        apevtmsg.evtBodyLen = 0;
        break;
#ifdef WDS_FEATURE
    case APEVT_UNKNOWN_WDS_ROUTE:
        apevtmsg.evtBodyLen = 6;
        memcpy(apevtmsg.staMACAddress, body, 6);
        break;
#endif
    default:
        apevtmsg.evtBodyLen = 0;
        break;
    }
    while ( !((evtCode >> evtIndex) == 1) )
    {
        evtIndex++;
    }
    if (evtIndex > API_EVENT_MAX_EVENT)
        return ;
    for (i = 0; i < API_EVENT_MAX_SUB; i++)
    {
        if (apiEvtCtrl[i].apiEvtIDAssigned)
        {
            if (apiEvtCtrl[i].apiEvtTbl[evtIndex])
                apiEvtCtrl[i].evtCbFunc((ApEvtMsg_t *)&apevtmsg);
        }
    }
}

/* rf tx power register */
void SetPowerRegValue(UINT8 pw_reg20, UINT8 pw_reg26)
{
    extern UINT8 reg20, reg26;
	reg20 = pw_reg20;
	reg26 = pw_reg26;
}

unsigned char ReadRfReg(unsigned char offset)
{
    UINT8  regValRead;
    msi_wl_ReadPHY(offset, &regValRead);
    return regValRead;
}

void WriteRfReg(unsigned char offset, unsigned char value)
{
    msi_wl_WritePHY(offset, value);
}
#ifdef TURBO_SETUP
int SetSecurityWpa(UINT8 *PasswordData)
{
    unsigned char    chData;
    unsigned char wpaMode;
    BOOLEAN WpaEnabled;
    //if (strstr(tmp, "WPA-PSK")!=NULL)
    {
        chData =  2;
        wpaMode = 1;
    }
/*    else if (strstr(tmp, "WPA-802.1x")!=NULL)
    {
        chData =  1;
        wpaMode = 1;
    }
    else
    {
        wpaMode = 0;
    }*/

    if ( wpaMode==1 )
        WpaEnabled=TRUE;
    else
        WpaEnabled=FALSE;       

    //set Wpa Mode
    {
        APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_PRIVACY_TBL_RSN_ENABLED;
        cmdBuf.cmdBodyLen = 1;
        cmdBuf.Enabled = wpaMode;
        apctl(&cmdBuf);


    }
    //set Wpa Auth Meth
    if ( WpaEnabled==TRUE ) {
        APICMDBUF_SET_RSN_CFG_AUTH_SUITE cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_RSN_CFG_AUTH_SUITE;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_RSN_CFG_AUTH_SUITE) - 4*sizeof(unsigned short);
        cmdBuf.AuthSuites[0] = 0x0;
        cmdBuf.AuthSuites[1] = 0x50;
        cmdBuf.AuthSuites[2] = 0xf2;
        cmdBuf.AuthSuites[3] = chData;
        cmdBuf.Enabled = wpaMode;
        apctl(&cmdBuf);
    }

    // WPA Auth Method
    /*if ( strstr(tmp, "TKIP")!=NULL )*/ 
    {
        chData = 2;
    }
    /*else if ( strstr(tmp, "CCMP")!=NULL ) {
        chData = 4;
    }
    else {
        chData = 0;
    }*/

    //set Wpa Unicast Cipher
    {
        APICMDBUF_SET_RSN_CFG_UNICAST_CIPHERS cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_RSN_CFG_UNICAST_CIPHERS;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_RSN_CFG_UNICAST_CIPHERS) - 4*sizeof(unsigned short);
        cmdBuf.UnicastCipher[0] = 0x0;
        cmdBuf.UnicastCipher[1] = 0x50;
        cmdBuf.UnicastCipher[2] = 0xf2;
        cmdBuf.UnicastCipher[3] = chData;
        cmdBuf.Enabled = wpaMode;
        apctl(&cmdBuf);
    }
    //set Wpa Multicast Cipher
    {
        APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_RSN_CFG_MULTICAST_CIPHERS;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS) - 4*sizeof(unsigned short);
        cmdBuf.MulticastCipher[0] = 0x0;
        cmdBuf.MulticastCipher[1] = 0x50;
        cmdBuf.MulticastCipher[2] = 0xf2;
        cmdBuf.MulticastCipher[3] = chData;
        apctl(&cmdBuf);
    }
    if (WpaEnabled==TRUE)
    {
        APICMDBUF_SET_PRIVACY_OPTION cmdBuf;
        chData = DATA_ENCRYPT_NOT_REQUIRED;
        cmdBuf.cmdCode = APCMD_SET_PRIVACY_OPTION;
        cmdBuf.seqNum = 1;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_PRIVACY_OPTION) - 4*sizeof(unsigned short);
        cmdBuf.dataEncrypt = chData;
        apctl(&cmdBuf);

    }

    //WPA Pass Phrase
    //Set Pass Phrase
    {
        APICMDBUF_SET_PSK_PASSPHRASE cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_PSK_PASSPHRASE;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_PSK_PASSPHRASE) - 4*sizeof(unsigned short);;
        strcpy(cmdBuf.passphrase, PasswordData);
        apctl(&cmdBuf);
    }            
    //Set Grp Rekey Timeout time
    {
        APICMDBUF_SET_GRP_REKEY_TIME cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_GRP_REKEY_TIME;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_GRP_REKEY_TIME) - 4*sizeof(unsigned short);;
        cmdBuf.rekey_time = 24*60*60;
        apctl(&cmdBuf);
    }            

    SetUserConfig();  
    return 0; 
}

int SetSecurityWep(UINT8 *key, UINT8 keytype, UINT8 keyindex)
{
    unsigned char    chData;
    unsigned int    keysize;

/*  if (strstr(tmp, "Open")!=NULL)
    {
        chData = OPEN_SYSTEM_ONLY;
    }
    else if ( strstr(tmp, "Shared")!=NULL )
    {
        chData = SHARED_KEY_ONLY;
    }
    else /*if (strstr(tmp, "Both")!=NULL) */
    {
        chData = OPEN_SYSTEM_AND_SHARED_KEY;
    }
    {
        APICMDBUF_SET_AUTH_MODE cmdBuf;

        cmdBuf.seqNum = 1;
        cmdBuf.cmdCode = APCMD_SET_AUTH_MODE;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_AUTH_MODE) - 4*sizeof(unsigned short);;
        cmdBuf.authMode = chData;
        apctl(&cmdBuf);
    }            
    /* Privacy Option */
/*    if (strstr(tmp, "Not")!=NULL)
    {
        chData = DATA_ENCRYPT_NOT_REQUIRED;
    }
    else*/
    {
        chData = FULL_DATA_ENCRYPT_REQUIRED;
    }

    {
        APICMDBUF_SET_PRIVACY_OPTION cmdBuf;
        cmdBuf.cmdCode = APCMD_SET_PRIVACY_OPTION;
        cmdBuf.seqNum = 1;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_PRIVACY_OPTION) - 4*sizeof(unsigned short);
        cmdBuf.dataEncrypt = chData;
        apctl(&cmdBuf);

    }
    {
        APICMDBUF_SET_DEFAULT_WEP_KEYS cmdBuf;

        cmdBuf.cmdCode = APCMD_SET_DEFAULT_WEP_KEYS;
        cmdBuf.seqNum = 1;
        cmdBuf.cmdBodyLen = sizeof(APICMDBUF_SET_DEFAULT_WEP_KEYS) - 4*sizeof(unsigned short);
        cmdBuf.keyindex = keyindex;  
        cmdBuf.keyType0 = keytype; 
        cmdBuf.keyType1 = keytype; 
        cmdBuf.keyType2 = keytype; 
        cmdBuf.keyType3 = keytype; 
        if(keytype == 1)
        {
            memcpy(cmdBuf.keyValue0, key, 5);
            memcpy(cmdBuf.keyValue1, key+5, 5);
            memcpy(cmdBuf.keyValue2, key+10, 5);
            memcpy(cmdBuf.keyValue3, key+15, 5);
        }else if(keytype == 2)
        {
            memcpy(cmdBuf.keyValue0, key, 13);
            memcpy(cmdBuf.keyValue1, key, 13);
            memcpy(cmdBuf.keyValue2, key, 13);
            memcpy(cmdBuf.keyValue3, key, 13);
        }
        apctl(&cmdBuf);
    }

    SetUserConfig();  
    return 0;
}
#endif
