/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    dist.c
 * \brief   Distribution service module API module
 *
 * $Revision: 1.30 $
 * $Date: 2004/04/29 02:11:58 $
 */

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "osif.h"
#include "memmap.h"
#include "idma.h"
#include "tkip.h"
#include "rx80211.h"
#include "ds.h"
#include "sysinfo.h"
#include "StaDb.h"
#include "tx80211.h"
#include "txsched.h"
#include "macMgmtMlme.h"
#include "etherlow.h"
#include "ether.h"
#include "ap_api.h"
#include "ap_mib.h"
#include "wlmac.h"
#include "wl_macros.h"

#include "wltypes.h"
#include "bufmgt.h"
#include "eth_debug.h"
#include "stats.h"
//#include "api.h"
#include "wpa.h"

#ifdef ENABLE_RATE_ADAPTATION_BASEBAND
    #include "rateadapt.h"
#endif


/*#define __ECOS */
#ifndef NO_ECOS_TCP_IP_STACK
#include <pkgconf/net.h>
#include <pkgconf/net_w81_eth_drivers.h>
#include <netdev.h>
#endif /* ECOS_TCP_IP_STACK */

#include <cyg/infra/cyg_type.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/infra/diag.h>
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_cache.h>

#ifdef USE_NEW_OSIF
static QUEUE_HANDLE llc_dataQH; 
#endif
extern QHANDLE ethRxQ, ethTxQ, wlRxQ, wlTxQ;

extern UINT32 eaglehw_GetPhyRateIndex(UINT32 Rate);

extern int femac_stuffed_byte_cnt;
/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */

#define PROCEED_AFTER_DMA_DONE  1


#define DS_EVENTS ds_802_11_DATA_RCVD  | \
                  ds_802_3_DATA_RCVD   | \
                  ds_MGMT_NOTIFICATION_RCVD | \
                  ds_LLC_DATA_RCVD | \
				  ds_802_3_TX_DONE | \
				  ds_802_11_TX_DONE | \
                  ds_POLLING_TIMEOUT_RCVD

extern void TestTask(OS_DATA Data);
extern UINT32 EtherEventSentFlag;
extern UINT8 dma0_busy;
volatile UINT32 BufferErrorflag=0;

extern void w8100_eth_RxEvent(int length);
extern struct eth_drv_sc *sc1;
extern void SetDurationId(txInfo_t *txInfo_p);
UINT32 TxOnWlan(IEEEtypes_DataFrame_t * Data11Frame_p , UINT8 RateIndex, UINT16 StnId, UINT8 shortpreamble);
UINT32 apio_wlanRead(apio_bufdescr_t *ppBufDescr);
UINT32 apio_wlanWrite(apio_bufdescr_t *ppBufDescr);
UINT32 * apio_GetWlanTxBuf(void);
UINT32 * apio_GetEtherTxBuf(void);
UINT32 apio_FreeWlanRxBuf(apio_bufdescr_t *pBufDescr);
extern UINT32 apio_etherWrite( apio_bufdescr_t *pBufDescr);
extern UINT32 apio_etherRead(apio_bufdescr_t *pBufDescr);


/*============================================================================= */
/*                              TYPE DEFINITIONS */
/*============================================================================= */
/* */


/*============================================================================= */
/*                         IMPORTED PUBLIC VARIABLES */
/*============================================================================= */
extern IEEEtypes_MacAddr_t macBssId;
extern BOOLEAN dot11WepEnabled;
unsigned char * rcvdData;
unsigned char moreWlPkts;
extern BOOLEAN Decrypt(UINT32 des, UINT32 src, UINT32 len);

extern apio_ErrCode_t apio_wlanwrite( apio_bufdescr_t *pBufDescr);

#ifdef ERP
extern UINT32 eaglehw_GetPhyRxRateIndex(RxSign_t *RxSign_p);
#endif

#if 0
/*Some rate adaptation Variables. */
#ifdef ENABLE_ADAPTIVE_BAILOUT
extern UINT8 BailOutRate;
extern UINT8 BailOutFlag; /*If 1 => BailOutRateset to 11Mbps. */
/* If 0 => BailOutRate set to 1Mbps. */
extern UINT8 LowestRate;
extern UINT8 LowestRateAid;
#endif /*ENABLE_ADAPTIVE_BAILOUT */
#ifdef ENABLE_POWER_ADAPT
extern UINT8 POWER_ADAPT; /*1 to near/far range power adaptation */
extern UINT8 POWER_FLAG; /*0 means PA is adjusted to 74 (near range).. 1 means it is 68. (far range) */
extern UINT8 RFReg20;
extern UINT8 RFReg20Calibrated;
extern UINT8 RFReg26;
extern UINT8 RFReg26Calibrated;
extern UINT8 HighestRate;
extern UINT8 HighestRateAid;
#endif /*POWER_ADAPT */
#endif

/*============================================================================ */
/*                          MODULE LEVEL VARIABLES */
/*============================================================================= */

llc_snap_hdr_t llcSnap __SQRAM__;

/*extern UINT8 GetStationRate2(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT8 Aid); */
/*UINT8 (*GetStationRateFp)(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT8 Aid) =GetStationRate; */


/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

void InitDefaultHeader(IEEEtypes_GenHdr_t * Hdr_p);
void MemDma(UINT32 x, UINT32 y, UINT32 z);
void DoDmaDone(void);
void InitEtherWLCopyPool();

#ifdef DMA_INTR
inline void MemDma(UINT32 x, UINT32 y, UINT32 z)
{
    while ( dma0_busy )
        ;
    dma0_busy = 1;

    WL_REGS32(WL_IDMA_BYTECNT0) = z;
    WL_REGS32(WL_IDMA_SRCADDR0) = y;
    WL_REGS32(WL_IDMA_DESADDR0) = x;
    WL_REGS32(WL_IDMA_NEXTRECPTR0) = 0;
    WL_REGS32_SETBITS(WL_IDMA_CH0_CTRL, DCC_CHANEN | DCC_BLOCK_MODE | DCC_DTL_32
                      0x400);

}
#else
inline void MemDma(UINT32 x, UINT32 y, UINT32 z)
{
    UINT32 Chan0Ctrl;
#if 0
    while ( Chan0Ctrl &0x4000 )
    {
        WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    }
#endif
    WL_WRITE_WORD(WL_IDMA_CH0_CTRL, 0);

    WL_REGS32(WL_IDMA_BYTECNT0) = z;
    WL_REGS32(WL_IDMA_SRCADDR0) = y;
    WL_REGS32(WL_IDMA_DESADDR0) = x;
    WL_REGS32(WL_IDMA_NEXTRECPTR0) = 0;
    WL_REGS32_SETBITS(WL_IDMA_CH0_CTRL, DCC_CHANEN | DCC_BLOCK_MODE | DCC_DTL_32 |
                      0x400);
#if 0
    WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    while ( Chan0Ctrl &0x4000 )
    {
        WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    }
#endif
}

inline void MemDmaWPA(UINT32 x, UINT32 y, UINT32 z)
{
    UINT32 i;
    UINT32 Chan0Ctrl;
#if 0
    while ( Chan0Ctrl &0x4000 )
    {
        WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    }
#endif
/*
    while ((WL_REGS32(RX_WEP_CMD)&(WEP_CMD_TXWEP_BUSY | WEP_CMD_TXWEP_TRIG)))
    {
        for (i = 0;i < 30;i++)
            ;
    }
*/    
    WL_WRITE_WORD(WL_IDMA_CH0_CTRL, 0);

    WL_REGS32(WL_IDMA_BYTECNT0) = z;
    WL_REGS32(WL_IDMA_SRCADDR0) = y;
    WL_REGS32(WL_IDMA_DESADDR0) = x;
    WL_REGS32(WL_IDMA_NEXTRECPTR0) = 0;
    
    WL_REGS32_SETBITS(WL_IDMA_CH0_CTRL, DCC_CHANEN | DCC_BLOCK_MODE | DCC_DTL_32 |
                      0x400);

#if 1
/*
    WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    while ( Chan0Ctrl &0x4000 )
    {
        WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    }
*/

    for (i = 0;i < 50;i++)
            ;
    while(WL_REGS32(WL_IDMA_CH0_CTRL) & 0x4000)
    {
        for (i = 0;i < 30;i++)
            ;
    }
    
#endif
}

inline void CheckDmaDone(void)
{
    UINT32 Chan0Ctrl;

    WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
#if PROCEED_AFTER_DMA_DONE

    while ( Chan0Ctrl &0x4000 )
    {
        WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    }
#else
    if ( Chan0Ctrl & 0x4000 )
    {
        Stats.DmaNotFinishedCnt++;
    }
#endif
}
#endif

void MemDma1(UINT32 x, UINT32 y, UINT32 z)
{

    UINT32 Chan1Ctrl;
#if 0
    while ( Chan1Ctrl &0x4000 )
        WL_READ_WORD(WL_IDMA_CH1_CTRL, Chan1Ctrl);
#endif
    WL_WRITE_WORD(WL_IDMA_CH1_CTRL, 0);

    WL_REGS32(WL_IDMA_BYTECNT1) = z;
    WL_REGS32(WL_IDMA_SRCADDR1) = y;
    WL_REGS32(WL_IDMA_DESADDR1) = x;
    WL_REGS32(WL_IDMA_NEXTRECPTR1) = 0;
    WL_REGS32_SETBITS(WL_IDMA_CH1_CTRL, DCC_CHANEN | DCC_BLOCK_MODE | DCC_DTL_32 |
                      0x400);
#if 0
    WL_READ_WORD(WL_IDMA_CH1_CTRL, Chan1Ctrl);
    while ( Chan1Ctrl &0x4000 )
        WL_READ_WORD(WL_IDMA_CH1_CTRL, Chan1Ctrl);
#endif
}

void MemDma1WPA(UINT32 x, UINT32 y, UINT32 z)
{

    UINT32 Chan1Ctrl;
    UINT32 i;
#if 0
    while ( Chan1Ctrl &0x4000 )
        WL_READ_WORD(WL_IDMA_CH1_CTRL, Chan1Ctrl);
#endif
    WL_WRITE_WORD(WL_IDMA_CH1_CTRL, 0);

    WL_REGS32(WL_IDMA_BYTECNT1) = z;
    WL_REGS32(WL_IDMA_SRCADDR1) = y;
    WL_REGS32(WL_IDMA_DESADDR1) = x;
    WL_REGS32(WL_IDMA_NEXTRECPTR1) = 0;
    
    WL_REGS32_SETBITS(WL_IDMA_CH1_CTRL, DCC_CHANEN | DCC_BLOCK_MODE | DCC_DTL_32 |
                      0x400);
#if 1
/*
    WL_READ_WORD(WL_IDMA_CH1_CTRL, Chan1Ctrl);
    while ( Chan1Ctrl &0x4000 )
    {
        WL_READ_WORD(WL_IDMA_CH1_CTRL, Chan1Ctrl);
    }
*/

    for (i = 0;i < 50;i++)
            ;
    while(WL_REGS32(WL_IDMA_CH1_CTRL) & 0x4000)
    {
        for (i = 0;i < 30;i++)
            ;
    }
    
#endif
}
/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */


/******************************************************************************
 * 
 * Name: ds_Init 
 * 
 * Description: 
 *   This routine is called to initialize the the Distribution Service Task 
 *   and related components. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *   Create dsMgmtQ  
 *   If the queue was successfully created Then 
 *      Create the Distribution Service Task by calling os_TaskCreate() 
 *      If creating the Distribution Service Task succeeded Then  
 *         Return OS_SUCCESS 
 *      End If 
 *   End If 
 * 
 *   Return OS_FAIL 
 * END PDL 
 * 
 *****************************************************************************/
extern WL_STATUS ds_Init( void )
{
#ifdef USE_NEW_OSIF
    if ( os_QueueCreate(&llc_dataQH) != OS_SUCCESS )
    {
        return(OS_FAIL);
    }
    os_EventInit(&dsEH);
#else
    if ( os_QueueCreate(sysinfo_DS_LLC_DATA_Q) != os_SUCCESS )
    {
        return(OS_FAIL);
    }
    if ( os_QueueCreate(sysinfo_WPA_Q) != os_SUCCESS )
    {
        return(OS_FAIL);
    }
    os_EventInit(sysinfo_DS_EVENT);
#endif
    InitEtherWLCopyPool();
    llcSnap.Dsap = 0xAA;
    llcSnap.Ssap = 0xAA;
    llcSnap.Control = 0x03;
    llcSnap.Org[0] = 0;
    llcSnap.Org[1] = 0;
    llcSnap.Org[2] = 0;
    return(OS_SUCCESS);

}

void InitEtherWLCopyPool()
{
    return ;
}


/******************************************************************************
 * 
 * Name: ds_SendEvent 
 * 
 * Description: 
 *   This routine provides the mechanism to trigger an event indicating to 
 *   the Distribution Service Task that a message has been written to one 
 *   of its queues or data is available for processing. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   ds_Init(). 
 * 
 * Arguments: 
 *   Arg1 (i  ): Event - value indicating where the data or message is 
  *                       available for processing. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 *    If Event is ds_802_11_DATA_RCVD or ds_802_3_DATA_RCVD or 
 *       ds_MGMT_NOTIFICATION_MSG_RCVD  
 *    Then 
 *       Call os_EventTrigger to trigger the received event 
 *       Return OS_SUCCESS 
 *    Else 
 *       Return OS_FAIL 
 *    End If 
 * END PDL 
 * 
 *****************************************************************************/
extern WL_STATUS ds_SendEvent( UINT16 Event )
{


    if ( Event == ds_802_11_DATA_RCVD ||
         Event == ds_802_3_DATA_RCVD ||
         Event == ds_MGMT_NOTIFICATION_RCVD ||
         Event == ds_LLC_DATA_RCVD ||
         Event == ds_802_3_TX_DONE ||
         Event == ds_802_11_TX_DONE ||
         Event == ds_POLLING_TIMEOUT_RCVD ||
         Event == ds_POLLING_WPA_MSG)
    {
#ifdef USE_NEW_OSIF
        os_EventTrigger(dsEH, Event);
#else
        os_EventTrigger(sysinfo_DS_EVENT, Event);
#endif
        return(OS_SUCCESS);
    } else
    {
        return(OS_FAIL);
    }
}


/*============================================================================= */
/*                         CODED PRIVATE PROCEDURES */
/*============================================================================= */
/******************************************************************************
* 
* Name:InitDefaultHeader 
* 
* Description: 
 
* 
* Conditions For Use: 
*   None
* 
* Arguments: 
*   Arg1 (i  ): 
* 
* Return Value: 
*   None. 
* 
* Notes: 
*   None. 
* 
* PDL: 
* 
*****************************************************************************/
inline void InitDefaultHeader(IEEEtypes_GenHdr_t * Hdr_p)
{
    *(UINT16 *)&Hdr_p->FrmCtl = 0;

    Hdr_p->FrmCtl.Type = IEEE_TYPE_DATA;
    /* subtype is already set to DATA = 0 */
    Hdr_p->SeqCtl = 0;
}
/******************************************************************************
 * 
 * Name: ProcessLlcData
 * 
 * Description: 
 
 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 * PDL: 
 * 
 *****************************************************************************/
#ifndef NO_ECOS_TCP_IP_STACK
UINT32 ProcessLlcData(void)
{
    extern apio_handle_t llcHandle;
    ether_hdr_t *EtherHdr_p, *EtherHdr2_p;

    BOOLEAN BcastFlag;
    IEEEtypes_MacAddr_t da, sa;
    extStaDb_StaInfo_t *StaInfo_p;
    UINT32 Len;

    UINT32 noPkts = 0;
    ether_hdr_t EthHdr;

    dsQ_LlcData_t *LlcData_p;
    extStaDb_Status_e SearchResult;
    apio_bufdescr_t BufDescr;
#ifdef USE_NEW_OSIF
    while ( (   LlcData_p = os_QueueReadNoBlock(llc_dataQH)) )
#else
    while ( (   LlcData_p = os_QueueReadNoBlock(sysinfo_DS_LLC_DATA_Q)) )
#endif
    {
        noPkts++;
        EtherHdr_p = (ether_hdr_t *)LlcData_p->data;
        Len = LlcData_p->totalLen;
        MACADDR_CPY(&da, &EtherHdr_p->da);
        MACADDR_CPY(&sa, &EtherHdr_p->sa);
        EthHdr.type = EtherHdr_p->type;
        BcastFlag = FALSE;

        if ( IS_GROUP((UINT8 *)&da) )
            BcastFlag = TRUE;
        if ( ((StaInfo_p = extStaDb_GetStaInfo(&da, 0)) != NULL &&
              StaInfo_p->State == ASSOCIATED) || BcastFlag )
        {

            if ( (EtherHdr2_p = (ether_hdr_t *) apio_GetWlanTxBuf()) == NULL )
            {
                /* free the llc msg */
                eth_drv_tx_done(sc1,LlcData_p->Key,0);
                msi_wl_FreePktBuf(LlcData_p);
                continue;
            }


            memcpy((UINT8 *)EtherHdr2_p, (UINT8*)EtherHdr_p, Len);
            BufDescr.frameLen = Len;
            BufDescr.framePtr = (UINT8*)EtherHdr2_p;
            apio_wlanWrite(&BufDescr);
        }

        if ( BcastFlag || (StaInfo_p == NULL ) )
        {
            /* to ethernet port */

            if ( (EtherHdr_p = (ether_hdr_t *) apio_GetEtherTxBuf()) == NULL )
            {
                /*call the tx done and free the llc msg */
                eth_drv_tx_done(sc1,LlcData_p->Key,0);
                msi_wl_FreePktBuf(LlcData_p);
                continue;
            }
            /*memcpy(EtherHdr_p,  LlcData_p->data,   LlcData_p->totalLen); */

            memcpy((UINT32)EtherHdr_p,  (UINT32)LlcData_p->data,   LlcData_p->totalLen);

            BufDescr.frameLen = Len;
            BufDescr.framePtr = (UINT8*)EtherHdr_p;
            apio_etherWrite(&BufDescr);

        }

        eth_drv_tx_done(sc1, LlcData_p->Key, 0);
        msi_wl_FreePktBuf(LlcData_p);
    } /* while end */
    return (noPkts);
}


#ifdef WDS_FEATURE
UINT32 ProcessLlcDataWds(void)
{
    extern apio_handle_t llcHandle;
    ether_hdr_t *EtherHdr_p, *EtherHdr2_p;

    BOOLEAN BcastFlag;
    IEEEtypes_MacAddr_t da, sa;
    extStaDb_StaInfo_t *StaInfo_p;
    UINT32 Len;

    UINT32 noPkts = 0;
    ether_hdr_t EthHdr;

    dsQ_LlcData_t *LlcData_p;
    extStaDb_Status_e SearchResult;
    apio_bufdescr_t BufDescr;
#ifdef USE_NEW_OSIF
    while ( (   LlcData_p = os_QueueReadNoBlock(llc_dataQH)) )
#else
    while ( (   LlcData_p = os_QueueReadNoBlock(sysinfo_DS_LLC_DATA_Q)) )
#endif
    {
        noPkts++;

        BufDescr.framePtr = (UINT8 *)LlcData_p->data;
        BufDescr.frameLen = LlcData_p->totalLen;
        DSForwardPacket(llcHandle,&BufDescr);
    } /* while end */
    return (noPkts);
}
#endif /* WDS_FEATURE */
#endif /* ECOS_TCP_IP_STACK */


/******************************************************************************
 *
 * Name: dsQ_LlcDataWriteNoBlock
 *
 * Description:
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
/*this function is called from ecos driver */
extern int dsQ_LlcDataWriteNoBlock(dsQ_LlcData_t *LlcData_p )
{
    int status;
#ifdef USE_NEW_OSIF
    if ( (status = os_QueueWriteNoBlock(llc_dataQH,
#else
    if ( (status = os_QueueWriteNoBlock(sysinfo_DS_LLC_DATA_Q,
#endif
                                        LlcData_p)) == OS_SUCCESS )
    {
        ds_SendEvent(ds_LLC_DATA_RCVD);

        return(OS_SUCCESS);
    } else
    {
        msi_wl_FreePktBuf(LlcData_p);
        return(OS_SUCCESS);
    }
}

/******************************************************************************
 *
 * Name: dsQ_WpaDataWriteNoBlock
 *
 * Description:
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
//this function is called from the macMgmtMlme driver
extern int dsQ_WpaDataWriteNoBlock(DistTaskMsg_t *pDistMsg)
{
    if (os_QueueWriteNoBlock(sysinfo_WPA_Q, pDistMsg) != OS_SUCCESS)
    {
        msi_wl_FreePktBuf((char *)pDistMsg);
        return 0;
    }
    //KeyMgmtHskCtor(&StaInfo_p->keyMgmtHskHsm);
    //KeyMgmtHskCtor(StaInfo_p);
    //HsmOnStart((Hsm *)&StaInfo_p->keyMgmtHskHsm);
    ds_SendEvent(APIO_POLL_WPA_MSG);
    //ds_SendEvent(1 << 8);
    return 1;
}

/******************************************************************************
 *
 * Name: ds_GetLlcDataBuf
 *
 * Description:
 *
 * Conditions For Use:
 *   The queue has been initialized by calling macMgmtQ_Init().
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
/*ecos driver use this to allocate buffer */
dsQ_LlcData_t *ds_GetLlcDataBuf(void)
{
    UINT8 *data_p =  msi_wl_AllocPktBuf(ethTxQ);
    return((dsQ_LlcData_t *)data_p);
}



/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern UINT32 apio_etherWrite( apio_bufdescr_t *pBufDescr)
{
#if 0
    if ( tx8023Q_DataWriteNoBlock(pBufDescr) == OS_FAIL )
        msi_wl_FreePktBuf(pBufDescr->framePtr );

#else

    USER_BUF_INFO bd;
    if ( ether_port_down )
    {
        msi_wl_Eth_free_tx_queue (&eth_dev,0);
        msi_wl_FreePktBuf(pBufDescr->framePtr );
        return (-1);
    }

#ifdef AP_USE_BUFFER_OFFSET
    *(pBufDescr->framePtr-1) = AP_BUFFER_OFFSET;
#endif

    bd.framePtr = pBufDescr->framePtr;
    bd.frameLen = pBufDescr->frameLen;

    while(msi_wl_EthTransmit(&eth_dev, &bd)==FALSE)
    {
        os_TaskDelay(2);
    }
    return (0);
#endif
}
/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern UINT32 apio_etherRead(apio_bufdescr_t *pBufDescr)
{
#if 1
    UINT32 served;
    ETHERNET_PRIV *ether_priv;
    USER_BUF_INFO bd;
    UINT8 *real_data;

    USR_BUF_DESC *usp;
        served = msi_wl_EthReceive(&eth_dev, &bd);

        if ( served )
        {
            /* Need to set back pointer for buffer managment. */

            usp = msi_wl_GetUsrBufPtr(bd.framePtr);
            if(!usp)
            {
                return (0); 
            }
            /* This is needed for our new buffer management. */
            real_data = msi_wl_AdjtUsrHdrPtr(usp, femac_stuffed_byte_cnt);

            if ( real_data == NULL )  /* not a known buffer structure */
            {
                msi_wl_FreeUsrBuf(usp);
                bd.framePtr = 0;
                bd.frameLen = 0;
                served--;
            } else
                bd.framePtr = real_data;
            bd.frameLen +=femac_stuffed_byte_cnt;
            pBufDescr->framePtr = bd.framePtr;
            usp->byte_count = bd.frameLen;
        }
    ether_priv = (ETHERNET_PRIV *)eth_dev.PrivData;
    EtherEventSentFlag &= ~(0x100);
        GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ether_priv->port ,
                      ~(unsigned int)(0x101));
    if ( served )
    {
        pBufDescr->frameLen = bd.frameLen;
#ifdef AP_USE_BUFFER_OFFSET
        *(pBufDescr->framePtr-1) = AP_BUFFER_OFFSET;
#endif
    } else
    {
        /* if not more pkts, enable the intr */
        GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ether_priv->port ,
                      ~(unsigned int)(0x00ff0101));
        GT_REG_WRITE (ETHER0_SDMA_COMM_REG + ETH_ADDR_GAP * ether_priv->port , BIT7);
    }
    return(served);
#else
    UINT32 Len, result;
    ETHERNET_PRIV *ether_priv;
    result = w8100_eth_receive(&eth_dev, 0, (int *) & pBufDescr->framePtr,
                               &Len );

    ether_priv = (ETHERNET_PRIV *)eth_dev.PrivData;
    EtherEventSentFlag &= ~(0x100);
    GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ether_priv->port ,
                  ~(unsigned int)(0x101));
    if ( result )
    {
        pBufDescr->frameLen = Len;
    } else
    {
        /* if not more pkts, enable the intr */
        GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ether_priv->port ,
                      ~(unsigned int)(0x00ff0101));
        GT_REG_WRITE (ETHER0_SDMA_COMM_REG + ETH_ADDR_GAP * ether_priv->port , BIT7);
    }
    return(result);
#endif
}

/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
void apio_FreeEtherRxBuf(apio_bufdescr_t *pBufDescr)
{
    msi_wl_FreePktBuf(pBufDescr->framePtr);
}

/******************************************************************************
 *
 * Name: 
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   None: 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
UINT32 * apio_GetEtherTxBuf(void)
{
    UINT8 *data;


    if ( (data = msi_wl_AllocPktBuf(ethTxQ)) != NULL )
    {
        return((UINT32 *)data);
    } else
    {
        return(NULL);
    }

}

#ifndef NO_ECOS_TCP_IP_STACK
void SendToStack(IEEEtypes_8023_Frame_t *frame_p,int length)
{
    if((frame_p->Hdr.type == 0x0008)                /*ip*/
    && ((frame_p->Body[6] & 0x4) || frame_p->Body[7])  /*fragment*/
    && (frame_p->Body[9] == 0x01))               /*ICMP*/
        return;
    rcvdData = (UINT8 *)frame_p;
    w8100_eth_RxEvent(length );
}
#endif
