/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/src/fragment.c,v 1.9 2004/01/20 04:01:04 bwswang Exp $
*
* Purpose:
*    This file contains the implementations of the fragment job. The DMA will
*    be started here and all the headers will be filled and moved to Tx Buffer.
*
* Public Procedures:
*		WL_TxMacDataServiceInit
*		WL_TxMacDataServiceStart
*
* Private Procedures:
*
* Notes:
*    None.
*
*****************************************************************************/ 
/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "osif.h"
#include "buildModes.h"
#include "oem.h"
#include "sysinfo.h"
#include "IEEE_types.h"
#include "memmap.h"
#include "timer.h"
#include "wl_mib.h"
#include "tkip.h"
#include "wl_hal.h"
#include "stadb.h"
#include "bufmgt.h"
#include "Fragment.h"
#include "ap_mib.h"
#include "idma.h"
#include "wl_macros.h"
#include "mcu.h"


#define HIGHEST_11B_RATE_REG_INDEX   4
typedef struct Frag_t
{
    WLAN_TX_FRAME* TxFrame;
    UINT8* wlanPdu; /*pointer to frame without 802.3 or 802.11 header */
    UINT32 wlanPduLen; /*lehgth of frame minus 802.3 or 802.11 header */
    UINT32 DmaDb_p;
    UINT8 FragCnt;
UINT8 PsFlag:
    1;   /*the des station's ps mode */
UINT8 reserved:
    7;
}
Frag_t;

/* Imported functions */

extern QHANDLE wlRxQ, wlTxQ;
/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */

#define MAX_DMADB_NUM         12
#define MAX_FRAGOUT_NUM       MAX_BUF_UNIT*TX_BUF_NUM
#define WL_80211_HEADER_SIZE 32
#define FRAGMENT_HD_SIZE      32
#define FCS_SIZE              4
#define NUM_DMA_MAP           2     /*represent 32 DMA BD's */
#define IEEE80211_SIFS        10    /*10us */
#define IEEE80211_SHORT_PREAM 56    /*56us */
#define IEEE80211_LONG_PREAM 128   /*128us */
#define IEEE80211b_SHORT_PREAM 56    /*56us */
#define IEEE80211b_LONG_PREAM 128   /*128us */
#define IEEE80211g_SHORT_PREAM 8    /*56us */
#define IEEE80211g_LONG_PREAM 16   /*128us */
#define CTS_BASIC_DUR         112   /*14*8 */
#define WEPOVERHEAD           8 

/*#define SHORT_PREAMBLE        1 */
#define LLC_SNAP_SIZE         6

/*============================================================================= */
/*                              TYPE DEFINITIONS */
/*============================================================================= */

typedef struct DMADesBlck_t
{
    UINT32 ByteCt;
    UINT32 SrcAddr;
    UINT32 DestAddr;
    UINT32 NextAddr;
}
PACK_END DMADesBlck_t;


typedef struct DMADBArry_t
{
    DMADesBlck_t DmaDB[MAX_DMADB_NUM];
    UINT16 tail;
    UINT16 cnt;
}
PACK_END DMADBArry_t;


/*============================================================================= */
/*                         IMPORTED PUBLIC VARIABLES */
/*============================================================================= */
extern MIB_OP_DATA mib_OpData;

extern UINT8 dma0_busy;
extern volatile UINT32 txwep_busy;
extern UINT8 KeyId;
extern UINT8 WepTypeAll;

/*============================================================================= */
/*                          MODULE LEVEL VARIABLES */
/*============================================================================= */

extern UINT32 IV __SQRAM__;


DMADBArry_t DmaDBArry __SQRAM__;


/*CTS & ACK durations in microsecond at 1,2,5.5,11,22,54 Mbit/s */
UINT8 Cts_Ack_Dur[IEEEtypes_MAX_DATA_RATES_G]  __SQRAM__= {112,56,21,11,6,19,13,10,7,5,4,3,3,2};
UINT8 llc_snap[6] __SQRAM__ = {0xaa, 0xaa, 0x3, 0x0, 0x0, 0x0};

UINT32 DRIVERDOWEP __SQRAM__;
UINT32 SOFTWEP __SQRAM__;
UINT32 macRtsRateId  = 3;  /*11M */
UINT32 macgAckRateId = 5;  /*6M */

//DeFragBufInfo_t DeFragBufInfo[3];/*Data to keep WPA Defragmentation Buffer info*/
//static UINT32 timeout_val = 20000;

/*============================================================================= */
/*                   IMPORTED PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

#define DMAARRAY_HEAD  DmaDBArry.head
#define DMAARRAY_CNT   DmaDBArry.cnt


#ifdef WEP_ITCM
    #if WEP_ITCM > 1
extern void wep_encrypt(unsigned char *buf, int len) __attribute__ ((section (".itcmbuf")));
    #else
extern void wep_encrypt(unsigned char *buf, int len) __attribute__ ((section (".data")));
    #endif
extern void (*wep_encryptFp)(unsigned char *, int);
#else
extern void wep_encrypt(unsigned char *buf, int len);
#endif

extern void DoEncrypt(UINT32 des, UINT32 src, UINT32 len);
void FragDoDMA(DMADesBlck_t*);
void FragDoDMAnEncrypt(UINT32, UINT32, UINT32);
UINT8 GetCtsAckDur(UINT8 rate);
UINT16 GetDataRate(UINT32 rateid);
UINT32 DoFragAndWep(Frag_t *Frag_p);

/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */
/******************************************************************************
 *
 * Name: InitFragBuf
 *
 * Description:
 *   This routine is called by Tx  task to init fragment structures
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   buffer pointer
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
void InitFragBuf(void)
{
    int i;


    for (i = 0; i < MAX_DMADB_NUM - 1; i++)
    {
        DmaDBArry.DmaDB[i].NextAddr = (UINT32) & (DmaDBArry.DmaDB[i + 1]);
    }
    DmaDBArry.DmaDB[i].NextAddr = 0;
    DmaDBArry.tail = 0;
    DmaDBArry.cnt = MAX_DMADB_NUM;


    IV = WL_REGS32(TX_TSF_LO);

    /* */
    /* if the chip rev is 4 and under, use software wep */
    /* */
    /*if(ChipRev > 4) */
    /*{ */
    DRIVERDOWEP = 0;
    SOFTWEP = 0;
    /*} */
    /*else */
    /*{ */
    /*       DRIVERDOWEP = 1; */
    /*       SOFTWEP = 1; */
    /*} */

}
/******************************************************************************
 *
 * Name: FragAndTx
 *
 * Description:
 *   Setsup the Frag descriptor and calls the FragAndWep function
 *
 * Conditions For Use:
 *   Called only if the pkt is to fragmented.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Success or Failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
UINT32 FragAndTx(WLAN_TX_FRAME *txFrame_p)
{
    Frag_t Frag;
    Frag.TxFrame = txFrame_p;
    Frag.wlanPduLen = txFrame_p->IEEEFrame->Hdr.FrmBodyLen;
    Frag.wlanPdu = ((UINT8 *)(txFrame_p->IEEEFrame)) + sizeof(IEEEtypes_GenHdr_t);
    DoFragAndWep(&Frag);
    return OS_SUCCESS;

}
/******************************************************************************
 *
 * Name: FragRequired
 *
 * Description:
 *   This routine checks if fragmentation is required or not
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   TRUE or FALSE
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
BOOLEAN FragRequired(WLAN_TX_FRAME *txFrame_p)
{
    UINT8* FrmBody;
    UINT16 FragThreshold;
    int llc_size = LLC_SNAP_SIZE;
    int WepOverhead = 0;
    BOOLEAN MgmtFrame;
    Frag_t Frag;

    FrmBody = ((UINT8*)(txFrame_p->IEEEFrame)) + WL_80211_HEADER_SIZE;
    Frag.TxFrame = txFrame_p;
    Frag.wlanPduLen = txFrame_p->IEEEFrame->Hdr.FrmBodyLen;
    Frag.wlanPdu = ((UINT8 *)(txFrame_p->IEEEFrame)) + sizeof(IEEEtypes_GenHdr_t);
    /* */
    /* Determine the wep overhead, so proper space can be reserve for wep */
    /* */
    if (mib_PrivacyTable_p->PrivInvoked)
    {
        txFrame_p->IEEEFrame->Hdr.FrmCtl.Wep = 1;
        WepOverhead = WEPOVERHEAD;
    }
    /* */
    /* Determine if this is the management frame. */
    /* if yes, no SNAP insertion, no DMA */
    /* */
    if (txFrame_p->IEEEFrame->Hdr.FrmCtl.Type == IEEE_TYPE_MANAGEMENT)
    {
        MgmtFrame = TRUE;
        llc_size = 0;
    }
    else
    {
        MgmtFrame = FALSE;
        /* driver has inserted llc and encrypted the pdu */
        if (DRIVERDOWEP && WepOverhead)
        {
            llc_size = 0;
            WepOverhead = 0;
        }
    }
    /* */
    /* if driver already insert the llc, we don't need to insert it again */
    /* */
    if ((FrmBody[0] == 0xaa) || (FrmBody[0] == 0xe0) || (FrmBody[0] == 0xff) )
    {
        llc_size = 0;
    }


    /* */
    /*get the fragment threshold from MIB */
    /* */
    FragThreshold = mib_OpData_p->FragThresh;
    FragThreshold += WepOverhead;


    /* */
    /*Calculate fragment count */
    /* */
    Frag.FragCnt = (Frag.wlanPduLen > FragThreshold) ?
                   (Frag.wlanPduLen / FragThreshold) + 1 : 1;
    if ( Frag.FragCnt > 1)
        return TRUE;
    else
        return FALSE;
}

/******************************************************************************
 *
 * Name: DoFragAndWep
 *
 * Description:
 *   Framents and does Wep if required
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Fragment descriptor structure
 *   
 *
 * Return Value:
 *
 *
 * Notes:
 *   Only data packet is taken care. Managment packets are not passed to this.
 *   Fragmentation is done using memcpy 
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
UINT32 DoFragAndWep(Frag_t *Frag_p)
{
    UINT8* buf;
    UINT8* FrmBody ;
    UINT32 FragThreshold;
    UINT32 length;
    IEEEtypes_GenHdr_t header;
    DMADesBlck_t* DmaDBTmp;
    WLAN_TX_FRAME *TxFrame_p ;
    WLAN_TX_FRAME *TxFrame_p2;
    int llc_size = LLC_SNAP_SIZE;
    int WepOverhead = 0;
    int i;
    BOOLEAN MgmtFrame;
    UINT32 DestAddrTmp;
    UINT32 HeaderPtrs[MAX_FRAGMENT_NUM];
    UINT32 FragPktLen[MAX_FRAGMENT_NUM];
    UINT8 datarate;
    UINT8 ackdur;
    UINT8 preAmble;

    USR_BUF_DESC *usp;

    if (Frag_p == NULL)
    {
        return OS_FAIL;
    }
    FrmBody = Frag_p->wlanPdu;
    TxFrame_p = Frag_p->TxFrame;
    /* */
    /* Determine the wep overhead, so proper space can be reserve for wep */
    /* */
    if (TxFrame_p->IEEEFrame->Hdr.FrmCtl.Wep)
    {
        WepOverhead = WEPOVERHEAD;
    }
    /* */
    /* Determine if this is the management frame. */
    /* if yes, no SNAP insertion, no DMA */
    /* */
    if (TxFrame_p->IEEEFrame->Hdr.FrmCtl.Type == IEEE_TYPE_MANAGEMENT)
    {
        MgmtFrame = TRUE;
        llc_size = 0;
    }
    else
    {
        MgmtFrame = FALSE;
        /* driver has inserted llc and encrypted the pdu */
        if (DRIVERDOWEP && WepOverhead)
        {
            llc_size = 0;
            WepOverhead = 0;
        }
    }
    /* */
    /* if driver already insert the llc, we don't need to insert it again */
    /* */
    if ((FrmBody[0] == 0xaa) || (FrmBody[0] == 0xe0) || (FrmBody[0] == 0xff) )
    {
        llc_size = 0;
    }

    if ((FrmBody[0] == 0x81) && (FrmBody[1] == 0x37) )
    {
        llc_snap[5] = 0xf8;
    }
    else
    {
        llc_snap[5] = 0;
    }
    /* */
    /*get the fragment threshold from MIB */
    /* */
    FragThreshold = mib_OpData_p->FragThresh;
    FragThreshold += WepOverhead;


    /* */
    /*Calculate fragment count */
    /* */
    Frag_p->FragCnt = (Frag_p->wlanPduLen > FragThreshold) ?
                      (Frag_p->wlanPduLen / FragThreshold) + 1 : 1;
    if ( Frag_p->FragCnt > 1)
    {
        TxFrame_p->IEEEFrame->Hdr.FrmCtl.MoreFrag = 1;
    }

    /* */
    /* Calculate the max buffer space needed to process theis packet */
    /* pdu_length + */
    /* fragment header(if fragment is needed) + */
    /* wep overhead(if WEP is needed) + */
    /* FCS */
    /* */
    length = Frag_p->wlanPduLen +
             Frag_p->FragCnt * FRAGMENT_HD_SIZE +
             Frag_p->FragCnt * WepOverhead +
             llc_size +
             Frag_p->FragCnt * FCS_SIZE;


    if ( (length > FRAG_BUF_SIZE) || (buf = msi_wl_AllocPktBuf(wlTxQ)) == NULL)
    {
#ifdef VXWORKS_BUFFER_OPT
        msi_wl_FreeTxFrame(TxFrame_p);
#else
        msi_wl_FreePktBuf((UINT8 *)TxFrame_p->IEEEFrame);
#endif
        return OS_FAIL;
    }

    usp = msi_wl_GetUsrBufPtr((char *)buf);
    /* Use the same buffer for txinfo and packet */
    TxFrame_p2 = (WLAN_TX_FRAME *)usp->info;
    TxFrame_p2->IEEEFrame = (IEEEtypes_Frame_t *)buf;
    *TxFrame_p2->IEEEFrame = *TxFrame_p->IEEEFrame;
    
    /* set number of retry */
    /*TxInfo_p->RetryCnt = mib_OpData_p->ShortRetryLim; */

    TxFrame_p2->wlanTxHdr.StnId = TxFrame_p->wlanTxHdr.StnId;
    /* get the DMA desciptors */
    Frag_p->DmaDb_p = (UINT32)DmaDBArry.DmaDB;

    /* re-chain the DmaDB which has been terminate by the last operation */
    DmaDBArry.DmaDB[DmaDBArry.tail].NextAddr =
    (UINT32) & (DmaDBArry.DmaDB[DmaDBArry.tail + 1]);

    /* */
    /* construct DMA Descriptors */
    /* */
    for (i = 0; i < Frag_p->FragCnt; i++)
    {
        DmaDBTmp = (DMADesBlck_t*) & (DmaDBArry.DmaDB[i]);

        HeaderPtrs[i] = (UINT32)buf;

        buf += WL_80211_HEADER_SIZE;

        /* */
        /* llc is in the 1st fragment */
        /* */
        if (i == 0)
        {
            buf += llc_size;   /* +llc */
        }

        buf += (WepOverhead >> 1);

        DmaDBTmp->SrcAddr = (UINT32)FrmBody;
        DmaDBTmp->DestAddr = (UINT32)buf;
        DmaDBTmp->ByteCt = (Frag_p->wlanPduLen > FragThreshold) ?
                           FragThreshold : Frag_p->wlanPduLen;
        /* temporarily uses memcpy instead of dma */
        memcpy((UINT8 *)DmaDBTmp->DestAddr, (UINT8*)DmaDBTmp->SrcAddr, DmaDBTmp->ByteCt);
        FrmBody += DmaDBTmp->ByteCt;
        FragPktLen[i] = DmaDBTmp->ByteCt + (WepOverhead >> 1);

        buf += FragPktLen[i];

        if (i == 0)
        {
            FragPktLen[i] += llc_size;
        }


        if ((UINT32)buf&0x3)
        {
            (UINT32)buf = (UINT32)(buf + 4) & 0xfffffffc;
        }

        /* Add another half of web overhead */
        FragPktLen[i] += (WepOverhead >> 1);

        Frag_p->wlanPduLen -= DmaDBTmp->ByteCt;
    }

    /* */
    /*terminate the last DMA Descriptor and increament DmaIndx */
    /* */
    DmaDBArry.DmaDB[i - 1].NextAddr = 0;    /* */
    DmaDBArry.tail = i - 1;

    /* */
    /* Move packet to Tx buffer. */
    /* */
    if (!MgmtFrame)
    {
        DmaDBTmp = (DMADesBlck_t*)Frag_p->DmaDb_p;
        if (TxFrame_p->IEEEFrame->Hdr.FrmCtl.Wep)
        {
            /* */
            /* move pdu and do encryption */
            /* */
            /***** later try using DMA instread of memcpy
                    Right now not sure how to check DMA done in chain mode, without
                    interrupt , later should use interrupt *******/ 
            /*         FragDoDMA(DmaDBTmp);  already done using memcpy */
            /* do it only if llc_size != 0 */
            if (llc_size)
            {
                memcpy( (char *)(DmaDBTmp->DestAddr - llc_size), llc_snap, llc_size);
            }



            for (i = 0; i < Frag_p->FragCnt; i++)
            {

                DoEncrypt((UINT32)HeaderPtrs[i]-WL_80211_HEADER_SIZE+4, (UINT32)HeaderPtrs[i]-WL_80211_HEADER_SIZE+4,
                          FragPktLen[i] - WepOverhead);

                DmaDBTmp = (DMADesBlck_t *)DmaDBTmp->NextAddr;
            }
        }
        else
        {
            /* */
            /* No encryption */
            /* */
            /* this is already taken care of using memcpy */
            /*       FragDoDMA(DmaDBTmp); */
        }

    }
    else
    {
        /**** this part is not used right now ****/
        /* */
        /* This is a mgmt packet, can't do DMA */
        /* We assume DmaDBTemp still points to the first DmaDB in Frag_p */
        DmaDBTmp = (DMADesBlck_t*)Frag_p->DmaDb_p;
        if (TxFrame_p->IEEEFrame->Hdr.FrmCtl.Wep)
        {
            memcpy((UINT8 *)DmaDBTmp->DestAddr, (UINT8 *)DmaDBTmp->SrcAddr, DmaDBTmp->ByteCt);
            WL_WRITE_WORD( (DmaDBTmp->DestAddr - 4), IV | (KeyId << 30));

            /* */
            /* set IV */
            /* */
            WL_TxSetWepIV(IV);
            IV = WL_REGS32(TX_TSF_LO);

            FragDoDMAnEncrypt(DmaDBTmp->DestAddr, DmaDBTmp->DestAddr, DmaDBTmp->ByteCt);

        }
        else
        {
            memcpy((UINT8 *)DmaDBTmp->DestAddr, (UINT8 *)DmaDBTmp->SrcAddr, DmaDBTmp->ByteCt);
        }
    }

    /* */
    /*move header, do this here to overlay with DMA operation */
    /* */
    memcpy((char*)&header, (char*)(TxFrame_p->IEEEFrame), sizeof(IEEEtypes_GenHdr_t));

    DestAddrTmp = ((DMADesBlck_t*)Frag_p->DmaDb_p)->DestAddr;

    TxFrame_p2->IEEEFrame = (IEEEtypes_Frame_t*)HeaderPtrs[0];
    if (!MgmtFrame)
    {
        DestAddrTmp -= llc_size;
        if (!WepOverhead)
        {
            memcpy( (char *)(DestAddrTmp), llc_snap, llc_size);
        }
    }
    datarate = GetDataRate(1/*Rate*/);
    ackdur = GetCtsAckDur(1/*Rate*/);

    preAmble = 0/*(TxInfo_p2->TxParam & TX_PARAM_PREAMBLE)*/ ?
               IEEE80211_SHORT_PREAM :
               IEEE80211_LONG_PREAM;
    for (i = 0; i < Frag_p->FragCnt; i++)
    {
        header.DurationId = ackdur + preAmble + IEEE80211_SIFS;

        header.FrmBodyLen = FragPktLen[i];
        if (i == (Frag_p->FragCnt - 1))
        {
            header.FrmCtl.MoreFrag = 0;
        }
        else
        {
            header.DurationId += (FragPktLen[i + 1] * 80 / datarate
                                  + ackdur
                                  + IEEE80211_SIFS * 2
                                  + preAmble * 2);
        }
        /* set duration to 0 for multi cast frame */
        /*if (TxInfo_p2->TxParam & TX_PARAM_MCBC) */
        /*{ */
        /*    header.DurationId = 0; */
        /*} */
        /*add fragment sequence number */
        header.SeqCtl = i;
        memcpy( (char *)HeaderPtrs[i], (char *)&header, WL_80211_HEADER_SIZE);
    }
    if (llc_snap[5] == 0xf8)
    {
        llc_snap[5] = 0x00;
    }
    /*DmaIndx++; */
    /*DmaIndx &= 0x1; */
    txsched_ScheduleFrame(TxFrame_p2);

#ifdef VXWORKS_BUFFER_OPT
    msi_wl_FreeTxFrame(TxFrame_p);
#else
    msi_wl_FreePktBuf((UINT8 *)TxFrame_p->IEEEFrame);
#endif
    return OS_SUCCESS;
}


/******************************************************************************
 *
 * Name: FragDoDMA
 *
 * Description:
 *   This routine is normally called by DoFragment() to move fragment to tx buffer
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   DmaDesBlck_p   pointer to the chain of DMA descriptors
 *
 * Return Value:
 *
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
void FragDoDMA(DMADesBlck_t *DmaDB_p)
{
    UINT32 Chan0Ctrl;
    /* */
    /* Make sure the DMA channel is not busy */
    /* */
    while (dma0_busy)
        ;


    /*if no, program DmaDesBlck_p to hardware */
    WL_WRITE_WORD(WL_IDMA_NEXTRECPTR0, (UINT32)DmaDB_p);

    /*set channel control register */
    WL_WRITE_WORD(WL_IDMA_CH0_CTRL, (DCC_DTL_32 | DCC_CHAIN_INT |
                                       DCC_BLOCK_MODE | DCC_CHANEN | DCC_FETNEXREC) );
    /* Chained mode DMA has started! */
    WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);

    while (Chan0Ctrl &0x4000)
    {
        WL_READ_WORD(WL_IDMA_CH0_CTRL, Chan0Ctrl);
    }



    /*WL_REGS32(WL_IDMA_BYTECNT0) = DmaDB_p->ByteCt; */
    /*WL_REGS32(WL_IDMA_SRCADDR0) = DmaDB_p->SrcAddr; */
    /*WL_REGS32(WL_IDMA_DESADDR0) = DmaDB_p->DestAddr; */
    /*WL_REGS32(WL_IDMA_NEXTRECPTR0) = 0; */
    /*WL_REGS32_SETBITS(WL_IDMA_CH0_CTRL, DCC_CHANEN|DCC_BLOCK_MODE|DCC_DTL_32); */
}

/******************************************************************************
 *
 * Name: FragDoDMAnEncrypt
 *
 * Description:
 *   This routine is normally called by DoFragment() to move fragment to tx 
 *   buffer as well as encrypt the pdu
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   DmaDesBlck_p   pointer to the chain of DMA descriptors
 *
 * Return Value:
 *
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
void FragDoDMAnEncrypt(UINT32 des, UINT32 src, UINT32 len)
{
    /* */
    /* Program read/write addresses and length */
    /* */
    WL_WRITE_WORD(RX_SW_ENC_RDADDR, src);
    WL_WRITE_WORD(RX_SW_ENC_WRADDR, des);
    WL_WRITE_WORD(RX_SW_ENC_LEN, len);

    txwep_busy = 1;

    /* */
    /* start wep */
    /* */
    if (WepTypeAll == 2)
    {
        WL_WRITE_WORD(RX_WEP_CMD, WEP_CMD_WEP_START | WEP_CMD_WEP128);
    }
    else if (WepTypeAll == 1)
    {
        WL_WRITE_WORD(RX_WEP_CMD, WEP_CMD_WEP_START);
    }

    /*   while(txwep_busy) */
    /*      ; */
    while (WL_REGS32(RX_WEP_CMD)&WEP_CMD_TXWEP_BUSY)
        ;

    /*clear TXWEP_DONE interrupt */
    WL_REGS32_CLRBITS(MIR_INTR_STATUS, TXWEP_DONE);
}
extern UINT8 GetCtsAckDur(UINT8 rateid)
{
    if ( (rateid >= 0) && (rateid <= IEEEtypes_MAX_DATA_RATES_G-1))
        return(Cts_Ack_Dur[rateid]);
    return(0);
}

UINT16 GetDataRate(UINT32 rateid)
{
    switch (rateid)
    {
    case 0:   /* 1 Mbit/s */
        return(10);
    case 1:   /* 2 Mbit/s */
        return(20);
    case 2:  /* 5.5 Mbit/s */
        return(55);
    case 3:  /* 11 Mbit/s */
        return(110);
    case 4:  /* 22 Mbit/s */
        return(220);
    case 5:   /* 6 Mbit/s */
        return(60);
    case 6:   /* 9 Mbit/s */
        return(90);
    case 7:  /* 12 Mbit/s */
        return(120);
    case 8:  /* 18 Mbit/s */
        return(180);
    case 9:  /* 24 Mbit/s */
        return(240);
    case 10:   /* 36 Mbit/s */
        return(360);
    case 11:   /* 48 Mbit/s */
        return(480);
    case 12:  /* 54 Mbit/s */
        return(540);
    case 13:  /* 72 Mbit/s */
        return(720);
    }
    return(20);
}
void SetDurationId(txInfo_t * TxInfo_p)
{
    UINT16 datarate;
    UINT16 ackdur,backdur,gackdur;
    UINT16 preAmble;
    datarate = GetDataRate(TxInfo_p->Rate);
    ackdur = GetCtsAckDur(TxInfo_p->Rate);
    if (TxInfo_p->Rate <= HIGHEST_11B_RATE_REG_INDEX)
        preAmble = (TxInfo_p->TxParam & TX_PARAM_PREAMBLE) ?
                   IEEE80211b_SHORT_PREAM :
                   IEEE80211b_LONG_PREAM;
    else
        preAmble = (TxInfo_p->TxParam & TX_PARAM_PREAMBLE) ?
                   IEEE80211g_SHORT_PREAM :
                   IEEE80211g_LONG_PREAM;
    if (TxInfo_p->TxParam & TX_PARAM_MCBC)
        ((IEEEtypes_GenHdr_t *)TxInfo_p->HdrAddr)->DurationId = 0;
    else
        ((IEEEtypes_GenHdr_t *)TxInfo_p->HdrAddr)->DurationId = ackdur + preAmble + IEEE80211_SIFS;
    if (TxInfo_p->TxParam & TX_PARAM_SEND_RTS)
    {
        backdur = GetCtsAckDur(macRtsRateId);
        gackdur = GetCtsAckDur(macgAckRateId);
        TxInfo_p->RtsDurId1 = (((IEEEtypes_GenHdr_t *)TxInfo_p->HdrAddr)->FrmBodyLen + 
                               sizeof(IEEEtypes_GenHdr_t) - sizeof(IEEEtypes_MacAddr_t) +
                               FCS_SIZE) * 80 / datarate 
                              + backdur + gackdur +
                              + IEEE80211_SIFS*3
                              + IEEE80211g_LONG_PREAM * 2
                              + IEEE80211b_LONG_PREAM;
    }
}

#if 0
void ResetDeFragBufInfo(DeFragBufInfo_t *pDeFragBufInfo)
{
    pDeFragBufInfo->SeqNo = 0;
    pDeFragBufInfo->FragNo = 0;
    pDeFragBufInfo->pCurrent = NULL;
}

inline void FreeBuffer(USR_BUF_DESC *usp)
{
    msi_wl_FreeUsrBuf(usp);
}

inline UINT32 GetDeFragFreeBuf()
{
    UINT32 BufNo; //buffer number

    if (DeFragBufInfo[0].pCurrent == NULL)
    {
        BufNo = 0;
    }
    else if (DeFragBufInfo[1].pCurrent == NULL)
    {
        BufNo = 1;
    }
    else if (DeFragBufInfo[2].pCurrent == NULL)
    {
        BufNo = 2;
    }
    else
    { //none of the buffers are available. So Check the time stamp to see expiry
        UINT32 current_time;

        WL_READ_WORD(TX_TSF_LO, current_time);
        if (DeFragBufInfo[0].time_out_time < current_time)
        {
            FreeBuffer(DeFragBufInfo[0].usp);
            ResetDeFragBufInfo(&DeFragBufInfo[0]);
            BufNo = 0;
        }
        else if (DeFragBufInfo[1].time_out_time < current_time)
        {
            FreeBuffer(DeFragBufInfo[1].usp);
            ResetDeFragBufInfo(&DeFragBufInfo[1]);
            BufNo = 1;
        }
        else if (DeFragBufInfo[2].time_out_time < current_time)
        {
            FreeBuffer(DeFragBufInfo[2].usp);
            ResetDeFragBufInfo(&DeFragBufInfo[2]);
            BufNo = 2;
        }
        else
        {
            BufNo = 0xFFFFFFFF;
        }

    }
    return BufNo;
}

/*Note- This function can only be used if pAddr1 and pAddr2
 start at a 4 byte Boundary
*/

#define MAC_ADDR_CMP4ByteAligned(pAddr1, pAddr2) \
                ((*(UINT32*)pAddr1 == *(UINT32*)pAddr2) && \
           (*(UINT16*)((UINT8*)pAddr1 + 4) == *(UINT16*)((UINT8*)pAddr2 + 4)))

inline UINT32 GetDeFragBuf(IEEEtypes_MacAddr_t *Addr, UINT16 SeqNo)
{
    UINT32 BufNo;

    if ((DeFragBufInfo[0].SeqNo == SeqNo) &&
        (MAC_ADDR_CMP4ByteAligned(Addr, 
          (((IEEEtypes_Frame_t *)DeFragBufInfo[0].usp->data)->Hdr.Addr2)) == 1))
    //  (memcmp(Addr, DeFragBufInfo[0].pFrame->Hdr.Addr2, 0) == 0))
    {
        BufNo = 0;
    }
    else if ((DeFragBufInfo[1].SeqNo == SeqNo) &&
             (MAC_ADDR_CMP4ByteAligned(Addr, 
           (((IEEEtypes_Frame_t *)DeFragBufInfo[1].usp->data)->Hdr.Addr2)) == 1))

    //        (memcmp(Addr, DeFragBufInfo[1].pFrame->Hdr.Addr2, 0) == 0))
    {
        BufNo = 1;
    }
    else if ((DeFragBufInfo[2].SeqNo == SeqNo) &&
             (MAC_ADDR_CMP4ByteAligned(Addr,
             (((IEEEtypes_Frame_t *)DeFragBufInfo[1].usp->data)->Hdr.Addr2)) == 1))
//   (memcmp(Addr, DeFragBufInfo[2].pFrame->Hdr.Addr2, 0) == 0))
    {
        BufNo = 2;
    }
    else
    { //none of the buffers are available
        BufNo = 0xFFFFFFFF;
    }
    return BufNo;
}


USR_BUF_DESC * DoWPADeFrag(USR_BUF_DESC *usp, UINT8 *RC4Key)
{
    UINT16 SeqNo;
    UINT8 FragNo;
    UINT32 BuffNo;
    UINT16 payload_size;
    IEEEtypes_Frame_t *pEncrData;
    IEEEtypes_Frame_t *pDecrData;

    pEncrData = (IEEEtypes_Frame_t *)usp->data;

    SeqNo = pEncrData->Hdr.SeqCtl >> 4;
    FragNo = pEncrData->Hdr.SeqCtl & 0x000F;

    payload_size = pEncrData->Hdr.FrmBodyLen -
                   sizeof(IEEEtypes_GenHdr_t) - sizeof(WLAN_RX_INFO) -
                   IV_SIZE - EIV_SIZE - ICV_SIZE;

    if (FragNo == 0)
    { //first fragment
        UINT32 current_time;

        BuffNo = GetDeFragFreeBuf();
        if (BuffNo == 0xFFFFFFFF)
        {
            FreeBuffer(usp);
            return NULL;
        }

        if ((DeFragBufInfo[BuffNo].usp = (IEEEtypes_Frame_t *)msi_wl_AllocUsrBuf(wlRxQ)) == NULL)
        {
            FreeBuffer(usp);
            return NULL;
        }

        pDecrData = DeFragBufInfo[BuffNo].usp->data;
        DeFragBufInfo[BuffNo].pCurrent = pDecrData->Body;

        memcpy(&pDecrData->Hdr, &pEncrData->Hdr, sizeof(IEEEtypes_GenHdr_t));
        pDecrData->Hdr.FrmBodyLen = payload_size + sizeof(IEEEtypes_GenHdr_t) + sizeof(WLAN_RX_INFO);
        //Start timer
        WL_READ_WORD(TX_TSF_LO, current_time);
        DeFragBufInfo[BuffNo].time_out_time = current_time + timeout_val; //TIMEOUT_VAL;
    }
    else
    {   //Get the DeFrag buff with the given sequence no
        BuffNo = GetDeFragBuf(&pEncrData->Hdr.Addr2, SeqNo);
        if (BuffNo == 0xFFFFFFFF)
        { //something is wrong
            FreeBuffer(usp);
            return NULL;
        }
        pDecrData = DeFragBufInfo[BuffNo].usp->data;
        if (DeFragBufInfo[BuffNo].FragNo + 1 == FragNo)
        {
            pDecrData->Hdr.FrmBodyLen += payload_size;
        }
        else
        { //not in sequence
            FreeBuffer(DeFragBufInfo[BuffNo].usp);
            ResetDeFragBufInfo(&DeFragBufInfo[BuffNo]);
            FreeBuffer(usp);
            return NULL;
        }
    }

#ifdef HW_ENCR
    if (DoTKIPDecrypt(DeFragBufInfo[BuffNo].pCurrent,
                      pEncrData->Body + IV_SIZE + EIV_SIZE,
                      payload_size + ICV_SIZE, RC4Key) == 0)
    {
        mib_RSNStats_p->TKIPICVErrors++;
        FreeBuffer(DeFragBufInfo[BuffNo].usp);
        ResetDeFragBufInfo(&DeFragBufInfo[BuffNo]);
        FreeBuffer(usp);
        return NULL;
    }
#else
    if (tkip_decrypt(DeFragBufInfo[BuffNo].pCurrent,
                     pEncrData->Body + IV_SIZE + EIV_SIZE,
                     payload_size, RC4Key) == 0) //ICV match failure
    {
        mib_RSNStats_p->TKIPICVErrors++;
        FreeBuffer(DeFragBufInfo[BuffNo].usp);
        ResetDeFragBufInfo(&DeFragBufInfo[BuffNo]);
        FreeBuffer(usp);
        return NULL;
    }
#endif

    if (pEncrData->Hdr.FrmCtl.MoreFrag == 0)
    {
        ResetDeFragBufInfo(&DeFragBufInfo[BuffNo]);
        memcpy(&DeFragBufInfo[BuffNo].usp->info, usp->info, sizeof(WLAN_RX_INFO));
        //DeFragBufInfo[BuffNo].usp->byte_count = pDecrData->Hdr.FrmBodyLen - sizeof(WLAN_RX_INFO);
        FreeBuffer(usp);
        return DeFragBufInfo[BuffNo].usp;
    }
    else
    {
        DeFragBufInfo[BuffNo].SeqNo = SeqNo;
        DeFragBufInfo[BuffNo].pCurrent += payload_size;
        DeFragBufInfo[BuffNo].FragNo = FragNo;
    }
    FreeBuffer(usp);
    return NULL;
}
#endif
