#                Copyright 2003, Marvell International Ltd.
# This code contains confidential information of Marvell Semiconductor, Inc.
# No rights are granted herein under any patent, mask work right or copyright
# of Marvell or any third party.
# Marvell reserves the right at its sole discretion to request that this code
# be immediately returned to Marvell. This code is provided "as is".
# Marvell makes no warranties, express, implied or otherwise, regarding its
# accuracy, completeness or performance.

# Common make settings for eCos to include at the beginning
# of the local makefile
#
# History:
#    5/3/03   cvu         Added more comments
#

###### ENVIRONMENT VARIABLES
# The makefile requires that the following variables be defined in the
# environment
#     ROOT_DIR
#     GCC_DIR
#     BUILD_DIR
#     HDR_MAK
#     FTR_MAK

###### The tools chain
CC   = $(GCC_DIR)/arm-elf-gcc -mcpu=arm9tdmi
LD   = $(GCC_DIR)/arm-elf-gcc
AR   = $(GCC_DIR)/arm-elf-ar

###### GLOBAL VARIABLES
# Local makefiles may override these.
# Tools chain options
ifndef NO_ECOS_STACK
INC_DIR   = -I$(PKG_INSTALL_DIR)/include -I$(WLANDRV_DIR)/incl
else
INC_DIR   = -I$(PKG_INSTALL_DIR_NO_STACK)/include -I$(WLANDRV_DIR)/incl
endif
CFLAGS    = -Wall $(INC_DIR) -ffunction-sections -fdata-sections -fshort-enums
CFLAGS   += -D__ECOS -DECOS -DAP_SW -DMARVELL_PHY -DAP21 -DCONDOR -DONE_POOL
ifndef NO_ECOS_STACK
LDFLAGS   = -nostartfiles -L$(PKG_INSTALL_DIR)/lib -Wl,--gc-sections\
    -Ttarget.ld -nostdlib  -Xlinker -Map -Xlinker $(basename $@).map
else
LDFLAGS   = -nostartfiles -L$(PKG_INSTALL_DIR_NO_STACK)/lib -Wl,--gc-sections\
    -Ttarget.ld -nostdlib  -Xlinker -Map -Xlinker $(basename $@).map
endif
ARFLAGS   = rv

###### VARIABLES
# Any of these can be overriden on the command-line or in your environment
#COPTFLAGS     = -O3 -DSELF_CTS -DBRIDGE_POLLING_1 -DWLAN_TO_WLAN_WEP_FIX -DBURST_MODE -DSW_FIX_TX_DONE_PROBLEM -DSWRESET -DFILTER_BSSID -DENABLE_WIFI_PROTECTION -DPUT_TIMER_CALL_BACK_IN_TASK -DFOR_PERFORMANCE -DPOOL_BUFFER_SIZE=3072 
 COPTFLAGS     = -g   -DSELF_CTS -DBRIDGE_POLLING_1 -DWLAN_TO_WLAN_WEP_FIX -DBURST_MODE -DSW_FIX_TX_DONE_PROBLEM -DSWRESET -DFILTER_BSSID -DENABLE_WIFI_PROTECTION -DPUT_TIMER_CALL_BACK_IN_TASK -DFOR_PERFORMANCE -DPOOL_BUFFER_SIZE=3072 -DBRIDGE_POLLING -DAP_USE_BUFFER_OFFSET  -DFREE_IN_BRIDGE 

ifdef WIPOD_DIR
COPTFLAGS    += -DWIPOD  -DWDS_FEATURE -DAP_SITE_SURVEY
ifdef WIPOD_WEB_DIR
COPTFLAGS    += -DWIPOD_NEW_GUI
endif
endif
ifdef AP31E_S
COPTFLAGS   += -DAP31E -DEXCLUDE_WEB -DETHERSWITCH_IN_BOOT -DETHERSWITCH -DOPTION_ENABLE -DAUTO_DETECT_W851X
endif
ifdef AP31E_P
COPTFLAGS   += -DAP31CLIENT -DSTA_INFO_DB -DETHERSWITCH -DOPTION_ENABLE -DAUTO_DETECT_W851X
endif
ifdef AP31E_L
COPTFLAGS   += -DAP31CLIENT -DOPTION_ENABLE -DAUTO_DETECT_W851X
endif
ifdef AP32_BOARD
COPTFLAGS   += -DAP31CLIENT -DSTA_INFO_DB -DETHERSWITCH -DETHERSWITCH_IN_OP -DOPTION_ENABLE
endif
ifdef AP31_BOARD
COPTFLAGS   += -DAP31CLIENT -DSTA_INFO_DB -DOPTION_ENABLE
endif
ifdef AP32_TURBO
COPTFLAGS   += -DSWITCH_LIB -DETHERSWITCH -DETHERSWITCH_IN_OP -DOPTION_ENABLE  -DTURBO_SETUP
endif
ifdef AP31_SC
COPTFLAGS   += -DAP_USE_BUFFER_OFFSET -DWLAN_BOUNDARY_CHECK -DOPTION_ENABLE -DAP31_SC
COPTFLAGS   += -DETHERSWITCH -DETHERSWITCH_IN_OP 
endif
ifdef EEPROM
COPTFLAGS   += -DEEPROM
endif
ifdef EXCLUDE_WEB
COPTFLAGS   += -DEXCLUDE_WEB
endif
ifdef AP_PCI
COPTFLAGS += -DAP_PCI
endif
ifdef AP_PCICARD
COPTFLAGS += -DAP_CARD
endif
ifdef VERSION_STRING
COPTFLAGS += -DVERSION=\"$(VERSION_STRING)\"
endif
ifdef NO_ECOS_STACK
COPTFLAGS   += -DNO_ECOS_TCP_IP_STACK
endif
ifdef DHCP_MAGIC_OPTION
COPTFLAGS   += -DDHCP_MAGIC_OPTION=\"$(DHCP_MAGIC_OPTION)\"
endif
DEPEND_FLAGS =  -Wp,-MD,$*.d
EXTRACFLAGS   = $(COPTFLAGS) $(DEPEND_FLAGS)
LIBS          = -Ttarget.ld -nostdlib

##### DIRECTORY VARIABLES
# Any of these can be overriden on the command-line or in your environment
#PKG_INSTALL_DIR = /ecos-d
SRC_DIR         = .
OBJ_DIR         = 
LIB_DIR         = 

###### BUILD RULES
.PHONY: depend all clean

%.o: %.c
	$(CC) -c -o $*.o $(CFLAGS) $(EXTRACFLAGS) -Wp,-MD,$*.d $<

%.d: %.c
	$(CC) -E $(CFLAGS) $(EXTRACFLAGS) -Wp,-MD,$*.d $< >/dev/null

###### MANDATORY LOCAL VARIABLES
# By convention the local makefile must define the following:
#    SRCS     :: list of source files
#    OBJS     :: list of object files
#    DST      :: name of build product, like a library
