#ifndef _flashdrvapi_h_
#define _flashdrvapi_h_ 
/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    flashdrvapi.h
 * \brief   Public interface for flash driver module
 *
 * $Revision: 1.2 $
 * $Date: 2003/09/23 00:04:35 $
 */

/* This driver struct is used to initialize the flash driver in AP */
/* If the flash supports word access, WrWord() must be provided,
   otherwise WrByte() must be given */
typedef struct apfdDriver_t
{
    int WordMode; /* true if word access is supported */

    /* Function to erase a sector */
    /* Return true on success, false on failure */
    int (*EraseSector)(int /* sector number */);

    /* Function to write a byte or a word */
    union
    {
        /* Return true on success, false on failure */
        int (*WrByte)(int /*offset */, unsigned char /* value */);
        int (*WrWord)(int /*offset */, unsigned short int /* value */);
    } wrfunc;

    /* Function that reads the factory default settings from flash */
    /* Return true on success, false on failure */
    int (*ReadFactorySettings)(mib_AP_t * /* data buffer */);

    /* Function that reads the user settings from flash */
    /* Return true on success, false on failure */
    int (*ReadUserSettings)(mib_AP_t * /* data buffer */);

    /* Function that writes the user settings to flash */
    /* Return true on success, false on failure */
#ifdef AP21N

    int (*WriteUserSettings)(mib_AP_t * /* data buffer */, int);
#else

    int (*WriteUserSettings)(mib_AP_t * /* data buffer */);
#endif
    /* Function that resets user settings to factory settings */
    /* Return true on success, false on failure */
    int (*ResetToFactorySettings)(void);

    /* Function that upgrades the firmware */
    /* Return true on success, false on failure */
    int (*UpgradeFirmware)(char *, unsigned long);

}
apfdDriver_t;

/**
 * User supplied function to initialize Flash driver struct
 * At startup time, the system will call this function to initialize
 * the flash driver.
 * 
 * @param apfdDriver_t
 *               Pointer to Flash Driver init struct.
 *               The initialized flash driver stuct is returned.
 */
extern void
    apfdDriverInit(apfdDriver_t *);

#endif /* _flashdrvapi_h_ */
