/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

#ifndef PSQUE_H
#define PSQUE_H

/*!
 * \file    psQue.h
 * \brief   This file contains power saving quing definitions
 *
 */
#include "list.h"
typedef struct StaQueueElem_t
{
    struct StaQueueElem_t *nxt;
    struct StaQueueElem_t *prv;
    void *frame_p;
}
StaQueueElem_t;

typedef struct StaQueInfo_t
{
    IEEEtypes_MacAddr_t MacAddr;  /* Dest Station's MAC address */
    IEEEtypes_PwrMgmtMode_e PwrMgmtMode;
    UINT32 msgCnt;               /* Number of messages in the Station's List */
    UINT16 Aid;
}StaQueInfo_t;

extern StaQueInfo_t *StnQingInfo;

extern List *StaQueList;
extern UINT16 psPwStaCnt;
extern UINT32 MaxResourceCnt;
void psQue_Init(int maxques);

void * psQue_Get(UINT16 stnId);

WL_STATUS psQue_Put(UINT16 stnId,
                      void *txInfo_p,
                      BOOLEAN AddToAllStnListFlag,
                      UINT8 MsgType);

void psStaPwrModeSet(UINT16 stnId, IEEEtypes_PwrMgmtMode_e pwMode); 

#define isStaInPowerSave(stnId) (StnQingInfo[stnId].PwrMgmtMode == PWR_MODE_PWR_SAVE)
#define psStaAid(stnId) (StnQingInfo[stnId].Aid)
#define isMoreDataQuedFor(stnId) (StaQueList[stnId].cnt)
#define isAnyStaInPwrSaveMode() (psPwStaCnt)
#define psStaMacAddr(stnId) (StnQingInfo[stnId].MacAddr)
extern void FreePowerSaveQueue(UINT32);

#endif
