#if !defined(_WL_HAL_H_)
#define _WL_HAL_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    wl_hal.h
 * \brief   This file contains the API definitions for the WLAN HAL.
 *
 */

/*! Define for size of commonly used structs */
#define SZ_PHY_ADDR 6   /*!< Number of bytes in ethernet MAC address */

/*============================================================================= */
/*                    HAL SYSTEM APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

/* mfg data struct*/
#define SZ_BOARD_NAME       8
#define SZ_BOOT_VERSION     12
#define SZ_PRODUCT_ID       58
#define SZ_INTERNAL_PA_CFG  14
#define SZ_EXTERNAL_PA_CFG  1
#define SZ_CCA_CFG          8
#define SZ_LED              4

#ifdef AP_PGRINE
typedef struct PaOpt_t
{
   UINT8  IntPaCalTabOpt:1;  /* 0 = Delta Method, 1 = Absolute Method */
   UINT8  ant2NotCalSep:1;
   UINT8  ExtPaNegPol:1;     /* Ext PA polarization. 0 = Positive, 1 = Negative */
   UINT8  ExtPaUsed:1;
   UINT8  Reserved:4;
} PACK_END PaOpt_t;

typedef struct Ant_t
{
   UINT8   Tx:2;        /* AntDivModeTx[7:6](BBP:0x38),  00 = ant0; 01 = ant1; 1x = diversity */
   UINT8   Rx:2;        /* AntDivModeRx[7:6](BBP:0x37),  00 = ant0; 01 = ant1; 1x = diversity */
   UINT8   Reserved:4;
} PACK_END Ant_t;

typedef struct Delta_t
{
   UINT8 CalVal_C0;
   UINT8 CalVal_C1;
} PACK_END Delta_t;

typedef struct Absolute_t
{
   UINT8 Reg_1C;
   UINT8 Reg_1B;
} PACK_END Absolute_t;

typedef union Pa_t
{
   Delta_t    Delta;
   Absolute_t Absolute;
} PACK_END Pa_t;

typedef struct PaConfig_t
{
   UINT32 Reg_1E:8;
   UINT32 Reg_1F:8;
   UINT32 Reg_20:8;
   UINT32 Reg_1A:6;
   UINT32 Reg_1D_0:1;
   UINT32 _2CalValTbl:1;
} PACK_END PaConfig_t;

typedef struct AssmSN_t
{
   UINT32 AssmCode:8;
   UINT32 SerialNum:24;
} PACK_END AssmSN_t;

typedef struct LCTCal_t
{
   UINT8 LctCalCh11:3;
   UINT8 LctCalCh4:3;
   UINT8 Valid0:1;
   UINT8 Valid1:1;	
}PACK_END LCTCal_t; 
#endif
typedef struct _MFG_CAL_DATA
{
    UINT8           BrdDscrpt[SZ_BOARD_NAME];      /* 8 byte ASCII to descript the type and version of the board */
    UINT8           Rev;
#ifdef AP_PGRINE
/* To define Peregrine and Falcon structure is different. To add auto-detect RF Chip function need to 
   modify MFG_CAL_DATA. We define AUTO_DETECT_MFG to discriminate auto-detect RF MFG function */
#ifdef AUTO_DETECT_MFG
    union {
        struct mfg_peregrine_s {
            PaOpt_t         PaOpt_p;
            uint8           ExtPA_p[SZ_EXTERNAL_PA_CFG];
            Ant_t           Ant_p;
            Pa_t            Pa_p[SZ_INTERNAL_PA_CFG];;
            PaConfig_t      PaConfig_p;
            AssmSN_t        AssmSN_p;
            uint16          Domain_p;            /* Wireless domain */
            uint8	        ECO_p;
            LCTCal_t        LCT_cal_p;
            uint8           SocVer_p;
            uint8           RFVer_p;
            uint8	        SPISize_p;
            uint8    	    AntPart_p;
            uint16          TestVer_p;
            uint16          DllVer_p;
            uint16          Xosc_p;
        } PACK_END mfg_peregrine;
        struct mfg_falcon_s {
            uint8 PAOptions_f;
            uint8 ExtPA_f[SZ_EXTERNAL_PA_CFG];
            uint8 Ant_f;
            uint16 IntPA_f[SZ_INTERNAL_PA_CFG];
            uint8 CCA_f[SZ_CCA_CFG];
            uint16 Domain_f;            // Wireless domain
            uint16 CstmrOpts_f;
            uint8 LED_f[SZ_LED];
            UINT16 Xosc_f;
            UINT8 Reserved_1_f[2];
            uint16 Magic_f;
        } PACK_END mfg_falcon;
    } PACK_END mfg_u;
    uint16 ChkSum;
#else /* ! AUTO_DETECT_MFG */
    PaOpt_t         PaOpt;
    UINT8           ExtPA[SZ_EXTERNAL_PA_CFG];
    Ant_t           Ant;
    Pa_t            Pa[SZ_INTERNAL_PA_CFG];;
    PaConfig_t      PaConfig;
    AssmSN_t        AssmSN;
    UINT16          Domain;            /* Wireless domain */
    UINT8	        ECO;
    LCTCal_t        LCT_cal;
    UINT8           SocVer;
    UINT8           RFVer;
    UINT8	        SPISize;
    UINT8    	    AntPart;
    UINT16          TestVer;
    UINT16          DllVer;
    UINT16          Xosc;
    UINT16          ChkSum;
#endif /* AUTO_DETECT_MFG */
#else /* ! AP_PGRINE */
    UINT8 PAOptions;
    UINT8 ExtPA[SZ_EXTERNAL_PA_CFG];
    UINT8 Ant;
    UINT16 IntPA[SZ_INTERNAL_PA_CFG];
    UINT8 CCA[SZ_CCA_CFG];
    UINT16 Domain;            /* Wireless domain */
    UINT16 CstmrOpts;
    UINT8 LED[SZ_LED];
    UINT16 Xosc;
    UINT8 Reserved_1[2];
    UINT16 Magic;
    UINT16 ChkSum;
#endif /* AP_PGRINE */
    UINT8 MfgMacAddr[SZ_PHY_ADDR];     /* Mfg mac address */
    UINT8 Reserved_2[4];
    UINT8 PID[SZ_PRODUCT_ID];          /* Production ID */
    UINT8 BootVersion[SZ_BOOT_VERSION];
    #if !defined(AP_SW) && !defined(WB_EXCEPTION)
    /* This will affect other platforms! */
//  LCT_cal
	UINT16 CountryCode;
	UINT8  ECO;
	UINT8  LCT_cal;
	#endif
}
MFG_CAL_DATA;

#ifdef AP_PGRINE
#ifdef AUTO_DETECT_MFG
#define PAOptions   mfg_u.mfg_falcon.PAOptions_f
#define ExtPA       mfg_u.mfg_falcon.ExtPA_f
#define Ant         mfg_u.mfg_falcon.Ant_f
#define IntPA       mfg_u.mfg_falcon.IntPA_f
#define CCA         mfg_u.mfg_falcon.CCA_f
#define Domain      mfg_u.mfg_falcon.Domain_f
#define CstmrOpts   mfg_u.mfg_falcon.CstmrOpts_f
#define LED         mfg_u.mfg_falcon.LED_f
#define Xosc        mfg_u.mfg_falcon.Xosc_f
#define Reserved_1  mfg_u.mfg_falcon.Reserved_1_f
#define Magic       mfg_u.mfg_falcon.Magic_f

#define PaOpt       mfg_u.mfg_peregrine.PaOpt_p       
#define ExtPAp      mfg_u.mfg_peregrine.ExtPA_p       
#define Antp        mfg_u.mfg_peregrine.Ant_p         
#define Pa          mfg_u.mfg_peregrine.Pa_p          
#define PaConfig    mfg_u.mfg_peregrine.PaConfig_p    
#define AssmSN      mfg_u.mfg_peregrine.AssmSN_p      
#define Domainp     mfg_u.mfg_peregrine.Domain_p      
#define ECO         mfg_u.mfg_peregrine.ECO_p         
#define LCT_cal     mfg_u.mfg_peregrine.LCT_cal_p     
#define SocVer      mfg_u.mfg_peregrine.SocVer_p      
#define RFVer       mfg_u.mfg_peregrine.RFVer_p       
#define SPISize     mfg_u.mfg_peregrine.SPISize_p     
#define AntPart     mfg_u.mfg_peregrine.AntPart_p     
#define TestVer     mfg_u.mfg_peregrine.TestVer_p     
#define DllVer      mfg_u.mfg_peregrine.DllVer_p      
#define Xoscp       mfg_u.mfg_peregrine.Xosc_p
#endif
#endif

typedef enum NetDevType_e
{
    NET_DEV_FAST_ETH = 1,   /*!< Fast Ethernet */
    NET_DEV_W8300,          /*!< W8300 Wireless MAC (802.11b) */
    NET_DEV_W8310           /*!< W8310 Wireless MAC (802.11g) */
}
NET_DEV_TYPE;

typedef void *(*RX_MEM_COPY)(void * dest, void * src, UINT32 size);
typedef struct _WLAN_CONTROL
{
    /*!< Optional: User defined functions to do memory copy of data from hardware buffer to destination */
    /* must be set to NULL if not used or if not applicable. */
    RX_MEM_COPY          rxDataCopy;      /* used for data frame transfer */
    RX_MEM_COPY          rxMgmtCopy;      /* used for management frame transfer */
    RX_MEM_COPY          rxBeaconCopy;    /* used for beacon frame transfer */
    RX_MEM_COPY          rxDataCopy1;      /* used for data frame transfer */
}
WLAN_CONTROL;

/*!
 *  Structure to represent a net device, like ethernet MAC or Wireless MAC
 *  The data specific to each net device type is hidden in PrivData.
 *  The code that makes use of NET_DEV must know how to decode PrivData
 *  based on the value in NetDevType.
 *  Some net device driver must setup ring buffers for receive and transmit.
 *  The user must provide two functions: a function to allocates a buffer,
 *  a function to free the buffer.
 *  Set the alloc and free function pointers to NULL if the net device driver
 *  does not need them. 
 */
typedef struct _NET_DEV
{
    UINT16 NetDevType; /* Net device type, see NET_DEV_TYPE */
    UINT8 PhyAddr[SZ_PHY_ADDR]; /*!< The physical address (MAC address) */
    UINT32 flags;    /*!< General control word */
    void *PrivData; /*!< Pointer to net device private data */
    /*!< User defined function to allocate buffer with allignment of "allign" bytes */
    UINT8* (*allocBuf)(UINT32 allign, UINT32 bytes);
    BOOLEAN (*freeBuf)(void *); /*!< User defined function to free buffer */
    /*!< User defined function to allocate blocks of memory, cannot be deallocated.*/
    UINT8* (*allocMem)(UINT32 allign, UINT32 bytes);
}
NET_DEV;

typedef enum _LBU_CPU_CLOCK
{
    CPU_MHZ_22,
    CPU_MHZ_44,
    CPU_MHZ_64,
    CPU_MHZ_80,
    CPU_MHZ_106,
    CPU_MHZ_128,
    CPU_MHZ_160,
    CPU_MHZ_176
}
LBU_CPU_CLOCK;

typedef enum _WL_OP_MODE
{
    WL_OP_MODE_AP,
    WL_OP_MODE_STA
}
WL_OP_MODE;

typedef struct _WL_SYS_CFG
{
    UINT32 OpMode; /*!< Mode of operation: client or access point */
    UINT32 CpuFreq; /* CPU frequency */
    MIB_802DOT11 *Mib802dot11; /* Initial 802.11 MIB settings */
    MFG_CAL_DATA *CalData; /*!< Calibration data */
    NET_DEV *EthNetDev; /*!< Ethernet port settings */
    NET_DEV *WlNetDev;  /*!< Wireless port settings */
    WLAN_CONTROL * WLANControl;
    UINT16 flag;
}
WL_SYS_CFG;


extern WL_SYS_CFG *msi_wl_CreateSysCfg(UINT32 opMode, MFG_CAL_DATA *calData);

/*  Depending on operation mode, msi_wl_SysInit will do following module initialiation.
 *      1. Save inputs and reset the chip.
 *      2. Initialize base band processor
 *      3. Initialize wireless mac 
 *      4. If in mode WL_OP_MODE_AP, initialize the fast ethernet mac.
 */
extern BOOLEAN msi_wl_SysInit(WL_SYS_CFG *sysCfg);

extern void msi_wl_SysReset(void);
extern UINT32 msi_wl_GetRevNum(void);
extern BOOLEAN msi_wl_InitLBUSysClk(void);
extern BOOLEAN msi_wl_InitSPI(void);
extern UINT8 msi_wl_ReadSPI8(UINT32 addr);
extern UINT32 msi_wl_ReadSPI32(UINT32 addr);
extern BOOLEAN mwi_wl_ApplyMfgCalData(MFG_CAL_DATA *calData);
extern BOOLEAN msi_wl_EnableSysWatchDog(UINT32 ms);
extern BOOLEAN msi_wl_DisableSysWatchDog(void);
extern BOOLEAN msi_wl_ResetSysWatchDog(void);
extern SINT32 msi_wl_GetErrno(void);
extern UINT32  msi_wl_GetCpuSpeed(void);
extern void msi_wl_InitPadResetPeriph(void);

/*============================================================================= */
/*                    HAL BASEBAND APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

/* Antenna definitions. */
#define  AP_ANT_DIVERSITY       0x00 /* antenna mode  */
#define  AP_ANT_A               0x01 /* antenna mode  */
#define  AP_ANT_B               0x02 /* antenna mode  */
#define  RX_ANTENNA             0x00 /* which antenna */
#define  TX_ANTENNA             0x01 /* which antenna */

/* Max and Min Power definitions. */
#define  MIN_PWR_LVL            0
#define  MAX_PWR_LVL            20

/* Power Amplifier options. */
#define PAOPTIONS_USE_TABLE_1   0x01
#define PAOPTIONS_ANT2_NOT_SPRT 0x02
#define PAOPTIONS_EXT_PA_POL    0x04
#define PAOPTIONS_EXTERNAL_PA   0x08
#define PAOPTIONS_EXTERNAL_PA_DRVBY 0x18

extern BOOLEAN msi_wl_InitPHY(void);
extern BOOLEAN msi_wl_SoftResetBBP(void);
extern BOOLEAN msi_wl_HardResetPHY(void);
extern BOOLEAN msi_wl_SetRFOutputPower(UINT32 chan, UINT32 power);
extern BOOLEAN msi_wl_SetRFChan(UINT32 chan);
extern UINT32 msi_wl_GetRFChan(void);
extern BOOLEAN msi_wl_SetAntMode(UINT32 whichAnt, UINT32 antMode);
extern UINT32 msi_wl_GetAntMode(UINT32 whichAnt);
extern BOOLEAN msi_wl_ReadPHY(UINT8 reg, UINT8 *value);
extern BOOLEAN msi_wl_WritePHY(UINT8 reg, UINT8 value);
extern BOOLEAN msi_wl_initPA(UINT32 chan, UINT32 dBm, MFG_CAL_DATA *calData);
extern BOOLEAN msi_wl_SetPHYMode(UINT32 mode);

extern BOOLEAN msi_wl_InitBBP(MFG_CAL_DATA *calData);
extern BOOLEAN msi_wl_ReadBBP(UINT8 reg, UINT8 *value);
extern BOOLEAN msi_wl_WriteBBP(UINT8 reg, UINT8 value);
extern BOOLEAN msi_wl_EnableProtectMode(BOOLEAN flag);

/*============================================================================= */
/*                    HAL 802.11 MAC APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

typedef enum _WL_KEY_TYPE
{
    WEB_40_BIT,
    WEB_104_BIT
}
WL_KEY_TYPE;

typedef enum _MCU_OP_MODE
{
    MCU_MODE_AP,
    MCU_MODE_STA_INFRA,
    MCU_MODE_STA_ADHOC
}
MCU_OP_MODE;


typedef struct _WL_MCU_COUNTERS
{
    UINT32 rtsFailCounter;
    UINT32 rtsGoodCounter;
    UINT32 beaconTxCounter;
    UINT32 probeRspTxCounter;
    UINT32 fcsErrorCounter;
    UINT32 ucastFrameCounter;
    UINT32 mcastFrameCounter;
    UINT32 dupFrameCounter;
    UINT32 trafficCounter;
    UINT32 trafficErrCounter;
    UINT32 icvErrCounter;
    UINT32 undecryptFrameCounter;
    UINT32 excludedFrameCounter;
}
WL_MCU_COUNTERS;

typedef struct _MCU_CORE_TIMER_PARAMS
{
    UINT32 txTsfLo;             /* TSF low 32 bit */
    UINT32 txTsfHi;             /* TSF high 32 bit */
    UINT32 txAccIntvlLo;        /* accumulated interval: low 32 bit */
    UINT32 txAccIntvlHi;        /* accumulated interval: high 32 bit */
    UINT32 txDTIMCount;         /* DTIM count */
    UINT32 txCfpCount;          /* CFP count */
    UINT32 txBeaconPeriod;      /* Beacon period */
    UINT32 txATIMWindow;        /* ATIM window */
    UINT32 txDTIMPeriod;        /* DTIM period */
    UINT32 txListenInterval;    /* listen interval */
    UINT32 txCfpPeriod;         /* CFP period */
    UINT32 txCfpMaxDur;         /* CFP maximum duration */
}
MCU_CORE_TIMER_PARAMS;

extern BOOLEAN msi_wl_InitWLMAC(UINT32 mode, NET_DEV* netDev, WLAN_CONTROL* control);
extern BOOLEAN msi_wl_SetRxWEPKey(UINT32 keyId, UINT32 keyType, UINT8 * key);
extern BOOLEAN msi_wl_SetTxWEPKey(UINT32 keyType, UINT8 * key);
extern BOOLEAN msi_wl_SetMcastAddr(UINT32 MacAddrLo, UINT32 MacAddrHi);
extern BOOLEAN msi_wl_ResetMcastAddr(void);
extern BOOLEAN msi_wl_EnableRxWEP(UINT32 flag);
extern BOOLEAN msi_wl_EnableRxWEP40bit(void);
extern BOOLEAN msi_wl_EnableRxWEP104bit(void);
extern BOOLEAN msi_wl_EnableRxWEPAESMode(UINT32 flag);
extern BOOLEAN msi_wl_InitSpecHw(void);

/* Frame types for msi_wl_BlockRx802dot11Frames call */
typedef enum _WLAN_FRAME_TYPE
{
    FRAME_ASSOC_REQ = 0,        /* Association Request */
    FRAME_ASSOC_RSP,                         /* Association Reponse */
    FRAME_REASSOC_REQ,                       /* Reassociation Request */
    FRAME_REASSOC_RSP,                       /* Reassociation Response */
    FRAME_PROBE_REQ,                         /*  Probe Request */
    FRAME_PROBE_RSP,                         /*  Probe Reponse */
    FRAME_BEACON,                            /*  Beacon */
    FRAME_ATIM,                              /*  ATIM */
    FRAME_DISASSOC,                          /*   Disassociation */
    FRAME_AUTH,                              /*   Authentication */
    FRAME_DEAUTH,                            /*    Deauthentication */
    FRAME_PS_POLL,                           /*     PS-POLL */
    FRAME_RTS,                               /*  RTS */
    FRAME_CTS,                               /*  CTS */
    FRAME_ACK,                               /*  ACK */
    FRAME_CF_END,                            /*  CF-End */
    FRAME_CF_END_CF_ACK,                     /*     CF-End+CF-Ack */
    FRAME_DATA,                              /*     Data */
    FRAME_DATA_CF_ACK,                       /*      Data+CF-Ack */
    FRAME_DATA_CF_POLL,                      /*      Data+CF-Poll */
    FRAME_DATA_CF_ACK_CF_POLL,               /*      Data+CF-Ack+CF-Poll */
    FRAME_NULL,                              /*      Null function */
    FRAME_CF_ACK,                            /*      CF-Ack */
    FRAME_CF_POLL,                           /*      CF-Poll */
    FRAME_CF_ACK_CF_POLL,                    /*  CF-Ack+CF-Poll */
    FRAME_TYPE_NUM
}
WLAN_FRAME_TYPE;

extern BOOLEAN msi_wl_BlockRx802dot11Frames(UINT8* frmType);
extern BOOLEAN msi_wl_UnBlockRx802dot11Frames(UINT8* frmType);

extern BOOLEAN msi_wl_SetSSID(UINT8* ssid, UINT32 nbytes);
extern BOOLEAN msi_wl_GetSSID(UINT8* ssid, UINT32 nbytes);
extern BOOLEAN msi_wl_SetBeaconMissedThreashold(UINT32 count);
extern BOOLEAN msi_wl_SetStaAID(UINT32 aid);
extern UINT32 msi_wl_GetStaAID(void);
extern BOOLEAN msi_wl_SetBSSID(UINT8* bssid);
extern BOOLEAN msi_wl_SetBSSID2(UINT32 hi, UINT32 lo);
extern BOOLEAN msi_wl_GetBSSID(UINT8* bssid);
extern BOOLEAN msi_wl_EnableBcastSSID(UINT32 flag);
extern BOOLEAN msi_wl_SetMACAddr(UINT8* macAddr);
extern BOOLEAN msi_wl_GetMACAddr(UINT8* macAddr);
extern BOOLEAN msi_wl_SoftResetTxMCU(void);
extern BOOLEAN msi_wl_SoftResetRxMCU(void);
extern BOOLEAN msi_wl_GetMCUCounters(WL_MCU_COUNTERS * counters);
extern BOOLEAN msi_wl_SetCWMin(UINT32 queueNum, UINT32 tu);
extern UINT32 msi_wl_GetCWMin(UINT32 queueNum);
extern BOOLEAN msi_wl_SetCWMax(UINT32 queueNum, UINT32 tu);
extern UINT32 msi_wl_GetCWMax(UINT32 queueNum);
extern BOOLEAN msi_wl_SoftResetMCU(void);
/* set/get transmit TSF registers: high word: low word */
extern BOOLEAN msi_wl_SetMCUCoreTimerTxTSF(UINT32 hi, UINT32 lo);
extern BOOLEAN msi_wl_GetMCUCoreTimerTxTSF(UINT32 *hi, UINT32 *lo);
extern BOOLEAN msi_wl_SetMCUCoreTimerAccIntvl(UINT32 hiHalf, UINT32 lowHalf);
extern BOOLEAN msi_wl_GetMCUCoreTimerAccIntvl(UINT32 *hiHalf, UINT32 *lowHalf);
extern BOOLEAN msi_wl_SetMCUCoreTimerParams(MCU_CORE_TIMER_PARAMS * tmrParams);
extern BOOLEAN msi_wl_GetMCUCoreTimerParams(MCU_CORE_TIMER_PARAMS * tmrParams);
extern BOOLEAN msi_wl_SetOpModeMCU(UINT32 mode);
extern UINT32 msi_wl_GetOpModeMCU(void);

/* register settings for RX mode control */
#define WL_FILTER_ON_BSS_ID  0x0
#define WL_PROMISCUOUS_ON    0x1
#define WL_ALLOW_ALL_BSS_ID  0x2
#define WL_ALLOW_BRDCST_SSID 0x4
#define WL_ALLOW_RCV_MSGS    0x8
#define WL_STOP_RCV_MSGS     0xFFFFFFF7
extern BOOLEAN msi_wl_SetRxModeMCU(UINT32 mode);
extern UINT32 msi_wl_GetRxModeMCU(void);

extern BOOLEAN msi_wl_SetWLMacTransmitWatchDog(UINT32 time);

#define WLAN_POWER_MODE_ACTIVE      0x0
#define WLAN_POWER_MODE_SAVE        0x3
#define WLAN_POWER_MODE_AUTO_PSM    (1<<2)
#define WLAN_POWER_MODE_PWR_DOWN_RF (1<<3)
extern BOOLEAN msi_wl_SetPowerMode(UINT32 mode);

#define WLAN_TIMER_RUN_DISABLE      0x0
#define WLAN_TIMER_RUN_ENABLE       0x1
extern BOOLEAN msi_wl_SetTimerRun(UINT32 mode);

#define WLAN_PS_POLL_NO_TRANSMIT    0
#define WLAN_PS_POLL_TRANSMIT       1
extern BOOLEAN msi_wl_SetPsPoll(UINT32 mode);

extern BOOLEAN msi_wl_resetWlmac(void);
extern BOOLEAN msi_wl_setCFParameters(UINT32 CfPeriod, UINT32 CfpMax);
extern BOOLEAN msi_wl_setBcnPeriod(UINT32 BcnPeriod);
extern BOOLEAN msi_wl_setDtimPeriod(UINT32 DtimPeriod);
extern BOOLEAN msi_wl_setDcfSlotTime(UINT32 DcfSlot);
extern BOOLEAN msi_wl_setAifsTime6(UINT32 AifsTime);
extern BOOLEAN msi_wl_setAifsTime7(UINT32 AifsTime);
extern BOOLEAN msi_wl_timerCommitAll(void);
extern BOOLEAN msi_wl_EnableBssidFilter(void);
extern BOOLEAN msi_wl_DisableBssidFilter(void);
extern BOOLEAN msi_wl_EnableMsgReceipt(void);
extern BOOLEAN msi_wl_DisableMsgReceipt(void);
extern BOOLEAN msi_wl_EnableBssidFilterAndMsgReceipt(void);
extern BOOLEAN msi_wl_EnableBssidFilterAndMsgReceiptAndBrdcstSsid(void);
extern BOOLEAN msi_wl_DisableBssidFilterAndEnableMsgReceipt(void);

/*!
 * routine pair for upper layer to transmit 802.11 frames.
 */
#define WLAN_TX_PREAMPLE_SHORT      0x00
#define WLAN_TX_PREAMPLE_LONG       0x01

/* Definitions for TxParam */
#define WLAN_TX_HEADER_TYPE1       0x0001
#define WLAN_TX_SHORT_PREAMBLE     0x0004
#define WLAN_TX_ANTENNA1           0x0008
#define WLAN_TX_LONG_RETRY_LIMIT   0x0010
#define WLAN_TX_SEND_RTS           0x0020
#define WLAN_TX_MULTI_CAST         0x0040
#define WLAN_TX_NOT_DIRECTED_QAP   0x0080
#define WLAN_TX_TX_USE_CCK         0x0100
#define WLAN_TX_LISTEN_USE_CCK     0x0200
#define WLAN_TX_LISTEN_USE_OFDM    0x0400

typedef PACK_START struct _WLAN_TX_INFO
{
    UINT16              QosControl;             /* Not used for non-QoS frames: 0-7 default: 0 */
    UINT8               RateIdx;                /* transmit rate index as defined in MIB_PHY_SUPP_DATA_RATES_TX */
    UINT8               Preample;
    UINT16              StnId;
    UINT16              TxParam;
    UINT8               reserved[44];
} WLAN_TX_INFO;

typedef struct _WLAN_TX_FRAME
{
    WLAN_TX_INFO        wlanTxHdr;
    IEEEtypes_Frame_t * IEEEFrame;
    UINT32              cookie;
}
WLAN_TX_FRAME;

extern BOOLEAN msi_wl_RFTxMgmt(WLAN_TX_FRAME *wlFrame);
extern BOOLEAN msi_wl_RFTxData(WLAN_TX_FRAME *wlFrame);
extern WLAN_TX_FRAME *msi_wl_RFTxDoneFrame(void);
#ifndef AP_SW
BOOLEAN msi_wl_RFTxDataPending(WLAN_TX_FRAME * wlFrame);
void msi_wl_RFTxDataCommit(void);
#endif

/* data structure user to pass in to get the done info. */
/* Users are expected to call msi_wl_RFTxDone on interrupt/event of transmit done. */
typedef struct _MRVL_TX_DONE_INFO
{
    WLAN_TX_FRAME * wlFrame;    /*!< frame pointer sent to transmit calls */
    BOOLEAN         status;     /*!< trasmit status: IEEEtypes_STATUS_SUCCESS - successful, otherwise - error */
    UINT8           retryCnt;   /*!< trasmit retry count: valid for chip rev 6 and up */
    UINT8           rateID;     /* the rate the mac sent the packet */
}
MRVL_TX_DONE_INFO;
extern BOOLEAN msi_wl_RFTxDone(MRVL_TX_DONE_INFO * info);

/*!
 * Structure of the 802.11 frame received from the lowest layer
 */

/*!
 * Extra information provided by WLAN MAC on each received packet
 */

typedef PACK_START struct _WLAN_RX_INFO
{
   UINT8    resvd0;           /* reserved */
   UINT8    Sq2;              /* Signal Quality 2 */
   UINT16   resvd1;           /* reserved */
   UINT8    resvd2;           /* reserved */
   UINT8    Sq1;              /* Signal Quality 1 */
   UINT8    Rate;             /* rate at which frame was received */
   UINT8    RSSI;             /* RF Signal Strength Indicator */
   UINT16   QosControl;       /* QoS Control field */
   UINT16   resvd3;           /* reserved */
}
PACK_END WLAN_RX_INFO;

/* Structure of the 802.11 frame returned */
typedef PACK_START struct _WLAN_RX_FRAME
{
    WLAN_RX_INFO         wlanRxHdr;
    IEEEtypes_Frame_t   *Frame802dot11; /*!< 802.11 frame */
}
PACK_END WLAN_RX_FRAME;

/*
 *  Get received 802.11 data frame from hardware MAC
 *  This function returns TRUE if successful.
 *  Number of bytes stored in rxBuf is returned in nbytes
 */
BOOLEAN msi_wl_GetRcvd802dot11DataFrame(WLAN_RX_FRAME *rxBuf, UINT32 *nbytes);

/*
 *  Get received 802.11 management frame from hardware MAC
 *  This function returns TRUE if successful.
 *  Number of bytes stored in rxBuf is returned in nbytes
 */
BOOLEAN msi_wl_GetRcvd802dot11MgtFrame(WLAN_RX_FRAME *rxBuf, UINT32 *nbytes);
/*
 *  Get received 802.11 beacon frame from hardware MAC
 *  This function returns TRUE if successful.
 *  Number of bytes stored in rxBuf is returned in nbytes
 */
BOOLEAN msi_wl_GetRcvd802dot11BeaconFrame(WLAN_RX_FRAME *rxBuf, UINT32 *nbytes);

/* Wireless MAC interrupt status bits returned from msi_wl_GetWLMacIntrStatus */
#define TX_UNDERFLOW          (1<<0)
#define TX_WDOG_TIMEOUT       (1<<1)
#define RX_FRAG_ERR           (1<<2)
#define RX_MEM_ERR            (1<<3)
#define PTR_ERR               (1<<4)
#define RX_OVERFLOW           (1<<5)
#define DATA_BUF_RDY          (1<<7)
#define MGMT_BUF_RDY          (1<<11)
#define TX_DONE               (1<<15)
#define TXWEP_DONE            (1<<17)
#define POWER_CHNG            (1<<19)
#define CFP_END               (1<<23)
#define CFP_START             (1<<24)
#define BCN_LOST              (1<<25)
#define BCN_DATA_RDY          (1<<26)
#define BCN_FREE              (1<<27)
#define TBTT_EARLY            (1<<28)
#define CTRL_MSG_RDY          (1<<31)

extern UINT32 msi_wl_GetWLMacIntrStatus(void);
extern BOOLEAN msi_wl_SetWLMacIntrStatus(UINT32 bitMap);

/*
 *  Wireless MAC interrupt mask bits set/cleared through msi_wl_SetWLMacIntrMask and 
 *  msi_wl_ClearWLMacIntrMask 
 */
#define MSK_TX_UNDERFLOW      (1<<0)
#define MSK_TX_WDOG_TIMEOUT   (1<<1)
#define MSK_RX_FRAG_ERR       (1<<2)
#define MSK_RX_MEM_ERR        (1<<3)
#define MSK_PTR_ERR           (1<<4)
#define MSK_RX_OVERFLOW       (1<<5)
#define MSK_DATA_BUF_RDY      (1<<7)
#define MSK_MGMT_BUF_RDY      (1<<11)
#define MSK_TX_DONE           (1<<15)
#define MSK_TXWEP_DONE        (1<<17)
#define MSK_POWER_CHNG        (1<<19)
#define MSK_CFP_END           (1<<23)
#define MSK_CFP_START         (1<<24)
#define MSK_BCN_LOST          (1<<25)
#define MSK_BCN_DATA_RDY      (1<<26)
#define MSK_BCN_FREE          (1<<27)
#define MSK_TBTT_EARLY        (1<<28)
#define MSK_CTRL_MSG_RDY      (1<<31)
BOOLEAN msi_wl_SetWLMacIntrMask(UINT32 bitMap);
BOOLEAN msi_wl_ClearWLMacIntrMask(UINT32 bitMap);

BOOLEAN msi_wl_SetWLMacRsrIntrMask(UINT32 bitMap);
BOOLEAN msi_wl_ClearWLMacRsrIntrMask(UINT32 bitMap);

/*============================================================================= */
/*                    HAL DMA APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

typedef struct _DMA_DESC
{
    UINT32 ByteCt;
    UINT32 SrcAddr;
    UINT32 DestAddr;
    UINT32 NextAddr;
}
DMA_DESC;

/******************************************************************************
 *
 * Name: DMA interrupt routine
 *
 * Description:
 *   This routine performs iDAM interrupts post interrupts processing
 * DMA HAL clients are expected to call this routine FIRST in irs ISR routine before 
 * doing anything else.
 *
 *****************************************************************************/
extern UINT32 hal_isr_iDma(UINT32 channel);

extern BOOLEAN msi_wl_InitDMA(UINT32 channel);
extern BOOLEAN msi_wl_AbortDMA(UINT32 channel);
extern BOOLEAN msi_wl_IsDMAActive(UINT32 channel);
extern BOOLEAN msi_wl_DoDMABlockMode(UINT32 channel, UINT32 dest, UINT32 src, UINT32 size, BOOLEAN wait);
extern BOOLEAN msi_wl_DoDMAChainMode(UINT32 channel, DMA_DESC *DmaDB_p, BOOLEAN wait);

/*============================================================================= */
/*                    HAL GPIO APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */
#define LED_BY_FW   0xfffc  /* Controlled by FW */
#define LED_DATA_IO 0xfffe  /* Data IO  */
#define LED_ON      0xfffd  /* LED ON  */
#define LED_OFF     0xffff  /* LED OFF */
typedef enum _GPIO_LED_CYCLE
{
    LED_CYCLE_37MS = 0,
    LED_CYCLE_74MS,
    LED_CYCLE_149MS,
    LED_CYCLE_298MS,
    LED_CYCLE_596MS,
    LED_CYCLE_1192MS
}
GPIO_LED_CYCLE;

typedef enum _GPIO_LED_DUTY_CYCLE
{
    LED_DUTY_1_2TH = 0,
    LED_DUTY_1_4TH,
    LED_DUTY_1_8TH,
    LED_DUTY_1_16TH,
    LED_DUTY_1_32TH
}
GPIO_LED_DUTY_CYCLE;

typedef struct _GPIO_LED_CFG
{
    UINT32 gpioNum;   /* GPIO number: 0-7 */
    UINT32 ledCycle;
    UINT32 dutyCycle;
}
GPIO_LED_CFG;

typedef enum _GPIO_SRC_SELECT
{
    GPIO_SRC_SELECT_RESERVED = 0,
    GPIO_SRC_SELECT_REGISTER = 1,
    GPIO_SRC_SELECT_ARM_ETM = 2,
    GPIO_SRC_SELECT_LED = 3
}
GPIO_SRC_SELECT;

typedef struct _GPIO_CFG
{
    UINT32 dirBitMap;            /* bit map for direction: 0 -- output, 1 -- input */
    UINT32 isrTrigger;           /* 0 = from 0->1, 1 = from 1->0 */
    UINT32 isrMask;

    /* GPIO Reset Select: */
    /* 0=write to isr does not clear GPIO_ISR, */
    /* 1=write to isr clears GPIO_ISR */
    UINT32 isrResetSelect;

    UINT32 outputEnable;
}
GPIO_CFG;

extern BOOLEAN msi_wl_initGPIO(GPIO_CFG * gpioCfg);
extern UINT32 msi_wl_ReadGPIO(UINT32 gpioNum);
extern BOOLEAN msi_wl_WriteGPIO(UINT32 gpioNum, UINT32 state);
extern BOOLEAN msi_wl_EnableLED(UINT32 gpioNum, GPIO_LED_CFG * ledCfg);
extern BOOLEAN msi_wl_DisableLED(UINT32 gpioNum);
extern BOOLEAN msi_wl_EnableGPIOInterrupt(UINT32 bitMap);


extern BOOLEAN SetGPIOPinState(UINT8 gpioNum, UINT8 State);
extern UINT8 GetGPIOPinStatus(UINT8 gpioNum);		   

/*============================================================================= */
/*                    HAL UART APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */
typedef enum _UART_PARITY
{
    UART_PARITY_ODD = 0,
    UART_PARITY_EVEN = 1,
    UART_PARITY_NONE = 2
}
UART_PARITY;

typedef enum _UART_BAUD_RATE
{
    UART_BAUD_RATE_38400 = 0,
    UART_BAUD_RATE_19200,
    UART_BAUD_RATE_9600,
    UART_BAUD_RATE_4800,
    UART_BAUD_RATE_2400
}
UART_BAUD_RATE;

typedef struct _UART_CONFIG
{
    UINT8 BitsPerSecond;
    UINT8 DataBits;
    UINT8 Pariry;
    UINT8 StopBits;
}
UART_CONFIG;

typedef enum _UART_REGISTER
{
    HAL_UART_RBR = 0,
    HAL_UART_THR,
    HAL_UART_DLL,
    HAL_UART_IER,
    HAL_UART_DLH,
    HAL_UART_ISR_ID,
    HAL_UART_FIFO_CTRL,
    HAL_UART_LCR,
    HAL_UART_LSR,
    HAL_UART_SCR
}
UART_REGISTER;

extern BOOLEAN msi_wl_InitUART(UART_CONFIG * config);
extern UINT32 msi_wl_ReadUART(UART_REGISTER uartReg);
extern BOOLEAN msi_wl_WriteUART(UART_REGISTER uartReg, UINT32 value);


/*============================================================================= */
/*                    HAL HARDWARE TIMER APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

/*
 *  Four*timers*are*defined*within*w81xx*devices.*
 *  
 *  Timer*1*is*the*slow*timer*with*1KHz*input,*given*resolution*of*1*ms.*
 *  Timer*2,3,4*is*the*fast*timer*with*1MHz*input,*given*resolution*of*1*us.*
 *  
 *  Two*modes*of*operation*are*available,*periodic*and*one-shot.*In*periodic
 *  timer*mode,*the*counter*will*generate*an*interrupt*at*constant*interval.*
 *  In*one-shot*mode,*the*timer*will*only*generate*one*interrupt*when*it*
 *  reaches*zero.*
 *  
 *  Timer*interruptsa*are*latched*by*interrupt*source*status*register*and*
 *  interrupt*request*stataus*register,*which*can*be*read*by*ARM.*The*interrupt
 *  signals*also*feed*into*interrupt*controller*to*produce*FIQ*and*IRS*interrupt.
 */

typedef enum _WL_HTIMER
{
    WL_HTIMER_T1 = 0,
    WL_HTIMER_T2,
    WL_HTIMER_T3,
    WL_HTIMER_T4
}
WL_HTIMER;


/*
 *  The following routines are sufficient in most cases
 */

/*
 *  Timer interrupt duration is controled by tick length. On start, Timer load the length. On each clock
 *  tick, the length is decrement by 1. On length reaching 0, an interrupt is generated.
 */
extern BOOLEAN msi_wl_TimerStart(UINT32 timer, BOOLEAN Periodic, BOOLEAN IRQEnable, UINT32 length);
extern BOOLEAN msi_wl_TimerStop(UINT32 timer);

/*
 *  The following routines provide more register controls
 */
extern BOOLEAN msi_wl_TimerSetLength(UINT32 timer, UINT32 length);
extern UINT32 msi_wl_TimerGetLength(UINT32 timer);

/* return current length value */
extern UINT32 msi_wl_TimerGetValue(UINT32 timer);

/* set timer prescaler, default on chip reset is 0 (no scaler) */
extern BOOLEAN msi_wl_TimerSet1KPrescaler(UINT32 timer, UINT32 value);
extern BOOLEAN msi_wl_TimerSet1MPrescaler(UINT32 timer, UINT32 value);

/*
 *  only bit [3:0] are used, 
 *  bit 0 for timer1, bit 1 for timer 2, bit 2 for timer 3, bit 3 for timer 4
 */
extern UINT32 msi_wl_TimerGetISRStatus(void);

/*
 *  only bit [3:0] are used, 
 *  bit 0 for timer1, bit 1 for timer 2, bit 2 for timer 3, bit 3 for timer 4
 */
extern BOOLEAN msi_wl_TimerSetISRMask(UINT32 mask);

/* Bit definitions for control regigster */
#define TIMER1_PERIODIC       (1<<0)
#define TIMER1_ENABLE         (1<<1)
#define TIMER1_INTR_ENABLE    (1<<2)
#define TIMER1_RELOAD         (1<<3)
#define TIMER2_PERIODIC       (1<<4)
#define TIMER2_ENABLE         (1<<5)
#define TIMER2_INTR_ENABLE    (1<<6)
#define TIMER2_RELOAD         (1<<7)
#define TIMER3_PERIODIC       (1<<8)
#define TIMER3_ENABLE         (1<<9)
#define TIMER3_INTR_ENABLE    (1<<10)
#define TIMER3_RELOAD         (1<<11)
#define TIMER4_PERIODIC       (1<<12)
#define TIMER4_ENABLE         (1<<13)
#define TIMER4_INTR_ENABLE    (1<<14)
#define TIMER4_RELOAD         (1<<15)

extern BOOLEAN msi_wl_TimerSetControlReg(UINT32 value);

/*============================================================================= */
/*                    HAL HARDWARE INTERRUPT APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

#define INTR_FIQ             (1<<0)
#define INTR_SWI             (1<<1)
#define INTR_TIMER1          (1<<4)
#define INTR_TIMER2          (1<<5)
#define INTR_TIMER3          (1<<6)
#define INTR_TIMER4          (1<<7)
#define INTR_HIIRQ           (1<<8)
#define INTR_FEIRQ           (1<<9)
#define INTR_WLMACIRQ        (1<<10)
#define INTR_GPIO            (1<<12)
#define INTR_DMA0IRQ         (1<<15)
#define INTR_DMA1IRQ         (1<<16)
#define INTR_AEUIRQ          (1<<17)

extern BOOLEAN msi_wl_EnableInt(UINT32 intMap);
extern BOOLEAN msi_wl_DisableInt(UINT32 intMap);
extern UINT32 msi_wl_GetIntStatus(void);

/*============================================================================= */
/*                    FAST ETHERNET MAC APIS AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */
typedef struct net_dev_stats
{
    UINT32 rx_packets;
    UINT32 tx_packets;
    UINT32 rx_bytes;
    UINT32 tx_bytes;
    UINT32 rx_errors;
    UINT32 rx_dropped;
    UINT32 tx_dropped;
    UINT32 multicast;
    UINT32 collisions;
    UINT32 rx_length_errors;
    UINT32 rx_crc_errors;
    UINT32 tx_fifo_errors;
}
NET_DEV_STATS;

typedef struct user_buf_info
{
    UINT16 frameLen;    /* Frame length      */
    UINT8 frameId;      /* Frame Id (vlanId) */
    UINT8 queue;        /* Tx/Rx queue.      */
    UINT8 *framePtr;    /* Frame Pointer     */
    struct USER_BUF_INFO *nextBufInfo; /* next user buffer  */
}
USER_BUF_INFO;

/* Hash table definitions and for msi_wl_EthInitAddrTbl. */
#define HASH_MODE_FUNC_0            0
#define HASH_MODE_FUNC_1            1
#define HASH_SIZE_8K                0
#define HASH_SIZE_HALF_K            1
#define HASH_DEFAULT_MODE_DISCARD   0
#define HASH_DEFAULT_MODE_PASS      1

extern NET_DEV eth_dev;

BOOLEAN msi_wl_EthInit(NET_DEV *netDev);
BOOLEAN msi_wl_AddEthMACAddr(NET_DEV *netDev, UINT8 *macAddr);
BOOLEAN msi_wl_EthDelMACAddr(NET_DEV *netDev, UINT8 *macAddr);
BOOLEAN msi_wl_EthUpdateStats(NET_DEV *netDev);
BOOLEAN msi_wl_EthGetStats(NET_DEV *netDev, NET_DEV_STATS *netStats);
BOOLEAN msi_wl_EthReceive(NET_DEV *netDev, USER_BUF_INFO *bd);
BOOLEAN msi_wl_EthTransmit(NET_DEV *netDev, USER_BUF_INFO *bd);
BOOLEAN msi_wl_EthWritePHY(NET_DEV *netDev, UINT32 portNumber,
                           UINT32 MIIReg, UINT32 data);
BOOLEAN msi_wl_EthClearMIBCounters(NET_DEV *netDev);
BOOLEAN msi_wl_EthReadPHY(NET_DEV *netDev, UINT32 portNymber,
                          UINT32 MIIReg, UINT32 *value);
BOOLEAN msi_wl_EthEnableMACFiltering(NET_DEV *netDev);
BOOLEAN msi_wl_EthDisableMACFiltering(NET_DEV *netDev);
BOOLEAN msi_wl_EthInitAddrTbl(NET_DEV *netDev, UINT32 hashMode,
                              UINT32 hashSize, UINT32 hashDefaultMode,
                              UINT32 addressTableBase, BOOLEAN filter);
BOOLEAN msi_wl_EthClearAddrTbl(NET_DEV *netDev);
UINT32  msi_wl_Eth_free_tx_queue (NET_DEV *dev , int queue);
BOOLEAN msi_wl_Eth_bdAlloc (NET_DEV *dev);

/*============================================================================= */
/*            Memory Regions API AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */

/* Maximum number of regions. */
#define MAX_NUMBER_MEM_REGIONS 5

/* For creating managed buffer areas. */
typedef struct BUFF_INFO
{
    UINT32           buffMemStart;    
    UINT32           buffElementSize;
    UINT32           numberOfElements;
    void             *buffFreeListPtr;
} BUFF_INFO;

UINT32  msi_wl_CreateMemRgn(UINT32 memRegionStart, UINT32 memRegionSize);
UINT8*  msi_wl_CreateMemSect(UINT32 memRegId, UINT32 size, UINT32 alignment);
UINT32  msi_wl_mem_initFreeBufferList(BUFF_INFO *buffInfo);
UINT8*  msi_wl_mem_allocBuffer(UINT32 bufferId);
BOOLEAN msi_wl_mem_freeBuffer(void *pBuf);


/*============================================================================= */
/*            HAL PCI API AND DATA STRUCTURES (ANSI Prototypes) */
/*============================================================================= */
#define PCI_TXBD_CONTROL_ONWER_MASK  0x80000000
#define PCI_RXBD_CONTROL_ONWER_MASK  0x80
#define PCI_RXBD_CONTROL_LAST_FRAGMT 0x40

typedef struct TxNode_t {
    UINT32 ByteCnt;
    UINT32 FragPhyAddr;
    UINT32 FragVirtualAddr;
    UINT32 Next;
} TxNode_t;

typedef PACK_START struct TxBd_t
{
    UINT32   CtrlStat;
    TxNode_t *FragPtr[10];
    UINT16   PktLen;
    UINT8    DesAddr[6];
    UINT8    Rate;
    UINT8    FrameType;
    UINT8    NotUsed[2];
    struct   TxBd_t *Next;
    UINT32   Reserved[2];
    UINT32   FragCnt;
} PACK_END TxBd_t;

typedef PACK_START struct RxBd_t
{
    UINT8  Control;
    UINT8  RSSI;
    UINT16 wStatus;
    UINT16 PktLen;
    UINT8  SQ2;
    UINT8  Rate;
    UINT32 PktPtr;
	struct RxBd_t *Next;
} PACK_END RxBd_t;

typedef struct PciConfig_t
{
    TxBd_t *TxBdBasePtr;
    RxBd_t *RxBdBasePtr;
} PciConfig_t;

void msi_wl_PciSetConfig(PciConfig_t* pConfig);
BOOLEAN msi_wl_PciInit(UINT8* CbCis);
void msi_wl_PciRegisterHostCmdReceive(void (*CallbackFunc) (void));
void msi_wl_PciRegisterTxPktReceive(void (*CallbackFunc) (void));
void msi_wl_PciIsr(void);
TxBd_t* msi_wl_PciGetTxBdPtr(void);
void msi_wl_PciHandleTxDone(TxBd_t* TxBdDonePtr);
RxBd_t* msi_wl_PciGetRxBdPtr(void);
void msi_wl_PciHandleRxDone(RxBd_t* RxBdDonePtr);
UINT8* msi_wl_PciGetCmdInBuf(void);
UINT8* msi_wl_PciGetCmdOutBuf(void);
void msi_wl_PciHandleCmdDone(void);
void msi_wl_PciSetEvent(UINT32 event);
BOOLEAN hal_SetCpuSpeed(UINT32 cpuFreq);

#define msi_wl_BeaconStop() W81_REGS8_CLRBITS(TX_TIMER_RUN, 0x01)

#endif /* _WL_HAL_H_ */

