#if !defined(_WL_MIB_H_)
#define _WL_MIB_H_

#include "IEEE_types.h"

#ifdef AP_SW
#include "timer.h"
#endif

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    wl_mib.h
 * \brief   This file contains the MIB structure definitions based on IEEE 802.11 specification.
 *
*/


/*============================================================================= */
/*                    Management Information Base STRUCTURES (IEEE 802.11) */
/*============================================================================= */

/*-----------------------------*/
/* Station Configuration Table */
/*-----------------------------*/
typedef struct MIB_StaCfg_s
{
    IEEEtypes_MacAddr_t StationId;
    UINT16 MedOccLimit;    /* 0 to 1000             */
    UINT8 CfPollable;      /* SNMP_Boolean_e values */
    UINT8 CfPeriod;        /* 0 to 255              */
    UINT16 CfpMax;         /* 0 to 65535            */
    UINT32 AuthRspTimeOut; /* 0 to 4294967295       */
    UINT8 PrivOption;      /* SNMP_Boolean_e values */
    UINT8 PwrMgtMode;      /* PwrMgmtMode_e values  */
    UINT8 DesiredSsId[33]; /* 32 byte string. So size should be 33 to accomodate for NULL termination -Rahul        */
    UINT8 DesiredBssType;  /* Bss_e values          */
    UINT8 OpRateSet[126];  /* 126 byte string       */
    UINT16 BcnPeriod;      /* 1 to 65535            */
    UINT8 DtimPeriod;      /* 1 to 255              */
    UINT32 AssocRspTimeOut; /* 1 to 4292967295      */
    UINT16 DisassocReason;  /* 0 to 65535           */
    IEEEtypes_MacAddr_t DisassocSta;
    UINT16 DeauthReason;    /* 0 to 65535           */
    IEEEtypes_MacAddr_t DeauthSta;
    UINT16 AuthFailStatus;  /* 0 to 65535           */
    IEEEtypes_MacAddr_t AuthFailSta;
#if defined (AP_SW) || defined (ECL_WPA)
    UINT8 RSNOptionImplemented; /* SNMP_Boolean_e values */
    UINT8 TKIPNumberOfReplayCounters; // 1 to 4292967295
#endif
#ifdef SELF_CTS
    UINT8 SelfCts;
#endif
}
MIB_STA_CFG;



/*---------------------------------*/
/* Authentication Algorithms Table */
/*---------------------------------*/

typedef struct MIB_AuthAlg_s
{
    UINT32 Idx;
    UINT8 Type;    /* AuthType_e values */
    UINT8 Enable;  /* SNMP_Boolean_e values */
}
MIB_AUTH_ALG;



/*------------------------*/
/* WEP Default Keys Table */
/*------------------------*/

typedef struct MIB_WepDefaultKeys_s
{
    UINT8 WepDefaultKeyIdx;        /* 1 to 4 */
    UINT8 WepDefaultKeyValue[13];   /* 5 byte string */
}
MIB_WEP_DEFAULT_KEYS;


/*------------------------*/
/* WEP Key Mappings Table */
/*------------------------*/

typedef struct MIB_WepKeyMappings_s
{
    UINT32 WepKeyMappingIdx;
    IEEEtypes_MacAddr_t WepKeyMappingAddr;
    UINT8 WepKeyMappingWepOn;   /* SNMP_Boolean_e values */
    UINT8 WepKeyMappingVal[13];  /* 5 byte string */
    UINT8 WepKeyMappingStatus;  /* SNMP_Rowstatus_e values */
}
MIB_WEP_KEY_MAPPINGS;


/*---------------*/
/* Privacy Table */
/*---------------*/

typedef struct MIB_PrivacyTable_s
{
    UINT8 PrivInvoked;       /* SNMP_Boolean_e values */
    UINT8 WepDefaultKeyId;   /* 0 to 3 */
    UINT32 WepKeyMappingLen;  /* 10 to 4294967295 */
    UINT8 ExcludeUnencrypt;  /* SNMP_Boolean_e values */
    UINT32 WepIcvErrCnt;
    UINT32 WepExcludedCnt;
//#if defined (AP_SW) || defined (ECL_WPA) :: TODO :: Will this be commented ???
    UINT8    RSNEnabled;      /* SNMP_Boolean_e values */
//#endif
}
MIB_PRIVACY_TABLE;


/*---------------------------------*/
/* Disassociate Notificaton Object */
/*---------------------------------*/

typedef struct MIB_DisassocNot_s
{
    UINT16 DisassocReason;  /* 0 to 65535 */
    IEEEtypes_MacAddr_t DisassocSta;
}
MIB_DISASSOC_NOT;


/*-----------------------------------*/
/* Deauthenticate Notificaton Object */
/*-----------------------------------*/

typedef struct MIB_DeauthNot_s
{
    UINT16 DeauthReason;  /* 0 to 65535 */
    IEEEtypes_MacAddr_t DeauthSta;
}
MIB_DEAUTH_NOT;



/*--------------------------------------*/
/* Authenticate Fail Notificaton Object */
/*--------------------------------------*/

typedef struct MIB_AuthFailNot_s
{
    UINT16 AuthFailStatus;  /* 0 to 65535 */
    IEEEtypes_MacAddr_t AuthFailSta;
}
MIB_AUTH_FAIL_NOT;


/*============================================================================= */
/*                             MAC ATTRIBUTES */
/*============================================================================= */


/*---------------------*/
/* MAC Operation Table */
/*---------------------*/

typedef struct MIB_OpData_s
{
    IEEEtypes_MacAddr_t StaMacAddr;
    UINT16 RtsThresh;      /* 0 to 2347 */
    UINT8 ShortRetryLim;  /* 1 to 255 */
    UINT8 LongRetryLim;   /* 1 to 255 */
    UINT16 FragThresh;     /* 256 to 2346 */
    UINT32 MaxTxMsduLife;  /* 1 to 4294967295 */
    UINT32 MaxRxLife;      /* 1 to 4294967295 */
    UINT8 ManufId[128];   /* 128 byte string */
    UINT8 ProdId[128];    /* 128 byte string */
}
MIB_OP_DATA;


/*----------------*/
/* Counters Table */
/*----------------*/

typedef struct MIB_Counters_s
{
    UINT32 RxFrmCnt;
    UINT32 MulticastTxFrmCnt;
    UINT32 FailedCnt;
    UINT32 RetryCnt;
    UINT32 MultRetryCnt;
    UINT32 FrmDupCnt;
    UINT32 RtsSuccessCnt;
    UINT32 RtsFailCnt;
    UINT32 AckFailCnt;
    UINT32 RxFragCnt;
    UINT32 MulticastRxFrmCnt;
    UINT32 FcsErrCnt;
    UINT32 TxFrmCnt;
    UINT32 WepUndecryptCnt;
}
MIB_COUNTERS;


/*-----------------------*/
/* Group Addresses Table */
/*-----------------------*/

typedef struct MIB_GroupAddr_s
{
    UINT32 GroupAddrIdx;
    IEEEtypes_MacAddr_t Addr;
    UINT8 GroupAddrStatus;  /* SNMP_Rowstatus_e values */
}
MIB_GROUP_ADDR;


/*----------------------------*/
/* Resource Information Table */
/*----------------------------*/

typedef struct MIB_RsrcInfo_s
{
    UINT8 ManufOui[3];         /* 3 byte string */
    UINT8 ManufName[128];      /* 128 byte string */
    UINT8 ManufProdName[128];  /* 128 byte string */
    UINT8 ManufProdVer[128];   /* 128 byte string */
}
MIB_RESOURCE_INFO;

/*============================================================================= */
/*                             PHY ATTRIBUTES */
/*============================================================================= */


/*---------------------*/
/* PHY Operation Table */
/*---------------------*/
typedef struct MIB_PhyOpTable_s
{
    UINT8 PhyType;        /* SNMP_PhyType_e values */
    UINT32 CurrRegDomain;
    UINT8 TempType;       /* SNMP_TempType_e values */
}
MIB_PHY_OP_TABLE;


/*-------------------*/
/* PHY Antenna Table */
/*-------------------*/

typedef struct MIB_PhyAntTable_s
{
    UINT8 CurrTxAnt;   /* 1 to 255 */
    UINT8 DivSupport;  /* SNMP_DivSupp_e values */
    UINT8 CurrRxAnt;   /* 1 to 255 */
}
MIB_PHY_ANT_TABLE;

typedef struct MIB_PhyAntSelect_s
{
    UINT8 SelectRxAnt;   /* 0 to 1 */
    UINT8 SelectTxAnt;   /* 0 to 1 */
    UINT8 DiversityRxAnt;  /* Boolean */
    UINT8 DiversityTxAnt;  /* Boolean */
}
MIB_PHY_ANT_SELECT;


/*--------------------------*/
/* PHY Transmit Power Table */
/*--------------------------*/
typedef struct MIB_PhyTxPwrTable_s
{
    UINT8 NumSuppPwrLevels;  /* 1 to 8 */
    UINT16 TxPwrLevel1;       /* 0 to 10000 */
    UINT16 TxPwrLevel2;       /* 0 to 10000 */
    UINT16 TxPwrLevel3;       /* 0 to 10000 */
    UINT16 TxPwrLevel4;       /* 0 to 10000 */
    UINT16 TxPwrLevel5;       /* 0 to 10000 */
    UINT16 TxPwrLevel6;       /* 0 to 10000 */
    UINT16 TxPwrLevel7;       /* 0 to 10000 */
    UINT16 TxPwrLevel8;       /* 0 to 10000 */
    UINT8 CurrTxPwrLevel;    /* 1 to 8 */
}
MIB_PHY_TX_POWER_TABLE;


/*---------------------------------------------*/
/* PHY Frequency Hopping Spread Spectrum Table */
/*---------------------------------------------*/
typedef struct MIB_PhyFHSSTable_s
{
    UINT8 HopTime;        /* 224? */
    UINT8 CurrChanNum;    /* 0 to 99 */
    UINT16 MaxDwellTime;   /* 0 to 65535 */
    UINT16 CurrDwellTime;  /* 0 to 65535 */
    UINT16 CurrSet;        /* 0 to 255 */
    UINT16 CurrPattern;    /* 0 to 255 */
    UINT16 CurrIdx;        /* 0 to 255 */
}
MIB_PHY_FHSS_TABLE;


/*-------------------------------------------*/
/* PHY Direct Sequence Spread Spectrum Table */
/*-------------------------------------------*/
typedef enum MIB_CCAMode_s{ ENERGY_DETECT_ONLY = 1, CARRIER_SENSE_ONLY = 2, CARRIER_SENSE_AND_ENERGY_DETECT = 4} MIB_CCA_MODE;

typedef struct MIB_PhyDSSSTable_s
{
    UINT8 CurrChan;     /* 0 to 14 */
    UINT8 CcaModeSupp;  /* 1 to 7 */
    UINT16 CurrCcaMode; /* MIB_CCA_MODE values only */
    UINT32 EdThresh;
}
MIB_PHY_DSSS_TABLE;


/*--------------*/
/* PHY IR Table */
/*--------------*/
typedef struct MIB_PhyIRTable_s
{
    UINT32 CcaWatchDogTmrMax;
    UINT32 CcaWatchDogCntMax;
    UINT32 CcaWatchDogTmrMin;
    UINT32 CcaWatchDogCntMin;
}
MIB_PHY_IR_TABLE;


/*----------------------------------------*/
/* PHY Regulatory Domains Supported Table */
/*----------------------------------------*/
typedef struct MIB_PhyRegDomainsSupp_s
{
    UINT32 RegDomainsSuppIdx;
    UINT8 RegDomainsSuppVal;  /*SNMP_RegDomainsSuppVal_e values */
}
MIB_PHY_REG_DOMAINS_SUPPPORTED;


/*-------------------------*/
/* PHY Antennas List Table */
/*-------------------------*/
typedef struct MIB_PhyAntList_s
{
    UINT8 AntListIdx;
    UINT8 SuppTxAnt;   /*SNMP_Boolean_e values */
    UINT8 SuppRxAnt;   /*SNMP_Boolean_e values */
    UINT8 RxDiv;       /*SNMP_Boolean_e values */
}
MIB_PHY_ANT_LIST;


/*-----------------------------------------*/
/* PHY Supported Transmit Data Rates Table */
/*-----------------------------------------*/
typedef struct MIB_PhySuppDataRatesTx_s
{
    UINT8 SuppDataRatesTxIdx;  /*1 to IEEEtypes_MAX_DATA_RATES_G */
    UINT8 SuppDataRatesTxVal;  /*2 to 127 */
}
MIB_PHY_SUPP_DATA_RATES_TX;


/*----------------------------------------*/
/* PHY Supported Receive Data Rates Table */
/*----------------------------------------*/
typedef struct MIB_PhySuppDataRatesRx_s
{
    UINT8 SuppDataRatesRxIdx;  /*1 to 8 */
    UINT8 SuppDataRatesRxVal;  /*2 to 127 */
}
MIB_PHY_SUPP_DATA_RATES_RX;


typedef struct MIB_DHCP_s
{
    UINT32 IPAddr;
    UINT32 SubnetMask;
    UINT32 GwyAddr;
}
MIB_DHCP;

/* Added for WB31 */
typedef struct _MIB_WB
{
    UINT8    devName[16];        // Must be a string: 15 Max characters  
    UINT8    cloneMacAddr[6];    // cloned MAC Address                   
    UINT8    opMode;             // 0 for infrastructure, 1 for ad-hoc
    UINT8    macCloneEnable;     // boolean
}MIB_WB;
/* Added for WB31 end */

#if defined (AP_SW) || defined (ECL_WPA)

/*---------------------*/
/* RSN Config Table */
/*---------------------*/
typedef struct MIB_RSNConfig_s
{
    UINT32  Index;
    UINT32  Version;
    UINT32  PairwiseKeysSupported;
    UINT8   MulticastCipher[4];
    UINT8   GroupRekeyMethod;
    UINT32  GroupRekeyTime;
    UINT32  GroupRekeyPackets;
    UINT8   GroupRekeyStrict;
    UINT8   PSKValue[40];
    UINT8   PSKPassPhrase[64];
    UINT8   TSNEnabled;
    UINT32  GroupMasterRekeyTime;
    UINT32  GroupUpdateTimeOut;
    UINT32  GroupUpdateCount;
    UINT32  PairwiseUpdateTimeOut;
    UINT32  PairwiseUpdateCount;
}
MIB_RSNCONFIG;

/*---------------------*/
/* RSN Unicast Cipher Suites Config Table */
/*---------------------*/
typedef struct MIB_RSNConfigUnicastCiphers_s
{
    UINT32  Index;
    UINT8   UnicastCipher[4];
    UINT8   Enabled;
}
MIB_RSNCONFIG_UNICAST_CIPHERS;

/*---------------------*/
/* RSN Authentication Suites Config Table */
/*---------------------*/
typedef struct MIB_RSNConfigAuthSuites_s
{
    UINT32  Index;
    UINT8   AuthSuites[4];
    UINT8   Enabled;
}
MIB_RSNCONFIG_AUTH_SUITES;

/*---------------------*/
/* RSN Statistics Table */
/*---------------------*/
typedef struct MIB_RSNStats_s
{
#if !defined(ECL_WPA)    
    UINT32  Index;
#endif	
    /*IEEEtypes_MacAddr_t MacAddr; */
    UINT32  Version;
    UINT8   SelectedUnicastCipher[4];
    UINT32  TKIPICVErrors;
    UINT32  TKIPLocalMICFailures;
    UINT32  TKIPRemoteMICFailures;
    UINT32  TKIPCounterMeasuresInvoked;
    UINT32  CCMPFormatErrors;
    UINT32  CCMPReplays;
    UINT32  CCMPDecryptErrors;  
#if defined (ECL_WPA)  
    UINT32  WrapFormatErrors;
    UINT32  WrapReplays;
    UINT32  WrapDecryptErrors;
#endif    
}
MIB_RSNSTATS;

typedef struct Mrvl_MIB_RSN_GrpKey_s
{
    UINT8  GrpMasterKey[32];
    UINT8  EncryptKey[16];
    UINT32  TxMICKey[2];
    UINT32  RxMICKey[2];
    UINT32 g_IV32;
    UINT16  g_IV16;
    UINT16 g_Phase1Key[5];
    UINT8 g_KeyIndex;
#if !defined (ECL_WPA)
    Timer timer;
#endif    
}
MRVL_MIB_RSN_GRP_KEY;

typedef struct MIB_BURST_MODE
{
    UINT8  mib_burstmode;
    UINT32 mib_burstrate;
}
MIB_BURST_MODE;
#endif

typedef struct MIB_802DOT11_s
{
    /*-----------------------------------------*/
    /* Station Management Attributes */
    /*-----------------------------------------*/
    MIB_STA_CFG StationConfig;      /* station configuration table */
    MIB_AUTH_ALG AuthAlg;            /* authentication algorithms table */
    MIB_WEP_DEFAULT_KEYS WepDefaultKeys[4];  /* wep default keys table */
    MIB_WEP_KEY_MAPPINGS WepKeyMappings;     /* wep key mappings table */
    MIB_PRIVACY_TABLE Privacy;            /* privacy table */

    /* SMT Notification Objects */
#ifdef AP_SW
    MIB_DISASSOC_NOT NoteDisassoc;       /* disassociate notification */
    MIB_DEAUTH_NOT NoteDeauth;         /* deauthentication notification */
    MIB_AUTH_FAIL_NOT NoteAuthFail;       /* authentication fail notification */
#endif

    /*-----------------------------------------*/
    /* MAC Attributes */
    /*-----------------------------------------*/
    MIB_OP_DATA OperationTable;
#ifdef AP_SW
    MIB_COUNTERS CountersTable;
    MIB_GROUP_ADDR GroupAddrTable;

    /*-----------------------------------------*/
    /* Resource Type                           */
    /*-----------------------------------------*/
    MIB_RESOURCE_INFO ResourceInfo;

    /*-----------------------------------------*/
    /* PHY Attributes                          */
    /*-----------------------------------------*/
    MIB_PHY_OP_TABLE PhyOpTable;
    MIB_PHY_TX_POWER_TABLE PhyPowerTable;
    MIB_PHY_FHSS_TABLE PhyFHSSTable;
    MIB_PHY_IR_TABLE PhyIRTable;
    MIB_PHY_REG_DOMAINS_SUPPPORTED PhyRegDomainsSupp;
    MIB_PHY_ANT_LIST AntennasListTable;
#endif
    MIB_PHY_ANT_TABLE PhyAntTable;
    MIB_PHY_DSSS_TABLE PhyDSSSTable;
    MIB_PHY_SUPP_DATA_RATES_TX SuppDataRatesTx[IEEEtypes_MAX_DATA_RATES_G];
    MIB_PHY_SUPP_DATA_RATES_RX SuppDataRatesRx;
#if defined (AP_SW) || defined (ECL_WPA)    
    MIB_RSNCONFIG RSNConfig;
    MIB_RSNCONFIG_UNICAST_CIPHERS UnicastCiphers;
    MIB_RSNCONFIG_AUTH_SUITES  RSNConfigAuthSuites;
    MIB_RSNSTATS RSNStats;
#ifdef BURST_MODE
    MIB_BURST_MODE BurstMode;
#endif
#endif
    MIB_PHY_ANT_SELECT PhyAntSelect;
}
MIB_802DOT11;

extern BOOLEAN mib_InitSta(MIB_802DOT11 * mib);
extern BOOLEAN mib_InitAp(MIB_802DOT11 * mib);

#endif /* _WL_MIB_H_ */

